#!/bin/bash
#
# check the MySQL database for values
# produce statistical figures of measurements
# build a new database table with average, std diviation, max and min 
# values per day
# filter out unusual values
#

# $Id: Check_DB.sh,v 6.29 2020/05/11 19:18:35 teus Exp teus $

#
# Copyright (C) 2014, Teus Hagen, the Netherlands
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# TO DO:
#	have mysql using named pipe or --socket=/path/my_socket
#	use better algorithm to detect invalid measurements and averages
#	or automate validation of measurements
#	diagrams for other sensors as the PM sensors
#	support normalisation factoring for RIVM originating values
#	delete measurements with YYYYMMDD HH:19 from RIVM database(s)

############# GLOBALS and DEFAULT values:
# for the next some arguments can be set in script environment
# as WDIR, DBUSER, DBPASS, WIDTH, HEIGHT, CSV_REPORT
declare NR			# station number
declare LOCATION		# station name
declare ORGANISATION		# station organisation
declare ASSOCIATION		# association making the graphs
declare ID			# station identifier
declare TBL			# database table
declare RRD			# RRD database for the graphs
# StdDev 6 X			# validation off
declare N_StdDev		# default X sigma per full operational period
declare N_DayStdDev		# default X sigma per day

declare NO_POL='(_[a-z]+|[A-Z]+|id|datum)' # names not pollutants in DB tables
declare AQIs='LKI AQI AQHI'     # air quality indexes in DB tables

# default definitions
    DB=luchtmetingen	# database name
    SITE=${SITE:-luchtkwaliteit.limburg.nl}
    SITE2=${SITE2:-luchtmeetnet-limburg.nl}

    VALID_CNT=20	# default: minimal amount of measurements per day

    AUTHOR="mr. Unknown"
    if which finger >/dev/null
    then
        AUTHOR=${AUTHOR:=$(finger $USER | grep Name | sed 's/.*Name: //')}
    fi
    ASSOCIATION=", Behoud de Parel, Grubbenvorst"

    # RRD database location
    RRDEXT="_normalized"
    WIDTH=${WIDTH:=785}
    HEIGHT=${HEIGHT:=120}
    if [ "$WIDTH" -le 500 ]
    then
        LOCATION=`echo "$LOCATION" | sed -e 's/\([A-Za-z]\)[a-z0-9][a-z0-9]*/\1/g' -e 's/ //g'`
    fi
    CSV_REPORT=${CSV_REPORT}	# ./report.csv the report in a spreadsheet

    WDIR=${WDIR:-/webdata/luchtmetingen/}
    RRD_DIR=${RRD_DIR:-$WDIR/rrd_data}
    RRD_DIR=${RRD_DIR//\/.\//\/}
    if [ -d $WDIR ]
    then
        cd $WDIR
    fi
    # all diagrams/graphs will be placed here
    DIAGRAMS=$WDIR./diagrams
    DIAGRAMS=${DIAGRAMS//\/.\//\/}
############## end of default valueswhich can be changed on the fly

# the perl script to collect data from the websites: prov limburg or RIVM
# this script will hold info as well
unset LC_TIME	# make sure date string can be parsede by date command
export LC_TIME=en_US.utf-8
GET_DATA=$WDIR./Get_data.pl
MY_NAME=$0
LOCATIONS=$WDIR./locations.pl
WARNING=WARNING
ATTENT=ATTENT
ERROR=ERROR
DEBUG=${DEBUG}  # do some extra debugging print outs
NO_CAL=0	# normalized/calibrated table is switched off?

export DBUSER=${DBUSER}	# database user
export DBPASS=${DBPASS}	# password database user
export DB=${DB:-luchtmetingen}		# database to use
HOST=${HOST:-localhost}		# database host
export DBHOST=${DBHOST:-$HOST}
MYSQL="mysql -u $DBUSER -p$DBPASS -N -B --silent -h $DBHOST $DB"

# default NORM to use
DFLT_NORM=${NORM:-EU2010}
unset NORM

function depends() {
    if echo "$1" | grep -q "[a-z]-perl"
    then
        dpkg-query -l "$1" 2>&1 >/dev/null
    else
        which "$1" >/dev/null
    fi
    if (( $? != 0 ))
    then
	local PACK="${2:+ from package $2}"
	echo "$ERROR Need command/package \"$1\"${PACK}." 1>&2
	if [ -n "$3" ] ; then exit 1; fi
	return 1
    fi
    return $?
}

depends perl "" 1
depends libstatistics-distributions-perl "" 1
depends libgetopt-mixed-perl "" 1
depends libspreadsheet-parseexcel-perl "" 1
depends libspreadsheet-xlsx-perl "" 1
depends awk "" 1
depends sed "" 1

# get from central definition file the other details
# args: {location|name|table} search_string {location|name|table}
# return empty if not found
# e.g.: Get_info name 'Horst aan de Maas' location
# make sure you run this after DB_pass call when used in subshell
function Get_info() {
    if create_stations
    then
	$MYSQL -e "SELECT stations.$3 FROM stations WHERE stations.$1 = '$2' LIMIT 1"
	return $?
    fi
    return 0
}

# print details of the available locations
# argument: location number, default: all available locations
function Print_stations() {
    local NR=($@) F STATION VALUE
    declare -A FLD
    declare -i CNT=0
    DB_pass
    if ! $MYSQL -e "show tables" | grep -q stations
    then
        echo "$ERROR Cannot find table stations in database $DB on $DBHOST!" 1>&2
        return 1
    fi
    for F in $($MYSQL -e "describe stations" | awk '{ print $1; }')
    do
        CNT+=1
        FLD[$F]=$CNT
    done
    if (( ${#NR[@]} < 1 ))
    then
        NR=($($MYSQL -e "SELECT nr FROM stations"))
    fi
    echo -e '\nDetails meetstation(s):\n'
    for F in ${NR[@]}
    do
        STATION=$($MYSQL -e "SELECT * FROM stations WHERE NR = '$F'" \
                  | sed 's/NULL/?/g')
        if [ -z "${STATION}" ]
        then
            echo "$ERROR: cannot find info for station $F in database $DB on $DBHOST" 1>&2
            continue
        fi
        echo -e -n "\nLokatie:\t\t\t${F}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[class]})
        if [ "$VALUE" != "?" ] ; then echo -e ", Class: $VALUE" ; fi ; echo
        VALUE=$(echo "$STATION" | cut -f ${FLD[name]})
                echo -e -n "\tPlaats/lokatie:\t\t${VALUE}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[municipality]})
                echo -e ", Gemeente: ${VALUE}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[organisation]})
                echo -e "\tOrganisatie:\t\t${VALUE}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[geolocation]})
                echo -e -n "\tGPS geolokatie:\t\t${VALUE}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[altitude]})
        if [ "$VALUE" != "?" ] ; then echo -e -n ", hoogte: ${VALUE} m" ; fi ; echo
        VALUE=$(echo "$STATION" | cut -f ${FLD[id]})
                echo -e "\tEuregio id:\t\t${VALUE}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[table]})
                echo -e "\tDatabase:\t\t$DB, tabel ${VALUE}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[first]})
                echo -e "\tOperationeel sinds:\t${VALUE}"
        VALUE=$(echo "$STATION" | cut -f ${FLD[last]})
        if [ "$VALUE" != "?" ] ; then echo -e "\tEinddatum op:\t\t${VALUE}" ; fi
        VALUE=$(echo "$STATION" | cut -f ${FLD[sense]})
                echo -e "\tType sensors: $VALUE"
        VALUE=$(echo "$STATION" | cut -f ${FLD[norm]})
        if [ "$VALUE" != "?" ]
        then
            echo "$VALUE" \
            | sed -e 's/pm_/PM/g' -e 's/PM25/PM2.5/g' \
                -e 's/:/: /g' -e 's/(/=/g' -e 's/)/ /g'  -e 's/,/ /g'\
            | awk -F ';' '{
                for( I=NF; I > 0; I-- ) {
                    printf( "\tCalibratie factoren voor %s\n", $I); }
                }'
        fi
        VALUE=$(echo "$STATION" | cut -f ${FLD[validated]})
        if echo "$VALUE" | grep -q "0000-00-00"
        then echo -e "\tLast statistical validation on:\t${VALUE}"
        else echo -e "\tNo statistical validation done so far."
        fi
        VALUE=$(echo "$STATION" | cut -f ${FLD[a_stddev]})
        if [ "$VALUE" != "6.00" ] ; then echo -e "\tStatistical filtering stddev avg/hour: $(Propability ${VALUE})" ; fi
        VALUE=$(echo "$STATION" | cut -f ${FLD[day_stddev]})
        if [ "$VALUE" != "6.00" ] ; then echo -e "\tStatistical filtering stddev avg/day: $(Propability ${VALUE})" ; fi
    done
    echo
    return 0
}
# print info from locations.pl
function Print_info() {
    local NR=$1
    local FLD=()
    if $MYSQL -e "show tables" | grep -q stations
    then
        Print_stations $1
        return $?
    fi
    if [ ! -f "$LOCATIONS" ]
    then
        echo "$ERROR Cannot find central file $LOCATIONS for location info!" 1>&2
        return 1
    fi
    perl \
    -e "my \$NR=\"${NR:-06}\"; my \$DB=\"$DB\";" -e "require '$LOCATIONS';" \
    -e '
    use feature "state";
    sub print_location {
        my $loc = shift; my $rts = 1;
        state $once = 0;
        if ( $loc !~ /^[0-9]{2,}$/ ) {
            for my $l (sort { $a cmp $b } keys %locations) {
                print_location($l);
            }
        }
        return 1 if not $locations{$loc};
        $rts = 0 if defined $locations{$loc}{norm};
        print STDOUT "\nDetails meetstation(s):\n" if not $once++;
        print STDOUT "\nLokatie:\t\t\t$loc\n";
        for my $l ("name","organisation","geolocation","id","table","first","last","sense") {
            next if not defined $locations{$loc}{$l};
            next if not $locations{$loc}{$l};
            my $strg;
            if( $l =~ /^name$/ )
                { $strg = "\tPlaats/lokatie:\t\t$locations{$loc}{$l}"; }
            elsif( $l =~ /^organisation$/ )
                { $strg = "\tOrganisatie:\t\t$locations{$loc}{$l}"; }
            elsif( $l =~ /^geolocation$/ )
                { $strg = "\tGPS geolokatie:\t\t$locations{$loc}{$l}"; }
            elsif( $l =~ /^id$/ )
                { $strg = "\tEuregio id:\t\t$locations{$loc}{$l}"; }
            elsif( $l =~ /^table$/ )
                { $strg = "\tDatabase:\t\t$DB, tabel $locations{$loc}{$l}"; }
            elsif( $l =~ /^first$/ )
                { $strg = "\tOperationeel sinds:\t$locations{$loc}{$l}"; }
            elsif( $l =~ /^last$/ )
                { $strg = "\tEinddatum op:\t\t$locations{$loc}{$l}"; }
            elsif( $l =~ /^sense$/ )
                { $strg = "\tType sensors: " . (join ", ", sort { $a cmp $b } keys %{$locations{$loc}{sense}}) if ( (keys %{$locations{$loc}{sense}}) > 0 ); }
           else
                { $strg = ""; }
            print STDOUT "$strg\n" if $strg;
        }
        if ( defined $locations{$loc}{norm} ) {
            for( my $nrm = $locations{$loc}{norm}; defined $locations{$loc}{$nrm}; $nrm--) {
                print STDOUT "\tCalibratie factoren voor $nrm:";
                foreach my $pm ( sort { $a cmp $b } keys %{$locations{$loc}{$nrm}} ) {
                    my $strg = $pm; $strg =~ s/_//; $strg =~ s/25/2.5/;
                    print STDOUT " $strg=$locations{$loc}{$nrm}{$pm}";
                }
                print STDOUT "\n";
            }
        }
        return $rts;
    }

    exit (print_location ( "${NR:-06}" ));
    '
    NO_CAL=$?
    return $NO_CAL
}

# sensor minimal and maximal values
declare -A MAX_sens MIN_sens
# sensors made available are:
# PM 10, PM 2.5, roet, air pressure, temperature, wind speed and direction
# values above MAX are expected to be filtered out in the graphs
  MAX_sens[temp]=38	# above 38 oC is expected to be illegal (6 values)
  MIN_sens[temp]="-30"	# below -40 oC is expected to be illegal
  MAX_sens[luchtdruk]=1500 # above 1350 mBar is expected to be illegal (0 values)
  MIN_sens[luchtdruk]=800  # below 500 mBar is expected to be illegal
  MAX_sens[wr]=360	# above 360 degrees is expected to be illegal
  MIN_sens[wr]=0	# below 0 degrees is expected to be illegal
  MAX_sens[ws]=40	# above 40 m/sec is expected to be illegal
  MIN_sens[ws]=0	# below 0 m/sec is expected to be illegal
  MAX_sens[rv]=100	# above 100% is expected to be illegal
  MIN_sens[rv]=0	# below 0 % is expected to be illegal
  MAX_sens[no]=175	# above 150 µg/m³ is expected to be illegal
  MIN_sens[no]=0	# below 0 µg/m³ is expected to be illegal
  MAX_sens[no2]=150	# above 150 µg/m³ is expected to be illegal
  MIN_sens[no2]=0	# below 0 µg/m³ is expected to be illegal
  MAX_sens[so2]=50	# above 50 µg/m³ is expected to be illegal
  MIN_sens[so2]=0	# below 0 µg/m³ is expected to be illegal
  MAX_sens[o3]=250	# above 250 µg/m³ is expected to be illegal
  MIN_sens[o3]=0	# below 0 µg/m³ is expected to be illegal
  MAX_sens[zwarte_rook]=20 # above 20 µg/m³ is expected to be illegal
  MIN_sens[zwarte_rook]=0  # below 0 µg/m³ is expected to be illegal
  MAX_sens[nh3]=1500	# above 1500 µg/m³ is expected to be illegal
  MIN_sens[nh3]=0	# below 0 µg/m³ is expected to be illegal
  # experimental max values allowing industrial fall out factors
  #MAX_sens[pm_10]=67.9	# 2.5 X stddev: 2.7% failures
  #MAX_sens[pm_25]=52.9	# 2.5 X stddev: 2.7% failures
  # NEN 14907 and 12341 describes max values for PM measurements
  MAX_sens[pm_10]=150	# max PM 10 NEN 12341
  MIN_sens[pm_10]=1
  MAX_sens[pm_25]=120	# max PM 2.5 NEN 14907
  MIN_sens[pm_25]=1
  MAX_sens[roet]=50	# max soot NEN ??? 
  MIN_sens[roet]=0

# Correction (calibration/normalisation) factors for sensor values.
# Defined as CAL[??]=F,N, a linear correction
# where F is multiplying factor, e.g. 1.4
#       N is correction, e.g. -1.6
# CAL[pm_10]=0.98,-1 -> pm_10 value = pm_10 measured * 0.98 + -1
# defaults: 1,0
declare -A CAL CAL_f CAL_n
# the following function will define CAL_f, CAL_n for sensors, deflt 1,0
# from CAL[pm_25] and CAL[pm_10]
function Set_calibration() {
    local CC
    for CC in $*
    do
	if [ -z "${CAL[$CC]}" ] ; then CAL[$CC]=1,0 ; fi
	if [ -z `echo "${CAL[$CC]}" \
            | perl -e '
                while( <> ) {
                    print $_ if  /^[0-9]+\.?[0-9]{0,},-?[0-9]+\.?[0-9]{0,}$/;
                }'` ]
	then
	    echo "$WARNING Correction factor $CC=${CAL[$CC]} has wrong format, using defaults (1,0)." 1>&2
	    CAL[$CC]=1,0
        fi
        CAL_f[$CC]=`echo "${CAL[$CC]}" | sed 's/,.*//'`
        CAL_n[$CC]=`echo "${CAL[$CC]}" | sed 's/.*,//'`
    done
}
# and set default sensor PM calibration factors
Set_calibration pm_10 pm_25

# we show progress when command seems to take some long time.
pid=""                          # need pid of forked process progress bar
timing=""                       # keeps start time
function progressing {
  # Sleep at least 1 second otherwise this algoritm will consume
  # a lot system resources.
  set +x
  local interval=1 message="Progressing  " count=0 chars=( "-" "\\" "|" "/" ) pos=0

  if [ -n "$1" ] ; then message="$1  " ; fi
  printf "%-60s" "$message...  " 1>&2
  while true
  do
    pos=$(($count % 4))
    echo -en "\b${chars[$pos]}" 1>&2
    count=$(($count + 1))
    sleep $interval
  done
  return 0
}

#
# Stop distraction
#
timing=""
function stop_progressing {
    # exec 2>/dev/null
    if [ -n "$pid" ] ; then kill $pid ; sleep 1; fi
    pid=""
    if [ -n "$timing" -a -t 2 ]
    then
        timing=$((`date "+%s"` - $timing))
        if [ $(($timing / 60)) -gt 0 ] ; then
            echo -en "\b \t$(($timing / 60)) min $(($timing % 60)) sec\n" 1>&2
        else
            echo -en "\b \t$(($timing % 60)) sec\n" 1>&2
        fi
    fi
    timing=""
    return 0
}

function start_progressing(){
    if [ -n "$pid" ]; then kill $pid; fi
    timing=`date "+%s"`
    if [ -t 2 ] && [ -z "$DEBUG" ]
    then
        progressing  "$1"  &
        pid=$!
    fi
    return 0
}

function GET_passwd() {
    local RTS=unknown
    if [ -n "$1" ] && [ $1 = USER ]
    then
        if [ -z "$DBUSER" ] && [ {PASSED:-0} = HIDDEN ]
        then
            export DBUSER=$(mysql_config_editor print --login-path=$DB \
                | awk '/user =/{ print $3 ; }')
        fi
        if [ -z "${DBHOST/localhost/}" ] && [ {PASSED:-0} = HIDDEN ]
        then
            export DBHOST=$(mysql_config_editor print --login-path=$DB \
                | awk '/host =/{ print $3 ; }')
        fi
        if [ -z "$DBUSER" ]
        then
            if [ -t 0 -a -e /usr/bin/zenity ] && xset q 2>/dev/null >/dev/null
            then
                RTS=$(zenity --title "user name for $DB access" --entry --text "Select user:")
                echo $RTS
            else
                echo $DBUSER
            fi
        fi
    else
        if [ -z "$DBPASS" ] || [ -z "${PASSED/HIDDEN/}" ]
        then
            if [ -t 0 -a -e /usr/bin/zenity ] && xset q 2>/dev/null >/dev/null
            then
                RTS=$(zenity --password --title "${DBUSER:-UNKNOWN} password for $DB database access")
                echo $RTS
            else
                echo $DBPASS
            fi
        fi
    fi
    if [ -z "${RTS/unknown/}" ]
    then
        echo "$ERROR Need $DB database password or db user name (via env: DBUSER) for user ${DBUSER:-UNKNOWN}" 1>&2
        exit 1
    fi
}

PASSED=""
function DB_pass(){
    local TBLS
    if [ -n "$TBL" ]
    then
       if [ -n "$PASSED" ] && echo "$PASSED" | grep -q "^${TBL}$" ; then return; fi
    else
       if [ -n "$PASSED"  -a -z "$TBL" ] ; then return ; fi
    fi
    if [ -z "$PASSED" ] && which mysql_config_editor >/dev/null
    then
        PASSED=$(mysql_config_editor print --login-path=$DB \
            | awk '/password =/{ print "HIDDEN" ; }')
        MYSQL="mysql --login-path=$DB -N -B --silent -h $DBHOST $DB"
    fi
    if [ -z "$DBUSER"  ] ; then DBUSER=$(GET_passwd USER) ; fi
    if [ -z "$DBPASS" ] ; then DBPASS=$(GET_passwd) ; fi
    if [ -n "$DBPASS" ] && [ ${PASSED:-0} != HIDDEN ]
    then
        MYSQL="mysql -u $DBUSER -p$DBPASS -N -B --silent -h $DBHOST $DB"
    elif [ -z "$DBPASS" ]
    then
        echo "$ERROR Need DB password!" 1>&2
        exit 1
    fi
    # integrity database test
    # check the databases and available tables
    depends mysql "MySQL server"  1
    TBLS=$($MYSQL -e 'SHOW TABLES')
    if [ -z "$PASSED" ] ; then PASSED=0 ; fi
    if [ -z "$TBLS" ]
    then
        echo "$ERROR The database $DB does not exists" 1>&2
        echo "$ERROR or the user $DBUSER@$DBHOST has no access permissions." 1>&2
        exit 1
    fi
    if [ -z "$TBL" ] ; then return; fi
    if ! echo "$TBLS" | grep -q "^$TBL\$" 
    then
        echo "$ERROR Table $TBL does not exists in database $DB" 1>&2
        echo "$ERROR Build one first!" 1>&2
	exit 1
    fi
    PASSED=$(echo "$TBLS" | grep "$TBL")
    # build the dayly average table if needed
    if ! echo "$TBLS" | grep -q "^${TBL}_dayly\$" 
    then
        Build_dayly ${TBL}
    fi
    if [ -n "${NR}" ] && [ "${NR/#0/}" -ge 100 ]
    then	# we have RIVM values no normalisation is done
	return 0
    fi
    if ! echo "$TBLS" | grep -q "^${TBL}_dayly_norm\$"
    then
	if [ "$NO_CAL" = 0 ]
	then
	    if Get_info table $TBL norm | grep -q pm_ && \
		! echo "$TBL" | grep -q "^NL1"
	    then
                Build_dayly ${TBL} normalized
                if ! $MYSQL -e 'SHOW TABLES' \
                    | grep -q ${TBL}_dayly_norm
	        then	# no calibration factors defined switch this table off
	            NO_CAL=1
	        fi
	    fi
	fi
    fi
}

# create if needed the DB table stations providing info of stations
Created_station=""
function create_stations() {
    if [ -n "$Created_station" -a -z "$1" ] ; then return $Created_station; fi
    DB_pass
    if [ -n "$1" ] || ! $MYSQL -e "SHOW TABLES;" | grep -q "stations"
    then
        if [ ! -f "$LOCATIONS" ]
        then
	    echo "$ERROR Cannot find $LOCATION stations definitions file." 1>&2
	    return 1
        fi
	$MYSQL -e "
                DROP TABLE IF EXISTS stations;
	        CREATE TABLE stations (
		  \`name\` VARCHAR(40) DEFAULT NULL COMMENT 'town - street station',
		  \`organisation\` VARCHAR(40) DEFAULT NULL COMMENT 'organisation name',
		  # number id taken from http://pmlab.irceline.be/index_nl.html
		  \`id\` VARCHAR(8) DEFAULT NULL COMMENT 'location number of Euregio',
		  \`table\`   VARCHAR(10) DEFAULT NULL COMMENT 'table name',
		  \`first\` CHAR(10) DEFAULT NULL COMMENT 'first date operational',
		  \`last\` CHAR(10) DEFAULT NULL COMMENT 'last date of operations',
		  \`nr\` VARCHAR(4) DEFAULT NULL COMMENT 'internal station number',
		  \`sense\` VARCHAR(150) DEFAULT NULL COMMENT 'list of sensors',
		  \`norm\` TEXT(200) DEFAULT NULL COMMENT 'calibration factors',
		  \`validated\` TIMESTAMP DEFAULT '2001-01-01 00:00:00' COMMENT 'last time ran validation',
		  \`a_stddev\` FLOAT(4,2) UNSIGNED DEFAULT 6 NOT NULL COMMENT 'N X sigma measurements validation',
		  \`day_stddev\` FLOAT(4,2) UNSIGNED DEFAULT 6 NOT NULL COMMENT 'N X sigma day measurements validation'
		  \`geolocation\` VARCHAR(30) DEFAULT NULL COMMENT 'geo location latitudeN longitudeE'
                  \'altitude\' SMALLINT(6) DEFAULT NULL COMMENT 'altitude in meters',
                  \'class\' VARCHAR(15) DEFAULT NULL COMMENT 'type of environment',
                  \'municipality\' VARCHAR(25) DEFAULT NULL COMMENT 'local authority'
                  \'aqi_url\' VARCHAR(128) DEFAULT NULL COMMENT 'URL with updated AQI values from e.g. AQICN.org'
	        ) ENGINE=InnoDB CHARSET=latin1 COMMENT='table of stations and their tables and global variables';
	"
    else
	Created_station=0
	return 0
    fi
    perl -e "require '$LOCATIONS';" \
         -e '
	# make sure next items have same order as with table create
	my @id = ( "name","organisation","id","table","first","last","nr","sense","norm","validated","a_stddev","day_stddev","geolocation");
	print "INSERT INTO `stations` VALUES \n"; my $st = 0;
	foreach my $nr (sort { $a <=> $b } keys %locations) {
	  my @val;
	  for( my $i = 0; $i <= $#id; $i++) {
	    $_ = $id[$i];
	    if ( /^nr$/ ) { $val[$i] = "'"'"'".$nr."'"'"'"; next; }
	    $val[$i] = "NULL"; next if not defined $locations{$nr}{$_};
	    if ( /^sense$/ ) { # sensors for this station
	        my $ss = "";
	        foreach my $s (sort { $a cmp $b } keys %{$locations{$nr}{sense}}) {
		    $ss .= ",".$s if $s;
		}
	        $ss =~ s/^,//;
		$val[$i] = "'"'"'".$ss."'"'"'" if $ss;
	    }
	    elsif ( /^norm$/ ) {
	        my $pms = "";
	        foreach my $y (sort { $a <=> $b } keys %{$locations{$nr}}) {
		    next if $y !~ /^[0-9]{4}$/;
	            $pms .= ";$y:"; my $pvs = "";
		    foreach my $pv (sort { $a cmp $b } keys %{$locations{$nr}{$y}} ) {
		        $pvs .= ",$pv($locations{$nr}{$y}{$pv})";
		    }
		    $pvs =~ s/^,//; $pms .= $pvs;
	        }
	        $pms =~ s/^;//;
		$val[$i] = "'"'"'".$pms."'"'"'";
	    }
	    elsif( /^(validated|day_stddev|a_stddev|geolocation)$/ ) { next; }
	    else { $val[$i] = "'"'"'".$locations{$nr}{$_}."'"'"'"; }
	  }
	  # sensor calibration factors YYYY:PM(val),...;YYYY:PM(...)
	  
	  for( my $i = 0; $i <= $#id; $i++ ) {
	    printf("%s$val[$i]",($i ? "," : ( $st ? ",(" :" ("))); $st++;
	  }
	  print ")\n";
	  
        }
	print ";\n";
    ' \
       | $MYSQL
    Created_station=$?
    return $Created_station
}

# put new std dev definition in DB and schedule revalidation
function Set_StdDev() {
    local TTBL=$1
    if [ -z "$2" -o -z "$1" ] ; then return 1; fi
    local VAL=${2/*=/} DEV=${2/=*/}
    if [ -z "$DEV" -o -z "$VAL" ] ; then return 1 ; fi
    VAL=$(echo "$VAL" | sed 's/[^0-9\.]//g')
    case "$DEV" in
    N_DayStdDev)
	DEV=day_stddev
    ;;
    N_StdDev)
	DEV=a_stddev
    ;;
    a_stddev|day_stddev)
	DEV=${DEV}
    ;;
    *)
	return 1
    ;;
    esac
    set_last_validated ${TTBL} "" "2000/1/1"
    $MYSQL -e "UPDATE stations SET $DEV = $VAL WHERE stations.table = '$TTBL';"
    return $?
}

# report amount of days from last TBL table validation run
declare -i Last_Validated=0		# has number of days for last validation
function days_last_validated() {
    local TBL=${1/_dayly/} FORCE=$2
    DB_pass
    if create_stations $2
    then
        $MYSQL -e "
	    SELECT (to_days(now()) - to_days(validated)) FROM stations
	        WHERE stations.table = '${TBL/_norm/}' LIMIT 1
	    "
	Last_Validated=$($MYSQL -e "
	    SELECT to_days(validated) FROM stations
		WHERE stations.table = '${TBL/_norm/}' LIMIT 1")
        return $?
    fi
    return 1
}

# set mark of last run of valid check for this table
function set_last_validated() {
    local TTBL=${1/_dayly/}; if [ -z "$TTBL" ] ; then return 1; fi
    local FORCE=$2
    local LAST="subdate(now(),1)"	# avoid incompleteness of measurements of today
    if [ -n "$3" ] && echo "$3" | grep -q "^20[0-9][0-9][-/][0-1]*[0-9][-/][[0-3]*[0-9]$"
    then
	LAST="from_days(to_days('$3')-1)"
    fi
    DB_pass
    if create_stations $2
    then
	Last_Validated=$($MYSQL -e "
	    UPDATE stations SET validated = $LAST
		WHERE stations.table = '${TTBL/_norm/}';
	    SELECT to_days(validated) FROM stations
                WHERE stations.table = '${TBL/_norm/}' LIMIT 1;
	")
        return $?
    fi
    return 1
}

# checking routines to mark sensor type X_valid cel on database tables
function add_valid() {
    local TTBL=$1 SENS=$2 DFLT=$3
    if [ -z "$SENS" ] ; then return 1; fi
    if [ -z "$DFLT" ] ; then DFLT=1; fi
    if ! $MYSQL -e "DESCRIBE $TTBL" \
        | awk '/_valid/{ print $1;}' \
        | grep -q "${SENS}_valid"
    then
        $MYSQL -e "
            ALTER TABLE $TTBL
            ADD COLUMN ${SENS}_valid BOOL DEFAULT ${DFLT} COMMENT 'sense valid mark';
        "
    fi
    return 0
}

# add sensor(s) to table in a forced way (not automatic via download data)
function add_sensor() {
    local TTBL=$1 ; shift
    DB_pass
    local SENS PPB COLOR
    for SENS in $*
    do
        PPB=''
        COLOR="ADD COLUMN ${SENS}_color TEXT(6) default NULL COMMENT 'rating color',"
        COLOR=''
        if ! $MYSQL -e "DESCRIBE $TTBL" \
	    | awk "/$SENS/{ print \$1;}" \
            | grep -q "^${SENS}\$"
        then
            if ! echo "pm_ roet zwarte_rook luchtdruk wr ws rv temp" | grep -q $SENS
            then    # this is a gas add column _ppb
                PPB="ADD COLUMN ${SENS}_ppb DECIMAL(7,3) default NULL COMMENT 'ppb value',"
            fi
            $MYSQL -e "
                ALTER TABLE $TTBL
                  ADD COLUMN ${SENS} DECIMAL(6,2) default NULL COMMENT 'stake holder',
                  $COLOR
                  $PPB
                  ADD COLUMN ${SENS}_valid BOOL default 0 COMMENT 'invalid by definition'
            "
	else
	    continue
        fi
	if  $MYSQL -e "SHOW TABLES" | grep -q "^${TTBL}_dayly"
	then
	    if ! $MYSQL -e "DESCRIBE ${TTBL}_dayly" | grep -q "${SENS}_avg"
	    then
	        $MYSQL -e "
                    ALTER TABLE ${TTBL}_dayly
		      ADD COLUMN \`${SENS}_avg\` decimal(6,2)
                        DEFAULT NULL COMMENT 'day average ${SENS}',
                      ADD COLUMN \`${SENS}_stddev\` decimal(5,2)
                        DEFAULT NULL COMMENT 'day std dev ${SENS}',
                      ADD COLUMN \`${SENS}_min\` decimal(5,2)
                        DEFAULT NULL COMMENT 'day minimum ${SENS}',
                      ADD COLUMN \`${SENS}_max\` decimal(6,2)
                        DEFAULT NULL COMMENT 'day maximum ${SENS}',
                      ADD COLUMN \`${SENS}_cnt\` int(3)
                        DEFAULT NULL COMMENT 'day count of ${SENS} measurements'
                    "
	    fi
	fi
    done
    return $?
}

# make sur we have sensors in the database which are used in this script
# so std queries will not fail
declare -A HAVE_added_sensors		# only once, has stake holder names
function check_dflt_sensors() {
    local TBL=$1; if [ -z "$TBL" ] ; then return ; fi
    TBL=`echo "$TBL" | sed 's/_.*//'`  # only main tables
    if [ -n "${HAVE_added_sensors[$TBL]}" ] ; then return; fi
    # TO DO this is just a temp patch should go away
    add_sensor $1 pm_10 pm_25	# we need PM10 and PM2.5 for statistics ??
    # filter out those sensors who have values nevertheless
    HAVE_added_sensors[$TBL]=$($MYSQL -e "SHOW CREATE TABLE ${TBL}" \
	| perl -e "my \$tbl = '$TBL';" -e 'while( <STDIN> ) {
	    foreach my $l (split(/\\n/,$_)) {
	    next if $l !~ /stake holder/;
	    $l =~ s/\` .*//; $l =~ s/.*\`//;
	    print "SELECT IF( count($l) > 0, '\'\'', '\''$l'\'') FROM $tbl WHERE NOT isnull($l);\n"; }}' \
        | $MYSQL)
    if [ -z "${HAVE_added_sensors[$TBL]}" ]
	then HAVE_added_sensors[$TBL]=" "
    fi
}

# set valid mark if measurement is in range
function valid_basic(){
    local SENS=$1
    local TTBL=$2; if [ -z "$TTBL" ] ; then return 1; fi
    # TO DO: next is extra and should not be there
    if [ $Last_Validated -le 0 ]; then days_last_validated $TTBL ; fi
    if [ -n "${MAX_sens[${SENS}]}" ]
    then
	if [ -z "${MIN_sens[${SENS}]}" ] ; then MIN_sens[${SENS}] = 0; fi
        $MYSQL -e "
                UPDATE $TTBL SET ${SENS}_valid = 1
		    WHERE not isnull(${SENS}) AND to_days(datum) > ${Last_Validated};
	        UPDATE $TTBL SET ${SENS}_valid = 0
		    WHERE to_days(datum) > ${Last_Validated} AND 
		    ( isnull(${SENS}) OR
		      ${SENS} < ${MIN_sens[${SENS}]} OR
		      ${SENS} > ${MAX_sens[${SENS}]}
		    );
	    "
    else
	$MYSQL -e "
                UPDATE $TTBL SET ${SENS}_valid = 1
		    WHERE to_days(datum) > ${Last_Validated}
		      AND  not isnull(${SENS});
	        UPDATE $TTBL SET ${SENS}_valid = 0
		    WHERE to_days(datum) > ${Last_Validated}
		      AND isnull(${SENS});
	"
    fi
    # table NETT (NRW) uses 5 ug/m3 as signal for unknown value
    if [ "${SENS}" = pm_10 -a "${TTBL}" = NETT ]
    then
	$MYSQL -e "
                UPDATE $TTBL SET ${SENS}_valid = 0
	            WHERE to_days(datum) > ${Last_Validated} AND ${SENS} = 5;
	"
    fi
    return $?
}
 
# seems for RIVM data, we introduced on a test some invalid hour measuremets
# all flagged with minute 19 (TO DO should delete them from DB)
function delete_doubles() {
    local SENS=$1
    local TTBL=$2; if [ -z "$TTBL" ] ; then return 1; fi

    $MYSQL -e "
              UPDATE $TTBL SET ${SENS}_valid = 0
	         WHERE to_days(datum) > ${Last_Validated} AND minute(datum) = 19;
	      "
}

# Delete those values which are above the normal Gauss distribution
# This invalidates for the full period of operation
# function to print propability % given the decimal (3.2) N X sigma on mean 0, sigma 1
# argument in percentage eg 95.3% will print the N factor (decimal 4,1).
function Propability() {
    perl -e "my \$x = '$1';" \
	-e '
	use strict; use warnings;
	use Statistics::Distributions qw(uprob);
	$x =~ s/[^0-9\.%]//g ; $x = 0 if not $x;
	my $mean    = 0; my $stdev   = 1;

	sub try {
    	    my ($prop, $l, $r) = @_;
    	    return 0 if not $prop;
    	    my $p = 100.0 * (1-2*uprob(((($r + $l) / 2)-$mean)/$stdev));
    	    if ( $p > ($prop + 0.05) ) {
	        return try( $prop, $l, ($r + $l) / 2);
    	    }
    	    elsif ( $p < ($prop - 0.05) ) {
	        return try( $prop, ($r + $l) / 2, $r );
    	    }
    	        return (($r + $l) / 2);
	}
	if( $x =~ /%/ ) { $x =~ s/%//;
	    printf ("%3.2f", try(abs($x), 0, 10) + 0.005);
	} else {
	    printf ("%3.1f%%\n", 100.0 * (1-2*uprob((abs($x)-$mean)/$stdev)) + 0.05);
	}
    	'
}

function delete_NxStdDev() {
    local SENS=$1 ; local TTBL=$2; if [ -z "$TTBL" ] ; then return 1; fi
    if [ -z "$N_StdDev" ] ; then
        N_StdDev=$(Get_info nr ${NR:-06} a_stddev)
    fi
    if perl -e "exit not ($N_StdDev <= 0 );" ; then N_StdDev=0.68; fi #  $N_StdDev" -lt 0
    if perl -e "exit not ($N_StdDev > 5.9 );" ; then return 0; fi # turns this off

    $MYSQL -e "
            SET @max = (SELECT avg($SENS) + (${N_StdDev} * stddev($SENS))
	        FROM $TTBL
	        WHERE ${SENS}_valid);
	    UPDATE $TTBL SET ${SENS}_valid = 0
	        WHERE to_days(datum) > ${Last_Validated} AND
                ${SENS} >= @max;
    "
}

# Invalidate those values per day if measurement >= day avg + M X day stddev
# this is a heavy check/validation process
declare N_DayStdDev
declare CheckFULLGRP=''
function delete_NxDayStdDev() {
    if [ -z "$N_DayStdDev" ] ; then
        N_DayStdDev=$(Get_info nr ${NR:-06} day_stddev)
    fi
    if perl -e "exit not ($N_DayStdDev <= 0 );" ; then N_DayStdDev=0.68; fi # 50%
    if perl -e "exit not ($N_DayStdDev > 5.9 );" ; then return 0; fi # turns this off
    local SENS=$1 ; local TTBL=$2; if [ -z "$TTBL" ] ; then return 1; fi
    ### ONLY_FULL_GROUP_BY can only be disabled by super user not per session!!!
    ###         setting it will flag errors if column does not exists
    if [ -z "$CheckFULLGRP" ]
    then
        if $MYSQL -e "SELECT @@sql_mode" | grep -q "ONLY_FULL_GROUP_BY" 
        then
            echo "ATTENTION You may see mysql errors on missing columns" 1>&2
            echo "Disable ONLY_FULL_GROUP_BY in mysql config" 1>&2
        fi
        CheckFULLGRP=NO-ONLY_FULL_GROUP_BY
    fi
        
    echo -n -e "Validating values for sensor $SENS per day average ...\r" 1>&2
    $MYSQL -e "
	    # SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));
            SELECT year(datum) AS jaar, month(datum) AS maand, day(datum) AS dag,
	        (avg($SENS) + ${N_DayStdDev} * stddev($SENS)) AS max
	        FROM $TTBL
	        WHERE to_days(datum) > ${Last_Validated} AND ${SENS}_valid
	        GROUP BY jaar, maand, dag;
              " \
    | awk "{ printf(\"UPDATE $TTBL SET ${SENS}_valid = 0 WHERE ${SENS} >= %.3f AND date(datum) = '%4.4d-%2.2d-%2.2d' ; \n\",
	\$4, \$1, \$2, \$3); }
    " \
    |  $MYSQL
}

# add more valid boolean settings functions here ....

declare -A Sensor_valid_DONE
# array of validation functions on PM_valid boolean settings
valid_func[0]="valid_basic"
valid_func[1]="delete_doubles"
valid_func[2]="delete_NxStdDev"
valid_func[3]="delete_NxDayStdDev"

# function will add valid mark columns if needed so
# run an array of functions to validate sensor measurements for all sensors
# TO DO may add to only invalidate measurements
function Set_Sensor_valids() {
    local TTBL=${1/_dayly*/} ; if [ -z "$TTBL" ] ; then return 1; fi
    if [ -n "${Sensor_valid_DONE[$TTBL]}" ] ; then return 0; fi
    local SENS NO_REPORT=$2
    local -i PERIOD=$(days_last_validated $TTBL)
    PERIOD=$(($PERIOD - 1))
    DB_pass
    
    if [ "$PERIOD" -gt 0 ] # can be forced to create table
    then 	# we do not a DB table valiodation run if dome < 1 day
    # reset database table first
        local -i I
        local SENSS=$(Get_sensors $TTBL)	# inclusive climate sensors
        start_progressing "Set valid indicator for sensors in table $TTBL"
	#printf "\t%14.14s\tsensor\tinvalid (incremental)\n" "functie"  >/tmp/${TTBL}_invalids
        for SENS in $SENSS
        do
            add_valid $TTBL ${SENS}
            for (( I=0; I < ${#valid_func[@]}; I++))
            do
	        if [ -z "${valid_func[$I]}" ] ; then continue; fi 
	        ${valid_func[$I]} "$SENS" $TTBL >/dev/null
		printf "%s\t" "${valid_func[$I]}"
    		if ! $MYSQL -e "
                        SET @val=(SELECT count(*) FROM $TTBL WHERE ${SENS}_valid);
	       		SET @inval=(SELECT count(*) FROM $TTBL WHERE not ${SENS}_valid);
			SELECT '$SENS' as Sensor,
			    concat(round(@inval/(@val + @inval) * 100, 1),'%') as 'invalid';
    		"
                then
                    echo "FATAL ERROR: mysql use in set sensor values with function ${valid_func[$I]}\n" 1>&2
                    return 1
                fi
            done
        done >/tmp/${TTBL}_invalids
        stop_progressing
	if [ -z "$NO_REPORT" ]
	then
	    echo "Valid/invalidatie berekeningen tabel $TTBL."
	    echo "    Standaard deviatie percentages (incremental per function) over periode van $PERIOD dagen."
	    echo "    Gebruikte std dev spreiding: operational periode $(Propability $N_StdDev), dag periode $(Propability $N_DayStdDev)."
	    perl -e '
               my %ma = (); @fc = ();
                while ( <STDIN> ) {
                    my @ln = split /\s+/; # function\tsensor name\tpercentage
                    push @fc, $ln[0] if not $ma{$ln[0]};
                    $ma{$ln[0]}{$ln[1]} = $ln[2];
                }
                if( $#fc >= 0 ) {
                    printf("%14.14s", "sensors |");
                    for( my $i = 0; $i <= $#fc; $i++ ) {
                         $_ = $fc[$i]; s/.*_//;
                         printf(" %.8s ",$_);
                    }
                    print "\n";
                }
                for my $j (sort { $a cmp $b } keys %{$ma{$fc[0]}}) {
                    printf("%14.14s", $j);
                    for( my $i = 0; $i <= $#fc; $i++ ) {
                        printf(" %6.6s   ",$ma{$fc[$i]}{$j});
                    }
                    print "\n";
                }
	        ' </tmp/${TTBL}_invalids
	    rm -f /tmp/${TTBL}_invalids
            # add here more statements to validate measurements
	fi
        rm -f /tmp/${TTBL}_invalids
        set_last_validated $TTBL
	Build_dayly $TTBL "" no_revalidate
    fi
    Sensor_valid_DONE[$TTBL]=1
    return 0
}

# print acceptable range for a sensor value
function Range() {
    local TBL=$1
    local S=$2
    Set_Sensor_valids $TBL no_report
    printf "  %-8.8s range: " "$S"
    $MYSQL -e "
            SELECT min($S), avg($S), max($S), std($S) FROM $TBL
	      WHERE ${S}_valid
	      AND minute(datum) != 19
              " \
    | awk '{ printf("min %d, avg %.1f, max %d, 95%%-range %.1f - %.1f\n",
                $1, $2, $3, (($2 - 2 * $4) < $1 ? $1 : ($2 - 2 * $4)),
		(($2 + 2.5 * $4) > $2 ? $2 : ($2 + 2.5 * $4))); } '
}

# delete the column color from a basic table (not supported any more)
function Del_Column() {
    local TBL=$1 TYPE=$2 A
    if [ -z "${TBL}" ] || [ -z "$2" ] ; then return 0 ; fi
    # collect columns to drop
    local TYPES=$($MYSQL -e "DESCRIBE ${TBL}" \
        | awk '{ print $1; }' \
        | grep "${TYPE}\$" \
        | sort \
        | uniq \
        )
    for TYPE in $TYPES
    do
        echo "ATTENTION: going to drop column $TYPE from table $TBL." 1>&2
        read -p "Continue? [Y/n]: " A
        if [ -n "${A/[Yy]*/}" ]
        then
            echo "Column $TYPE NOT dropped" 1>&2
            continue
        fi
        $MYSQL -e "ALTER TABLE $TBL DROP COLUMN $TYPE"
        echo "Column $TYPE DROPPED" 1>&2
    done
    return $?
}
    
# print sensor type and for each sensor type the max/min value
function Max_Min() {
    local TBL=$1 T
    local SENSOR=$2
    local TYPE=$3
    local QRY="AND ${SENSOR}_valid"
    if [ -z "${TBL/*_aqi/}" ] ; then QRY="AND ${SENSOR} > 0" ; fi
    local COLS=`$MYSQL -e "DESCRIBE ${TBL}" \
        | awk "/_${TYPE}/{ print \$1; }" \
        | grep "${SENSOR}${TYPE}" \
        | sed "s/_${TYPE}.*//" \
        | sort \
        | uniq`
    if [ "$COLS" != "$SENSOR" ] ; then return; fi
    local Types=`$MYSQL -e "SELECT ${SENSOR}${TYPE}
            FROM $TBL
            WHERE (not ISNULL(${SENSOR}${TYPE})
            ORDER BY ${SENSOR}${TYPE} DESC" | sort | uniq |\
            grep -v -e FFFFFF -e 000000 -e 0F0F0F `
    if [ -z "$Types" ] ; then return ; fi
    Set_Sensor_valids ${TBL} no_report
    echo "     Scheme type $TYPE for sensor $SENSOR:"
    for T in  $Types
    do
        echo -e -n "\t$TYPE $T range: "
        $MYSQL -e "
            SELECT min($SENSOR), max($SENSOR), count(*) FROM $TBL
	      WHERE ${SENSOR}${TYPE} = '${T}' $QRY
		AND not ISNULL(${SENSOR});
            SELECT count(*) FROM $TBL
	      WHERE not ISNULL(${SENSOR}) $QRY
                  " \
        | awk '{
	      if ( NF == 3 ) {
		printf("min=%.1f,\tmax=%.1f,\t# geldig: %d",$1,$2,$3); cnt=$3; }
	      else { printf(" (%.0f%%)", (cnt / $1)*100 + 0.5); }
	   }'
	echo
    done
    echo
}

# report sensor range, 95% range, avg of values and color indications
function report-sensor-stats() {
    local TBL=$1 COLOR=$2
    echo "Sensor ranges and color schemes used on the websites:"
    local I
    for I in `Get_sensors ${TBL}`
    do
        Range $TBL $I
        if [ -n "$COLOR" ]
        then
	    Max_Min $TBL $I $2
        fi
    done
    echo
}

# print overview of ${TBL}_dayly of days with less as N (arg) hourly measurements
function report-hour-stats {
    local TBL=$1 MIN=$2
    if [ -z "$MIN" ] ; then MIN=20 ; fi
    if $MYSQL -e 'SHOW TABLES' \
        | grep -q "${TBL}_dayly"
    then
	local S
	echo -e "\nOverzicht van # dagen / jaar met minder dan $MIN uurmetingen:" 
	for S in `Get_sensors ${TBL}_dayly no_climate`
	do 
	    # station at least 1 measurement on a day
	    # echo "SELECT year(datum) AS jaar, count(*) AS '${S}' FROM ${TBL}_dayly WHERE ${S}_cnt < $MIN AND ${S}_cnt > 1 GROUP BY year(datum);" 1>&2
        # ruling is: minimal 90% valid measurements: minimal 21 hours of measurements
	    # echo "SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));"
            echo "SELECT 'jaar', '${S}';"
            echo "SELECT year(datum) AS jaar, count(*) AS '${S}' FROM ${TBL}_dayly WHERE ${S}_cnt > $MIN GROUP BY year(datum);"
            echo "SELECT 'JAAR', 'MAX', 'MIN';"
            echo "select year(datum) as JAAR, max(to_days(datum)) as MAX, min(to_days(datum)) as MIN from ${TBL} where not ISNULL($S) group by year(datum);"
        done \
        | $MYSQL \
        | perl -e '
                  my $cnt = -1, @sensors=(), %jaar = (); my $type = 0;
                  while ( <STDIN> ) {
                    chomp; @line = split /\s+/, $_; next if $#line < 0;
                    if( $line[0] =~ /jaar/ ) {
			$sensors[++$cnt]=$line[1]; $type = 0; next;
		    }
                    if( $line[0] =~ /JAAR/ ) { $type = 1; next; }
                    $jaar{$line[0]}{$sensors[$cnt]}=$line[1] if not $type;
                    $jaar{$line[0]}{$sensors[$cnt]}{MAX}=$line[1] if $type;
                    $jaar{$line[0]}{$sensors[$cnt]}{MIN}=$line[2] if $type;
                  }
                  print "\tjaar";
                  for (my $ind = 0; $ind <= $#sensors; $ind++) {
                    $_ = $sensors[$ind]; s/25/2.5/; s/_//; $_ = uc($_) if /[0-9]/;
                    $_ = substr($_, 0, 7); print "\t$_ (\%OK)";
                  }
                  print "\n";
                  foreach my $ind (sort { $a <=> $b } keys %jaar) {
                    next if not $jaar{$ind}; print "\t$ind";
                    for (my $nd= 0; $nd <= $#sensors; $nd++) {
                      my $i = 0;
		      my $days = $jaar{$ind}{$sensors[$nd]}{MAX}-$jaar{$ind}{$sensors[$nd]}{MIN}+1;
                      $i = $jaar{$ind}{$sensors[$nd]} if $jaar{$ind}{$sensors[$nd]};
                      printf "\t%3.d (%d\%)", $days-$i, int($i*100/$days);
                    }
                   print "\n";
                 }
        '
    fi
}

# print info on missing averages per sensor
# arg1=table, argi=sensor
function Missing-Days() {
    local TB=$1 SENS S
    DB_pass
    SENS=$(Get_sensors $TB no_climate)
    (
	# echo "SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));"
       echo "SELECT YEAR(datum) AS jaar, MONTH(datum) AS maand, DAY(datum) AS dag"
        echo ", TO_DAYS(datum)"
        for S in $SENS
        do
            echo ", ROUND(AVG($S),1)"
        done
        echo "FROM $TB GROUP BY jaar, maand, dag"
    ) \
    | $MYSQL \
    | perl \
        -e "my \$SENS = '$SENS';" \
        -e '
        $SENS =~ s/pm_/PM/g; $SENS =~ s/PM25/PM2.5/;
        $SENS =~ s/zwarte_rook/roet/;
        my $D_cnt = 0; my @Sensors = split /\s+/, $SENS;
        my $yr = "0"; my $cnt = 0;
        while( <STDIN> ) {
            chop;
            my @flds = split /\s+/;
            $flds[1] =~ s/^(\d)$/0$1/; $flds[2] =~ s/^(\d)$/0$1/;
            my $date = "$flds[0]/$flds[1]/$flds[2]";
            $D_cnt = int($flds[3]) -1 if not $D_cnt;
            if ( $yr !~ /$flds[0]/ ) {
                printf("\t%d days with missed sensor values.\n\n", $cnt)
                    if $yr !~ /^0/;
                $cnt = 0; $yr = $flds[0];
            }
            if ( int($flds[3]) > $D_cnt + 1 ) {
                printf("Missing $d days before %s\n",
                   (int($flds[3]) - $D_cnt -1), $date);
                $cnt += int($flds[3]) - $D_cnt -1;
            }
            $D_cnt = int($flds[3]);
            next if not /NULL/;
            printf("%s", $date);
            for( my $i = 0; $i <= $#Sensors; $i++ ) {
                my $value = "";
                $value = " avg($flds[$i+4])" if $flds[$i+4] !~ /NULL/;
                printf("%s\t%s", ($i < 1 ? ":" : ","),
                    ($value ? "$Sensors[$i]$value" : "NO $Sensors[$i]"));
            }
            printf("\n");
            $cnt++;
        }
        printf("\t%d days with missed sensor values.\n", $cnt)
            if $yr !~ /^0/;
     '
}

# show which date has not 24 hourly measurements
function has-not-24() {
    local VERBOSE=$1
    if [ "$VERBOSE" != 0 ]
    then
        echo "Table of dates with less as 24 hourly measurement values:"
        echo -e "Date\t\tNr of measurements"
    fi
    local TTBL=$2
    DB_pass
    if [ -z "$TTBL" ]
    then
        TTBL=$(Get_info nr ${NR:-06} table)
    fi
    $MYSQL -e "
	# SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));
	SELECT YEAR(datum) AS jaar, MONTH(datum) AS maand, DAY(datum) AS dag,
	    count(*) AS aantal, to_days(datum) as dagen
	    FROM $TTBL GROUP BY jaar, maand, dag;
	# make sure end date id defined
	SELECT YEAR(now()) AS jaar, MONTH(now()) AS maand, DAY(now()) AS dag,
            24 AS aantal, to_days(now()) as dagen;
	      " \
        | awk  -v VERB=$VERBOSE '
             BEGIN { count = 0; dagen = 0; dagcnt = 0; jaar = 0; }
             /20[0-9][0-9]/ {
                if ( dagcnt == 0 ) { dagen = $5; jaar = $1; jcnt = count;}
                dagcnt = $5;
                if ( VERB == 0 && jaar != $1 ) {
                    if( jcnt < count ) {
                        printf("%d\t%d days\n", jaar, count - jcnt); jcnt = count;
                    }
                }
		if ( VERB > 0 && jaar != $1 ) { printf("\n"); }
                if ( dagcnt - dagen > 1 ) {
                    if ( VERB > 0 ) {
			if ( dagcnt - dagen > 2 ) {
			   printf("%s/%2.2d/--\t 0 (%d days)\n", 
				$1, $2, dagcnt - dagen - 1 );
			} else {
                            printf("%s/%2.2d/%2.2d\t 0\n", $1, $2, $3 -1);
			}
                    }
                    count += dagcnt - dagen - 1;
                }
                if ( $4 != 24 ) {
                    if( VERB > 0 ) { printf("%s/%2.2d/%2.2d\t%2.d\n", $1, $2, $3, $4); }
                    count++;
                }
                dagen = dagcnt; jaar = $1;
             }
             END {
                if ( VERB == 0 ) {
                    if( jcnt < count ) {
                        printf("%d\t%3.d days\n", jaar, count - jcnt); jcnt = count;
                    }
                }
                printf("Amount of days with less as 24 hourly measurements:\t%3.d\n", count);
             }
	     '
}

# norm matrix will define std limits
declare -A norm
# define the matrix of different std limits
function set_std_limits() {
    if [ -n "${norm[EU2005,black,pm_25]}" ] ; then return; fi

    # question marked values are expectations
    # black are black listed limits (not valid values)
    # site values are values as found for coloring on prov. website (Nov 2011)
    # site2 values are values as found for coloring on new prov. website (May 2014)

    # limits for different PM 2.5 standards
    # limit	green	yellow	orange	red max days	max year avg
    # site	0	24?		50?	35?	40?
    # site2	0	16?	26?	31?	35?	40?
    # EU2005	0	10?	20?	50?	35?	25
    # EU2010	0	10?	20?	50	7?	25  (20 in 2020)
    # WHO	0	5?	10?	25	7?	10

    norm[site,groen,pm_25]=0
    norm[site,geel,pm_25]=24
    norm[site,oranje,pm_25]=24
    norm[site,rood,pm_25]=50	# limit PM 2.5 EU 2008
    norm[site,black,pm_25]=${MAX_sens[pm_25]}	# limit for valid data
    norm[site,days,pm_25]=35	# MAX DAYS > LIMIT
    norm[site,year,pm_25]=40	# MAX AVG PER YEAR

    norm[site2,groen,pm_25]=0
    norm[site2,geel,pm_25]=16
    norm[site2,oranje,pm_25]=26
    norm[site2,rood,pm_25]=31	# limit PM 2.5 norm ???
    norm[site2,black,pm_25]=${MAX_sens[pm_25]}	# limit for valid data
    norm[site2,days,pm_25]=35	# MAX DAYS > LIMIT
    norm[site2,year,pm_25]=40	# MAX AVG per YEAR

    norm[EU2005,groen,pm_25]=0
    norm[EU2005,geel,pm_25]=10
    norm[EU2005,oranje,pm_25]=20
    norm[EU2005,rood,pm_25]=50	# limit PM 2.5 EU 2005
    norm[EU2005,black,pm_25]=${MAX_sens[pm_25]}	# limit for valid data
    norm[EU2005,days,pm_25]=35	# MAX DAYS > LIMIT
    norm[EU2005,year,pm_25]=25	# MAX AVG per YEAR

    norm[EU2010,groen,pm_25]=0
    norm[EU2010,geel,pm_25]=10
    norm[EU2010,oranje,pm_25]=20
    norm[EU2010,rood,pm_25]=50	# limit PM 2.5 EU 2010
    norm[EU2010,black,pm_25]=${MAX_sens[pm_25]}	# limit for valid data
    norm[EU2010,days,pm_25]=7	# MAX DAYS > LIMIT
    norm[EU2010,year,pm_25]=25	# MAX AVG per YEAR

    # roet has no norm (yet) here we use PM2.5 values
    norm[EU2010,groen,roet]=0
    norm[EU2010,geel,roet]=10
    norm[EU2010,oranje,roet]=20
    norm[EU2010,rood,roet]=50
    norm[EU2010,black,roet]=${MAX_sens[roet]}
    norm[EU2010,days,roet]=7	# MAX DAYS > LIMIT
    norm[EU2010,year,roet]=25	# MAX AVG per YEAR

    norm[WHO,groen,pm_25]=0
    norm[WHO,geel,pm_25]=5
    norm[WHO,oranje,pm_25]=10
    norm[WHO,rood,pm_25]=25	# limit WHO
    norm[WHO,black,pm_25]=${MAX_sens[pm_25]}	# limit for valid data
    norm[WHO,days,pm_25]=3	# MAX DAYS > LIMIT (EMEP Status Raports)
    norm[WHO,year,pm_25]=10	# MAX AVG per YEAR

    # roet has no norm (yet) here we use PM2.5 values
    norm[WHO,groen,roet]=0
    norm[WHO,geel,roet]=5
    norm[WHO,oranje,roet]=10
    norm[WHO,rood,roet]=25
    norm[WHO,black,roet]=${MAX_sens[roet]}
    norm[WHO,days,roet]=3	# MAX DAYS > LIMIT
    norm[WHO,year,roet]=10	# MAX AVG per YEAR

    
    # limits for different PM 10 standards
    # limit	green	yellow	orange	red max days	max year avg
    # site	0	24?		50?	35?	40?
    # site2	0	21?	31?	51?	35?	40?
    # EU2005	0	10?	20?	50	35	40
    # EU2010	0	10?	20?	50	7	25
    # WHO	0	10?	20?	50	7?	20
    
    norm[site,groen,pm_10]=0
    norm[site,geel,pm_10]=24
    norm[site,oranje,pm_10]=24
    norm[site,rood,pm_10]=50
    norm[site,black,pm_10]=${MAX_sens[pm_10]}
    norm[site,days,pm_10]=35
    norm[site,year,pm_10]=40
    
    norm[site2,groen,pm_10]=0
    norm[site2,geel,pm_10]=21
    norm[site2,oranje,pm_10]=31
    norm[site2,rood,pm_10]=51
    norm[site2,black,pm_10]=${MAX_sens[pm_10]}
    norm[site2,days,pm_10]=35
    norm[site2,year,pm_10]=40
    
    norm[EU2005,groen,pm_10]=0
    norm[EU2005,geel,pm_10]=10
    norm[EU2005,oranje,pm_10]=20
    norm[EU2005,rood,pm_10]=50
    norm[EU2005,black,pm_10]=${MAX_sens[pm_10]}
    norm[EU2005,days,pm_10]=35
    norm[EU2005,year,pm_10]=40
    
    norm[EU2010,groen,pm_10]=0
    norm[EU2010,geel,pm_10]=10
    norm[EU2010,oranje,pm_10]=20
    norm[EU2010,rood,pm_10]=50
    norm[EU2010,black,pm_10]=${MAX_sens[pm_10]}
    norm[EU2010,days,pm_10]=7
    norm[EU2010,year,pm_10]=25
    
    norm[WHO,groen,pm_10]=0
    norm[WHO,geel,pm_10]=10
    norm[WHO,oranje,pm_10]=20
    norm[WHO,rood,pm_10]=50
    norm[WHO,black,pm_10]=${MAX_sens[pm_10]}
    norm[WHO,days,pm_10]=3	# ref EMEP Status Reports
    norm[WHO,year,pm_10]=20	# WHO guideline maart 2014
}
    
# report which date has not 24 hourly measurements
# default norm $DFLT_NORM long (report days with exceeding limits)
# can be called with SHORT (not extensive report), VERBOSE (report failing days)
# PERIOD=[N|jaar] N=period in days, default is jaar (per calendar year)
# on norm all and CSV_REPORT name is defined:
# as side effect generate overview table in csv/html/png format
# the csv file is better (improved extrapolation of days) as AddComment csv file
CSV2=""	    # push statiscal output here if CSV_REPORT is defined
function report-day-stats() {
    local TTBL=$1
    # report defaults
    local VERBOSE=0	    # do not print days exceeding max avg
    local NRM=${DFLT_NORM:-EU2010} SHRT=1 # norm, normal verbosity, 0 short, 2 long all days
    local ALLOW=2	    # allow N missing hourly measurements in avg calc
    local PERIOD=jaar	    # report in priods of N days or calender year (dflt)
    local INFIN=5000	    # infinite max

    while [ -n "$2" ]
    do
        case "${2^^}" in
        EU2005|EU2010|SITE|SITE2|WHO)
            SHRT=0; NRM=$2; shift
        ;;
        LONG)
            SHRT=2; VERBOSE=1; shift
        ;;
	PERIOD=JAAR)
	    PERIOD=jaar ; shift
	;;
	PERIOD=[0-9][0-9]*)
	    PERIOD=$(echo "$2" | sed -e 's/.*=//' -e 's/[^0-9]//g') ; shift
	    if [ "$PERIOD" -le 30 ]
	    then
		echo "$ERROR Cannot calculate period of $PERIOD" 1>&2
		return
	    fi
	;;
        SHORT)		# short report, no dates exceeding dayly avg
            SHRT=0; shift
        ;;
	ALLOW=*)
	    ALLOW=$(echo "$2" | sed 's/.*=//') ; shift
	;;
	[0-9][0-9]*)		# allow N missing hourly measurements
	    ALLOW=$2; shift
        ;;
	ALL*)
	    NRM=ALL; shift
	;;
        *)
            shift
        ;;
        esac
    done
    if [ "${NR/#0/}" -ge 100 ] && echo "$TTBL" | grep "_norm"
    then    return 0
	    # exclude RIVM as those are "calibrated" already
    fi
    if echo "$TTBL" | grep -q "_dayly"
    then	# maybe tables need to be updated
	try_update_db
    fi
    if [ -n "$CSV_REPORT" ]
    then
	if [ -z  "$CSV2" ]
	then
	    echo "$ATTENT Adding csv rows to ${CSV_REPORT}-2-periodes_${PERIOD}-${TTBL/_*/}.csv." 1>&2 
        fi
	CSV2="${CSV_REPORT}-2-periodes_${PERIOD}-${TTBL/_dayly*/}.csv"  # name reflects table and correction factors
	CSV2="${CSV2/_norm/}"
	if [ ! -f "$CSV2" ]
	then
	    echo '"PM";"norm";"duur periode";"max dagen";"max gem.";"periode nr";"calibratie";"aantal";"toename";"comment";"dag/jaar gem.";"extrapolated"' >"${CSV2}"
	fi
    fi
    DB_pass     # be sure we have the correct DB credits
    if [ "${NRM^^}" = ALL  ]
    then
	local N
	for N in EU2005 EU2010  WHO site site2 
	do
	    report-day-stats $TTBL $N short ALLOW=$ALLOW PERIOD=$PERIOD 
	    # exclude RIVM as those are calibrated already
	    if [ -n "$CSV_REPORT" ] && \
	       $MYSQL -e "SHOW TABLES" \
               | grep "${TTBL}_dayly" | grep -q "_norm"
	    then	# purely to add NEN results to CSV file
	        report-day-stats ${TTBL/_dayly*/}_dayly_norm $N short ALLOW=$ALLOW PERIOD=$PERIOD >/dev/null
	    fi
	done
        if [ -f "${CSV2}" ]
        then
	    Convert_CSV_HTML "${CSV2}"
        fi
	return 0
    fi
    echo "****************************************************"
    echo "****** Dayly average statistics using norm $NRM **"
    if [ -z "${norm[$NRM,rood,pm_10]}" ] ; then set_std_limits ; fi
    if [ "$VERBOSE" != 0 ]
    then
        echo "Table of periods with less as 24 valid hourly measurement values:"
	printf "%4.4s %4.4s max %4.4s operational %s\n" year days days hours
    fi
    set_std_limits
    # be sure we have marks in the table to choose valid values
    Set_Sensor_valids ${TTBL/_dayly*/} no_report
    # per day overview of # measurements, # per PM, PM day averages
    # validation is done on N X sigma spread on full period and per day
    # should we also calculate Chi Square test for day measurement validations?
    # TO DO: with not all PM sensors this gives correct results!
    # some statical persons have infinite discussions on this
    local SENS DONE
    if echo "$PERIOD" | grep -q "[0-9]"
    then
       local QRY1="datediff(now(),datum) DIV $PERIOD"
       local QRY2="period=$PERIOD"
       local QRY3="year(datum)"
    else
       local QRY1="year(datum)"
       local QRY2="jaar"
       local QRY3="year(datum)"
    fi
    local NEN=ruw
    if echo "$TTBL" | grep -q "_norm" ; then NEN=NEN ; fi
    if $MYSQL -e 'SELECT @@sql_mode' | grep -q 'ONLY_FULL_GROUP_BY'
    then
        echo "ERROR: Need mysql configured with 'ONLY_FULL_GROUP_BY' disabled. No daylay average can be compiled." 1>&2
        return 1
    fi
    for SENS in `Get_sensors $TTBL no_climate`  # report only non climate sensors
    do
        local MAX_D=$INFIN MAX_Y=$INFIN MAX_S=$INFIN
        if [ -n "${norm[$NRM,days,${SENS}]}" ]
	    then MAX_D="${norm[$NRM,days,${SENS}]}" ; fi
        if [ -n "${norm[$NRM,rood,${SENS}]}" ]
	    then MAX_S="${norm[$NRM,rood,${SENS}]}" ; fi
        if [ -n "${norm[$NRM,year,${SENS}]}" ]
	    then MAX_Y="${norm[$NRM,year,${SENS}]}" ; fi
	local F=1.0 N=0 CAL="1,0"
	Set_calibration ${SENS}
        if [ -n "${CAL_f[${SENS}]}" ] ; then F="${CAL_f[${SENS}]}" ; fi
        if [ -n "${CAL_n[${SENS}]}" ] ; then N="${CAL_n[${SENS}]}" ; fi
        if [ -n "${CAL[${SENS}]}" ] ; then CAL="${CAL[${SENS}]}" ; fi
        # define nr of days per year
        # next is disbled. Cannot be disabled per session.
	# echo "SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));"
        if [ -z "$DONE" ]
        then
	    # seems there are hour duplicates from earlier tests
	    # operational as station per period
            echo " SELECT '${QRY2^^}', 'MAAND', 'DAG', 'HOURS', 'DAGCNT';"
            echo " SELECT ${QRY1} AS '${QRY2^^}', month(datum) AS MAAND,
		    DAY(datum) AS DAG, count(*) AS HOURS, to_days(datum) AS DAGCNT
		    FROM $TTBL WHERE minute(datum) != 19
		    GROUP BY ${QRY3}, month(datum), day(datum);"
	    DONE=1
        fi
	# statistics of dayly averages per sensor per period
	# as defined in table ${TTBL}_dayly,
	# on creation of ${TTBL}_dayly validated earlier on
        echo " SELECT 'define';"
        echo " SELECT 'SENS=$SENS MAX_Y=$MAX_Y MAX_D=$MAX_D MAX_S=$MAX_S F=$F N=$N CAL=$CAL' AS define; "
        # sensor name, day average, hour cnt per day
	if echo "$TTBL" | grep -q "_dayly"
        then
            echo "SELECT '${QRY2}', 'maand', 'dag', '${SENS}', '${SENS} avg', 'year';"
            echo " SELECT ${QRY1} AS '${QRY2}', MONTH(datum) AS maand,
		    DAY(datum) AS dag, ${SENS}_cnt AS ${SENS},
		    ${SENS}_avg AS '${SENS} avg', year(datum) as year
                    FROM $TTBL WHERE ${SENS}_cnt > 0
                    GROUP BY ${QRY3}, month(datum), day(datum);
	    "
	else
            echo "SELECT '${QRY2}', 'maand', 'dag', '${SENS}', '${SENS} avg', 'year';"
            echo " SELECT ${QRY1} AS '${QRY2}', MONTH(datum) AS maand,
		    DAY(datum) AS dag, count(${SENS}) AS ${SENS},
		    avg(${SENS}) AS '${SENS} avg', year(datum) as year
                    FROM $TTBL WHERE ${SENS}_valid
		        AND minute(datum) != 19
                    GROUP BY ${QRY3}, month(datum), day(datum);
	    "
	fi
    done \
    | $MYSQL \
    | perl \
        -e "my \$VERB=$VERBOSE; my \$SHRT=$SHRT; my \$NRM='$NRM';" \
	-e "my \$INFIN=$INFIN; my \$AL=$ALLOW; my \$PER='$PERIOD';" \
	-e "my \$CSV='$CSV2'; my \$NEN='$NEN';" \
        -e '
 	my %def = (); my %sens = (); my $sensor = "";
	my @ln = (); my $d = 0; my $counter = 0;
	my $y = 0; my %yr = (); 
	# put data as well in a CSV2 file, so recalculations can be done
	open CSV, ">>$CSV" if $CSV;
	while (<STDIN>) { chomp; @ln = split;
	    if( /define/ ) { $d = 1; next ; }
	    if( /DAGCNT/ ) { $y++; next; }
	    if( $y == 1 && $#ln == 4 && not /=/ ) { # $ln[0] has period ID
		# line: year month day hour_cnt days_since_epoch
		if (not defined $yr{$ln[0]} ) {
		    $yr{$ln[0]}{days} = 0;
		    $yr{$ln[0]}{hours} = 0;
		}
		# operational as station, any measurement
		$yr{$ln[0]}{days}++ if $ln[3] >= (24 - $AL); # operational days
		$yr{$ln[0]}{hours} += $ln[3];	# operational hours
		$yr{$ln[0]}{min} = ($ln[4] -1) if not $yr{$ln[0]}{min};
		$yr{$ln[0]}{max} = $ln[4];	# operational days
		next;
	    }
	    if( $d && /=/ ) {
		my $snr = $ln[0]; $snr =~ s/.*=//;
		for( my $i = 1; $i <= $#ln; $i++ ) {
		    $ln[$i] =~ s/\\n//g; $ln[$i] =~ s/\\t//g;
		    if( $ln[$i] =~ /^(.*)=(.*)$/ ) { $def{$snr}{$1} = $2; }
		}
		$d = 0; next;
	    }
	    if ( /^(jaar|period)/ ) {
		$sensor = $ln[3];	# defines the sensor
		$y = 0;
		if ( $VERB ) {
		    for my $per (sort { $a <=> $b } keys %yr) {
			next if $yr{$per}{max} < 7;
			printf("%s%4.1d %4.1d max(%3.1d), operational hours: %d\n",
			    ($PER =~ /jaar/ ? "" : "period "),
			    $per, $yr{$per}{days},
			    $yr{$per}{max} - $yr{$per}{min},
			    $yr{$per}{hours});
		    }
		    $VERB = 0;	# only once
		}
		next;
	    }
	    next if not $sensor;	# statistics for this sensor
	    # line: jaar maand datum aantal_uren #sensed day_average calenderyear
	    if( not defined $sens{$ln[0]}{$sensor} ) {
	        $sens{$ln[0]}{$sensor}{cnt} = 0;
	        $counter = 0;
		undef $sens{$ln[0]}{$sensor}{dmax} if defined $sens{$ln[0]}{$sensor}{dmax};
	        $sens{$ln[0]}{$sensor}{yavg} = 0;
	    }
	    next if $ln[3] < (24 - $AL);	# allow max $AL failures
	    if( int($ln[4] + 0.5) >= $def{$sensor}{MAX_S} ) {
	        #push @{$sens{$ln[0]}{$sensor}{dmax}},
		$sens{$ln[0]}{$sensor}{dmax}[$counter++] =
		    sprintf("%2.2d-%2.2d-%4.4d: %4.1d",
			$ln[2],$ln[1],$ln[5], int($ln[4]+0.5));
	    }
	    $sens{$ln[0]}{$sensor}{cnt}++;		 # avg counter
	    $sens{$ln[0]}{$sensor}{yavg} += $ln[4]; # on end dived by cnt
	}
	# print statistics
	my @year_sort = sort { $a <=> $b } keys %sens if $PER =~ /jaar/;
	@year_sort = sort { $b <=> $a } keys %sens if $PER !~ /jaar/;
	for my $yer (@year_sort) {
	    # amount of operational days for this year: full year 365 or 366
	    $yr{$yer}{max} -= $yr{$yer}{min}; $yr{$yer}{min} = 0;
	    next if $yr{$yer}{max} <= 7;  # at least one week operational
	    printf("\n%s %d has %d in period of %d days with less as %d hourly measurements,\n",
		($PER =~ /jaar/ ? "Year" : "Periode nr"),
		$yer, $yr{$yer}{max} - $yr{$yer}{days}, $yr{$yer}{max}, 24 - $AL);
	    printf("\t   %2.1f%% of max %d hours of sensor measurements.\n",
		($yr{$yer}{max} * 24 - $yr{$yer}{hours})*100.0/($yr{$yer}{max} * 24),
		$yr{$yer}{max} * 24);
	    for my $s (sort { $a <=> $b } keys %def) {
		next if not $s;
		next if not $sens{$yer}{$s}{cnt};
		# average 
		$sens{$yer}{$s}{yavg} = $sens{$yer}{$s}{yavg} / $sens{$yer}{$s}{cnt};
		my $nme = $s; $name =~ s/25/2.5/; $name =~ s/_//;
		$nme = uc($nme) if $name =~ /[0-9]/;
		print " $nme sensor:\n";
		# need to extrapolate?
		my $cnt = $#{$sens{$yer}{$s}{dmax}}+1; my $extra = 0;
		my $count;
		if ( $yer > 1900 ) {	# calendar year
		    $count = 365 + ( ($yer % 4) == 0 ? 1 : 0);
		    $count /= $sens{$yer}{$s}{cnt};
		} else {		# period of N days -> year of 365 days
		    $count = $PER;
		    $count /= $sens{$yer}{$s}{cnt};
		}
		$count = int( $count * $cnt + 0.5); # extrapol. #days > day avg
		$extra = 1 if $count != $cnt;
		$sens{$yer}{$s}{e_cnt} = int($count);
		my $s_name = $s; $s_name =~ s/pm_/PM_/;
		$s_name =~ s/_25/_2.5/; $s_name =~ s/_/ /g;
		if ( $def{$s}{MAX_S} < $INFIN ) {
		    printf("    %d %sdays >$def{$s}{MAX_S} µg/m³ day avg is %d more as $def{$s}{MAX_D} ($NRM)\n",
		        $cnt, ($extra ? "($count extrapol.) " : "" ), $count - $def{$s}{MAX_D})
		            if $count >= $def{$s}{MAX_D};
		    printf("    (stat $s_name %d %sdays >$def{$s}{MAX_S} µg/m³, max $def{$s}{MAX_D} %s, $NRM, period $yer)\n",
		        $cnt, ($extra ? "($count**) " : "" ),
		        $count >= $def{$s}{MAX_D} ? "NOT OK" : "OK") if $SHRT;
		}
		my $perc = 0; my $prev = $yer + ($PER =~ /jaar/ ? -1 : 1);
		$perc = ($count - $sens{$prev}{$s}{e_cnt}) * 100.0 / $sens{$prev}{$s}{e_cnt}
			if $sens{$prev}{$s}{e_cnt} != $count && $sens{$prev}{$s}{e_cnt} >= 1;
		printf CSV ("\"%s\";\"%s\";\"%s\";%s;%s;%d;\"%s\";%.1f;%.0f%%;\"%s\";\"%s\";%s;%s\n", 
		        $s_name, $NRM,
			($PER =~ /jaar/ ? "jaar" : "$PER dagen"),
			$def{$s}{MAX_D} < $INFIN ? $def{$s}{MAX_D} : "",
			$def{$s}{MAX_S} < $INFIN ? $def{$s}{MAX_S} : "",
			$yer, $NEN, $count, $perc,
		        $count >= $def{$s}{MAX_D} ? "NOK" : "OK", "dag gem.",
			$cnt != $count ? $cnt : "", $def{$s}{CAL}) if $CSV;
		printf("    day avg valid %d of %d (fail: %2.1f%%),\n",
		    $sens{$yer}{$s}{cnt}, $yr{$yer}{max},
		    ($yr{$yer}{max} - $sens{$yer}{$s}{cnt}) * 100.0 /$yr{$yer}{max});
		if( $def{$s}{MAX_Y} < $INFIN ) {
		    printf("    period average is %3.1f, max yr avg %d %s ($NRM)\n",
		        $sens{$yer}{$s}{yavg}, $def{$s}{MAX_Y},
		        int($sens{$yer}{$s}{yavg} + 0.5) >= $def{$s}{MAX_Y} ? "NOT OK" : "OK");
		    printf("    (stat $s_name yer avg %3.1f, max %d %s, $NRM, period $yer)\n",
		        $sens{$yer}{$s}{yavg}, $def{$s}{MAX_Y},
		        int($sens{$yer}{$s}{yavg} + 0.5) >= $def{$s}{MAX_Y} ? "NOT OK" : "OK") if $SHRT;
		} else {
		    printf("    period average is %3.1f\n",
		        $sens{$yer}{$s}{yavg});
		    printf("    (stat $s yer avg %3.1f, period $yer)\n",
		        $sens{$yer}{$s}{yavg}) if $SHRT;
		}
		$perc = 0;
		$perc = ($sens{$yer}{$s}{yavg} - $sens{$prev}{$s}{yavg}) * 100.0 
			/ $sens{$prev}{$s}{yavg}
			if $sens{$prev}{$s}{yavg} > 0;
			printf CSV ("\"%s\";\"%s\";\"%s\";%d;%s;%d;\"%s\";%.1f;%.0f%%;\"%s\";\"%s\";;%s\n", 
		        $s_name, $NRM,
			($PER =~ /jaar/ ? "jaar" : "$PER dagen"),
			$PER =~ /jaar/ ? 365 : $PER,
			$def{$s}{MAX_Y} < $INFIN ? $def{$s}{MAX_Y} : "",
			$yer, $NEN, $sens{$yer}{$s}{yavg}, $perc,
		        int($sens{$yer}{$s}{yavg} + 0.5) >= $def{$s}{MAX_Y} ? "NOK" : "OK",
			"jaar gem.",$def{$s}{CAL}) if $CSV;
		if ( $SHRT > 1 && $def{$s}{MAX_S} < $INFIN ) {
		    print "    The days with dayly average >= $def{$s}{MAX_S}:\n"
			    if $#{$sens{$yer}{$s}{dmax}} >= 0;
		    my $rows = int(($#{$sens{$yer}{$s}{dmax}}+3) / 3);
		    for( my $r = 0; $r < $rows; $r++  ){
		      for( my $c = 0; $c <= 2  &&
			    ($c*$rows+$r) <= $#{$sens{$yer}{$s}{dmax}}; $c++ ){
			printf("  %s µg/m³", $sens{$yer}{$s}{dmax}[$c*$rows+$r]);
		      }
		      print "\n";
		    }
		}
	    }
	}
	close CSV if $CSV;
        '
}

# report which date has not 24 hourly measurements
# default norm $DFLT_NORM long (report days with exceeding limits)
# statistics with either pm valid (both sensors needs to be in the database
# if not we add either column, column comment marked as "stake holder"
function year-has-not-24() {
    local TBL=$1; shift
    local VERBOSE=$1;
    local NRM=${DFLT_NORM:-EU2010} SHRT=1 # defaults
    while [ -n "$2" ]
    do
	case "$2" in
	EU2005|EU2010|site|site2|WHO)
	    SHRT=0; NRM=$2; shift
	;;
	long|LONG)
	    SHRT=0; shift
	;;
	short|SHORT)
	    SHRT=1; shift
	;;
	*)
	    shift
	;;
	esac
    done
    DB_pass	# be sure we have the correct DB credits
    if [ -z "${norm[$NRM,rood,pm_10]}" ]
    then
          set_std_limits
    fi
    if [ "$VERBOSE" != 0 ]
    then
        echo "Table of years with less as 24 hourly measurement values:"
        echo -e "Year\t\tNr of measurements"
    fi
    # be sure we have marks in the table to choose valid values
    Set_Sensor_valids ${TBL} no_report
    Set_calibration pm_10 pm_25	# be sure we have PM correction factors
    # statistics are with either pm_10_valid or pm_25_valid is TRUE
    # per day overview of # measurements, # per PM, PM day averages
    # we should calculate Chi Square test if day measurement is valid?
# TO DO: with not all PM sensors this gives incorrect results!
    if $MYSQL -e 'SELECT @@sql_mode' | grep -q 'ONLY_FULL_GROUP_BY'
    then
        echo "ERROR: Need mysql configured with 'ONLY_FULL_GROUP_BY' disabled. No daylay average can be compiled." 1>&2
        return 1
    fi
    $MYSQL -e "
            SELECT 'jaar', 'maand', 'dag', 'aantal', 'PM 10 avg', 'PM 2.5 avg';
	    # SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));
            SELECT YEAR(datum) AS jaar, MONTH(datum) AS maand, DAY(datum) AS dag,
	        count(*) AS aantal,
	        count(pm_10) AS PM10, avg(pm_10) AS 'PM 10 avg',
	        count(pm_25) AS PM25, avg(pm_25) AS 'PM 2.5 avg'
	      FROM $TBL WHERE
	        pm_10_valid OR
	        pm_25_valid
	      GROUP BY jaar, maand, dag;
                            " \
	| awk  \
	     -v VERB=$VERBOSE -v SHRT=$SHRT -v NRM=$NRM \
	     -v MAX_DAYS_PM10=${norm[$NRM,days,pm_10]} \
	     -v MAX_PM10=${norm[$NRM,rood,pm_10]} \
	     -v PM10_f=${CAL_f[pm_10]} -v PM10_n=${CAL_n[pm_10]} \
	     -v MAX_DAYS_PM25=${norm[$NRM,days,pm_25]} \
	     -v MAX_PM25=${norm[$NRM,rood,pm_25]} \
	     -v PM25_f=${CAL_f[pm_25]} -v PM25_n=${CAL_n[pm_25]} \
		'
	     BEGIN {    total = 0; jaar = "xxxx";
			hours = 24; allowed = 2; # max records/allow fails per day
			yr = 0;         # total of days with data
			p_yr = 0; tot_missing = 0; # days with missing data
			tot_nrm_cnt = 0; # days exceeding std limits
			nrm_cnt = 0;    # full total of days exceeding limits
			h_cnt = 0;	# total per year of days with missing data
			first = 0;	# header printing
			pm10_cnt = 0;
			pm25_cnt = 0;   # days per PM with missing data
	     }
	     /20[0-9][0-9]/ {
		D_strg = sprintf("%s/%s/%s", $1, $2, $3);
		# check if we have a new year, print old year totals
		if ( jaar != $1 ) {
		    if( jaar != "xxxx" ) {
		        printf("Year %s has %2d days with missing hourly (%3.1f%% of max %d) measurements.\n",
				jaar, p_yr, (h_cnt/(yr*hours))*100.0, yr*hours);
		        printf("\tYear %s: missing rate ", jaar);
			# if > 99.5% PM sensor has never been operational
			if( (pm10_cnt/yr)*100.0 < 99.5 ) {
			    printf(" PM 10: %2.1f%%", (pm10_cnt/yr)*100.0);
			}
			if( (pm25_cnt/yr)*100.0 < 99.5 ) {
			    printf(" PM 2.5: %2.1f%%\n", (pm25_cnt/yr)*100.0);
			}
			printf("\n");
			if( nrm_cnt > 0 ) {
		    	    printf("\tDays with average exceeding %s limits (max PM 10: %d): %d\n",
				NRM, MAX_DAYS_PM10, nrm_cnt);
			}
			printf("\n");
		    }
		    yr = 0; p_yr = 0; jaar = $1; nrm_cnt = 0;
		    h_cnt = 0; pm10_cnt = 0; pm25_cnt = 0; first = 0;
		}
		# check if we have less as (hours - allowed) hourly measurements
		if ( $4 < hours - allowed || $5 < hours - allowed || $7 < hours - allowed ) {
		    if( VERB > 0 ) { printf("%s\t# %d, PM10:%d, PM2.5:%d\n", D_strg, $4, $5, $7); }
		    tot_missing++; p_yr++; h_cnt += $4;
		    if( $5 < hours - allowed ) { pm10_cnt++; }
		    if( $7 < hours - allowed ) { pm25_cnt++; }
		}
		if( SHRT == 0 ) {
		  if ( $6 * PM10_f + PM10_n >= MAX_PM10 || $8 * PM25_f + PM25_n >= MAX_PM25 ) {
		    nrm_cnt++; tot_nrm_cnt++;
		    if( first++ < 1 ) {
			printf("Year %d: exceeding days %s PM limit(s):\n", jaar, NRM);
		        printf("Used calibration factors: PM10=(*%2.1f,+%2.1f), PM2.5=(*%2.1f,+%2.1f):\n",
			    PM10_f, PM10_n, PM25_f, PM25_n);
		    }
		    printf("    On %s:\n\t    ", D_strg);
		    if( $6 * PM10_f + PM10_n >= MAX_PM10 ) {
			printf("PM 10 day average: %d (limit is %d) ", $6 + 0.5, MAX_PM10);
			if( $8 * PM25_f + PM25_n >= MAX_PM25 ) {
				printf("\n\t    ");
			}
		    }
		    if( $8 * PM10_f + PM10_n >= MAX_PM25 ) {
			printf("PM 2.5 day average: %d (limit is %d)", $8 + 0.5, MAX_PM25);
		    }
		    printf("\n");
		  }
		}
		total++; yr++;
	     }
	     END {
		printf("Year %s has %2d days with missing hourly (%3.1f%% of max %d) measurements.\n",
		    jaar, p_yr, (h_cnt/(yr*hours))*100.0, yr*hours);
		printf("\tYear %s: missing rate PM 10: %2.1f%%, PM 2.5: %2.1f%%\n",
		    jaar, (pm10_cnt/yr)*100.0, (pm25_cnt/yr)*100.0);
		if( nrm_cnt > 0 ) {
		    printf("\tAantal dagen met daggemiddelde groter dan %s std limiet (PM 10: max %d): %d\n",
			NRM, MAX_DAYS_PM10, nrm_cnt);
		}
		printf("\nMax metingen per dag %d, sta max %d gemiste metingen toe.\n",
		    hours, allowed);
		printf("\tTOTAAL aantal DAGEN met (elke PM is valid) metingen: %d,\n\t%d dagen met incomplete %d uurmetingen",
		    total, tot_missing, hours);
		if( tot_nrm_cnt > 0 ) {
		    printf(",\n\ttotaal aantal dagen met overschrijding van %s dagen limiet limiet (max %d per jaar): %d",
			NRM, MAX_DAYS_PM10, tot_nrm_cnt);
		}
		printf("\n\n");
	     }
	     '

}

# get list of sensors operational once for this table
# default if arg1 is null
function Get_sensors() {
    local RTS CLIMATE
    if [ -z "$1" ] ; then return 1; fi
    if [ -n "$2" ] && echo "${2^^}" | grep -q -e "NO_"
    then
	CLIMATE="wr|ws|temp|luchtdruk|rv"
    fi
    DB_pass
    check_dflt_sensors $1
    if echo "$1" | grep -q '_dayly'
    then
	RTS=`$MYSQL -e "DESCRIBE $1" \
            | awk '/_cnt/{ print $1; }' \
            | sed 's/_cnt//' \
            | grep -E -v -e "^(${CLIMATE})\$" -e "$NO_POL"`
    else
	RTS=`$MYSQL -e "DESCRIBE $1" \
            | awk '{ print $1; }' \
            | grep -E -v -e "^(${CLIMATE})\$" -e "$NO_POL"`
    fi
    # may need to delete sensors added as stake holder
    local I
    for I in ${HAVE_added_sensors[$1]} #  already deleted: id datum
    do
        RTS=`echo "$RTS" | grep -v "^$I\$"`
    done
    echo "$RTS"
    return 0
}

# correct the measurement value to one year full of daily numbers
# put the days per year in an array Ex_Pol[]
# extrapolate ingrediants: 
# either measurement on hour 00 or nr of measurements for day > $VALID_CNT
# else do not take the day avg as valid
# the file $CH_EXTRA is used as initialiation file for statistics
# function AddComment (calculate extrapolation and flags if extrapolated)
CH_EXTRA=""
function Get_days_measured() {
    if [ -f "$CH_EXTRA" ] ; then return 0; fi
    DB_pass
    local TB=$TBL SENS CNT=$1
    local COND="hour(datum) = 0"
    if ! echo "$CNT" | grep -q '^[0-9][0-9]*' ; then CNT="" ; fi
    if [ -n "$CNT" ]
    then
	TB=${TBL}_dayly
    fi
    CH_EXTRA=$(mktemp --suffix=.Check.days$1)
    if $MYSQL -e 'SELECT @@sql_mode' | grep -q 'ONLY_FULL_GROUP_BY'
    then
        echo "ERROR: Need mysql configured with 'ONLY_FULL_GROUP_BY' disabled. No daylay average can be compiled." 1>&2
        return 1
    fi
    for SENS in $(Get_sensors $TB)
    do
	if [ -n "$CNT" ] ; then COND="${SENS}_cnt > $CNT" ; fi
        $MYSQL -e "
	    # SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));
            SELECT '$SENS', datediff(now(),datum) DIV 365 AS jaar,
		count(*) AS 'nr of days'
		FROM $TB WHERE $COND GROUP BY datediff(now(),datum) DIV 365;
	    SELECT '$SENS', year(datum) AS jaar, count(*) AS 'nr of days'
		FROM $TB WHERE $COND GROUP BY year(datum);
	"
    done \
    | awk  '{ printf("Ex_Pol_%s[%s]=%s\n",$1,$2,$3); }' >"$CH_EXTRA"
    # . "$CH_EXTRA"	# take values in environment
}

# convert a specialized CSV output file to a table in basic HTML format
# Convert_CSV_HTML "report-PM10:1,0-PM2.5:1,0".csv "PM10:1,0-PM2.5:1,0"
# arguments: csv file name, caption of table (creation date will be added)
# extract TBL id for location name from file name: xyz[-id][-period_N]-TBL.csv.
function  Convert_CSV_HTML() {
    local REPORT="$1" HEADER OUTPUT TL
    if [ ! -f "$REPORT" ]
    then
	echo "$ERROR Cannot find \"$REPORT\"." 1>&2
	return 1
    fi
    #echo "Spreadsheet data converted output ${REPORT/.csv/}" 1>&2
    OUTPUT=`basename --suffix=.csv "$REPORT"`.html
    OUTPUT=${OUTPUT//\/.\//\/}
    TL=$(echo "$OUTPUT" | sed -e 's/.html//' -e 's/.*-//g')
    Set_calibration pm_10 pm_25   # this info may be incorrect
    DB_pass
    TL=$(Get_info table "$TL" name)
    HEADER="Tabel metingen $TL"
    if [ "${CAL[pm_10]}" != "1,0" -o "${CAL[pm_25]}" != "1,0" ]
    then
        HEADER+=" (toegepaste calibraties: "
	if [ "${CAL[pm_10]}" != "1,0" ]
	then
	    HEADER+="PM<sub><small>10</small></sub> (${CAL[pm_10]})"
	fi
        if [ "${CAL[pm_25]}" != "1,0" ]
        then
	    if [ "${CAL[pm_10]}" != "1,0" ] ; then HEADER+=", " ; fi
	    HEADER+="PM<sub><small>2.5</small></sub> (${CAL[pm_25]})"
        fi
	HEADER+=")"
    fi
    echo "$ATTENT Convert $REPORT for location $TL:" 1>&2
    echo "          HTML tabel output in $OUTPUT" 1>&2
    perl -e "my \$Report = \"$REPORT\"; my \$HDR = \"$HEADER\";" \
	-e '
    use POSIX qw(strftime);
    use constant {
        PM	=> 0,	# PM type: PM 10, PM 2.5 or other sensor name
        NORM	=> 1,	# limit identifier eg EU2005[?}, EU2010[?], WHO
        PERIOD	=> 2,	# time period, 365 days, calendar
        DAYS	=> 3,	# max days in excess limit, or period size (365)
        MAX	=> 4,	# exceeding limit µg/m³
        NR	=> 5,	# nr of period 0-n, or calendar year
        CAL	=> 6,	# calibrated: ruw or NEN
        CNT 	=> 7,	# (extrapolated) value: day average or day cnt
        PERC	=> 8,	# percentage of increase to previous period
        CMNT	=> 9,	# comment OK or nok (EXCEEDING LIMIT)
        GEM	=> 10,	# year or day average: jaar gem. or dag gem.
        EXT	=> 11,	# original value when extrapolated
	COR	=> 12,	# PM correction factor used in calculations
    };
    my $ugm3 = "&micro;g/m<sup><small>3</small></sup>";
    my $m1 = "<sup><small>*</small></sup>&nbsp;";
    my $m2 = "<sup><small>**</small></sup>";
    my $OK = "OK";
    my $NOK = "NOK";
    
    sub cel {
        my ($cnt, $col, $ok ) = @_;
        $cnt =~ s/^\s+//;
        $col = "background-color:#FFFFFF" if not defined $col;
        print sprintf("\t<td style=\"$col; color: %s\">%s</td>\n",
    	( $ok =~ /^NOK/i ? "#a20218" : "#0b6121" ), $cnt) if defined $ok;
        print "\t<td style=\"$col\">$cnt</td>\n" if not defined $ok;
    }
    
    my @rows = (); my $modt = strftime("%Y/%m/%d",localtime((stat($Report))[9]));
    open CSV, "<$Report" || exit 1; $Report =~ s/.*-(.*)\.csv$/$1/;
    my @periods = (); my @types = (); my @norms = (); my @sensors = ();
    my %pers = (); my %tys = (); my %nrms = (); my %sns = ();
    while ( <CSV> ) {
        my @line = ();
        chomp;
        s/\.0;/;/g; s/"//g;	# delete string delimeters
        s/\?//; s/\\n//g; s/\\t//g;
        @line = split /;/, $_, 13;
        next if $line[1] !~ /^(EU20|WHO)/;
        # "PM","norm","periode","dagen","max","nr","calibratie","aantal","toename","comment"
        $pers{$line[NR]} = 1; $tys{$line[CAL]} = 1; $sns{$line[PM]} = 1;
	$nrms{$line[NORM]} = 1;
        # $line[CMNT] = $OK if $line[CMNT] =~ /^OK/;
        # $line[CMNT] = $NOK if $line[CMNT] =~ /^NOK/;
        push @rows, \@line;
    }
    @periods = sort { $b <=> $a} keys %pers; # for periods of N days
    @periods = sort { $a <=> $b} keys %pers if $periods[0] > 1900;
    @types = sort { lc($b) cmp lc($a) } keys %tys;
    @norms = sort { lc($a) cmp lc($b) } keys %nrms;
    @sensors = sort { lc($b) cmp lc($a) } keys %sns;
    close CSV;
    
    sub search {
        my @search = @_;
        for ( my $i = 0; $i <= $#rows; $i++ ) {
    	my $fnd = 0;
    	for( my $ind = 0; $ind <= $#search; $ind++ ) {
    	    my $gotit = 0;
    	    next if not defined $search[$ind]; next if $search[$ind] =~ /^$/;
    	    for( my $j = 0; $j <= $#{$rows[$i]}; $j++ ) {
    		next if not defined ${$rows[$i]}[$j]; next if ${$rows[$i]}[$j] =~ /^$/;
    		if (${$rows[$i]}[$j] =~ /^$search[$ind]$/ ) {
    		    $fnd++; $gotit++; last; 
    		}
    	    }
    	    last if $fnd >= $#search + 1;
    	}
    	return $i if $fnd >= $#search + 1;
        }
        return -1;
        my $found = join ", ", @search;
        die("Cannot find all search values ($found) in report\n");
    }
    
    print "<table noborder cellspacing=2 cellpadding=3 rules=group>\n";
    print "<caption align=bottom><small>$HDR<br>$Report tabel d.d. $modt</br></small></caption>\n";
    print "\t<colgroup span=5>\n";
    for( my $k = 0; $k <= $#periods; $k++ ) {
        printf ("\t<colgroup span=%d>\n", (($k > 0 && $k < $#periods) ? ($#types+2) : ($#types+1))); 
    }
    # header table
    my $lb = "#e8f8ff";
    my $db = "#014361";
    my @llg = ("#f0f0f0","#dcdcdc"); my @elg = ("#e1e1e1","#cdcdcd");
    my $span = 1;
    print "<tr>\n";
    print "\t<th style=\"color: $lb; background-color: $db\">norm</th>\n";
    print "\t<th style=\"color: $lb; background-color: $db\">soort gem.</th>\n";
    print "\t<th style=\"color: $lb; background-color: $db\">max $ugm3</th>\n";
    print "\t<th style=\"color: $lb; background-color: $db\">max dagen</th>\n";
    print "\t<th style=\"padding: 0px 2px 0px 2px\"></th>\n";
    for (my $k = 0; $k <= $#periods; $k++ ) {
        my $strg = "$periods[$k]";
        if ( $periods[$k] < 1900 ) { # periode van 365 dagen
    	$strg = sprintf("%d tot %d jaar",$periods[$k]+1, $periods[$k]);
    	$strg = "afgelopen jaar" if not $periods[$k];
        }
        printf ("\t<th style=\"color: $lb; background-color: $db\" colspan=%d>$strg</th>\n", ($k > 0 ? ($#types + 2) : ($#types + 1)));
        print "\t<th style=\"padding: 2px 0px 2px 0px\">&nbsp;</th>\n" if $k < $#periods;
    }
    print "</tr>\n";
    
    # content table
    my $tit = "xxx";
    for my $pm (@sensors) {
	$span = 5 + ($#types + 3) * ($#periods + 1) -2;
        print "<tr><td colspan=${span} height=4></td></tr>\n";
    	my $c_i = 1; my $cur = "xx";
        for my $norm (@norms) {
    	    my $Nrm = $norm; # $Nrm =~ s/20[0-9][0-9]//;
	    if ( $cur ne $norm ) { $cur = $norm; $c_i++; $c_i = $c_i % 2; }
            for my $per ("jaar gem.","dag gem.") {
    	    if( $tit !~ /$pm/ ) {
    		$tit = "$pm"; $tit =~ s/pm_25/PM2.5/i; $tit =~ s/pm_/PM/i;
    	        print "<tr><td colspan=4 style=\"background-color:#bababa; color:#ffffff\">$tit</td><td style=\"padding: 2px 0px 2px 0px\">&nbsp;</td>\n";
    		for ( my $i = 0; $i <= $#periods; $i++ ) {
		    for my $ttypes (@types) {
    		       cel($ttypes, "background-color:#bababa; color:#ffffff; text-align:center");
		    }
    		    cel("&nbsp;","background-color:#bababa") if $i > 0;
    		    cel("&nbsp;","padding: 2px 0px 2px 0px") if $i < $#periods;
    		}
    		print "</tr>\n";
    	    }
    	    print "<tr>\n";
    	    my $first = 1;
    	    my $index = search($pm, $periods[0], $types[0], $norm, $per);
    	    next if $index < 0;
    	    cel(sprintf("%s",($per !~ /jaar/ ? $norm : "$Nrm")),"background-color:$llg[$c_i]");
    	    cel(sprintf("%s", ${$rows[$index]}[GEM]),"background-color:$llg[$c_i]");
    	    cel("${$rows[$index]}[MAX] $ugm3","background-color:$llg[$c_i]");
    	    #if( $per !~ /jaar/ && $norm !~ /WHO/ ) {
    	    if( $per !~ /jaar/ ) {
    		my $mark = "";
    		#$mark = " $m2" if($pm =~ /PM\s2.5/i && $norm =~ /EU20(05|10)/);
    		$mark = " $m2" if($pm =~ /PM\s2.5/i && $norm =~ /(WHO|EU20(05|10))/);
    		cel("${$rows[$index]}[DAYS] dagen$mark","background-color:$llg[$c_i]; padding: 2px 4px 2px 4px");
    	    } else {
    		cel("&nbsp;","background-color:$llg[$c_i]; padding: 2px 0px 2px 0px");
    	    }
    	    for my $yr (@periods) {
    		cel("","padding: 2px 0px 2px 0px"); my $find = -1;
		for my $type (@types) {	# ruw, NEN, and maybe others
    		    my $mark = ""; my $rat = "";
    	            my $indextyp = search($pm, $yr, $type, $norm, $per);
		    $find = $indextyp if $find < 0;
		    if ( $indextyp >= 0 ) {
			if ( ${$rows[$indextyp]}[EXT] &&
			     ${$rows[$indextyp]}[EXT] != ${$rows[$indextyp]}[CNT]) {
    		            #$mark = $m1;
			    $mark = sprintf("<sup><small>%s%d</small></sup>",
				${$rows[$indextyp]}[CNT] > ${$rows[$indextyp]}[EXT] ?
				"+" : "-",
				abs(${$rows[$indextyp]}[CNT] - ${$rows[$indextyp]}[EXT])
				);
			}
		        $rat = "";
		        $rat = "&nbsp;&nbsp;" if ${$rows[$indextyp]}[CNT] !~ /\./;
		        $rat .= "&nbsp;" if $mark =~ /^$/;
		        $rat = "&nbsp;&nbsp;" if ${$rows[$indextyp]}[CNT] !~ /\./;
		        $rat .= "&nbsp;" if $mark =~ /^$/;
    		        cel("${$rows[$indextyp]}[CNT]$rat$mark",
		            "background-color:$elg[$c_i];text-align:right",
		            ${$rows[$indextyp]}[CMNT]) if $indextype >= 0;
		    } else { cel("","padding: 2px 0px 2px 0px"); }
		}
    		cel("${$rows[$find]}[PERC]",
		    "background-color:$llg[$c_i];text-align:right") if (not $first && $find >= 0);
    		$first = 0;
    	    }
    	    print "</tr>\n";
    	}
        }
    }
    
    print "</table>\n";
    ' >"$OUTPUT"
    if [ $? -ne 0 ]
    then
	echo "$ERROR Cannot convert $REPORT to $OUTPUT." 1>&2
	return 1
    fi
    if depends wkhtmltopdf  >/dev/null
    then
	local OUT=`basename --suffix=.html $OUTPUT`
	if ! wkhtmltopdf -O Landscape -s A4 $OUTPUT $OUT.pdf 2>/dev/null >/dev/null
	then
	    echo "$WARNING Cannot convert $OUTPUT HTML to $OUT.pdf PDF." 1>&2
	    return 1
	fi 
	echo "          PDF table output in $OUT.pdf" 1>&2
        if depends convert "Image Magic" >/dev/null
	then
	    local TMP=$(mktemp --suffix=.Check.png)
	    echo "          PNG table output in $OUT.png" 1>&2
	    convert -trim -density 300 $OUT.pdf $TMP
	    convert $TMP -trim +repage -shave 10x10 -trim +repage -bordercolor white -border 25x25 $OUT.png
	    local COMMENT="Overzicht luchtvervuilende stoffen overschrijdingen"
	    if [ -n "$LOCATION" ]
	    then COMMENT+=" $LOCATION, station ${NR:-06}"
	    else COMMENT+=" lokatie $TL"
	    fi
            if perl -e "exit not ($N_StdDev < 5.9 && $N_StdDev > 0.68);" ||
               perl -e "exit not ($N_DayStdDev < 5.9 && $N_DayStdDev > 0.68);"
	    then
		COMMENT+=" Gebruikte validaties"
		COMMENT+=" volle periode: ${N_StdDev}X sigma ($(Propability ${N_StdDev}))"
		COMMENT+=", dag ${N_DayStdDev}X sigma ($(Propability ${N_DayStdDev}))."
		COMMENT+=", norm used: ${NORM:-$DFLT_NORM}."
	    fi
	    exiftool -q -m -overwrite_original_in_place \
		-Title="Tabel luchtvervuilende stoffen normen en overschrijdingen $TL" \
		-Comment="$COMMENT" \
		-Author="$AUTHOR" \
		-Copyright="Creative Commons" \
		-Keywords=pollutants \
		-Description="regionale PM overschrijdingen" \
		-CreationTime="$(date)" \
		$OUT.png
	    if [ $? -gt 0 ] ; then return 1; fi
	    rm -r $TMP
	fi
    fi
}

# args: MaxDays NORM STRG LiMiT QRY2 [Calibrated] [UseDaylyAvgTable]
# adds comments and calculates days per period corrections to mysql output
# from ${TBL}_daylt table: year average, or dayly average
# YR_COR extrapolation on days with hourly avg cnt > VALID_CNT,
# default: if measurement on 00 am then that day has valid average.
# COR: id of calibration factor used e.g. NEN dflt: ruw.
# if defined CSV_RECORD create/add csv records to spreadsheet file
# this may differ and is less secure from report-day-stats generated csv file
CSV1=""		# current file name for adding csv record of results
function AddComment() {
    local SENS="$1"	# sensor type
    local MAX="$2"	# maximum of days allowed
    local NORM="$3" STRG="$4" LMT="$5"  QRY2="$6" COR="$7" YR_COR="$8"
    if [ -n "$CSV_REPORT" ]
    then
	local TYPE="-periodes_jaar-"
	if echo "$QRY2" | grep -q "dagen"
	then
	    TYPE=$(echo "$QRY2" | sed -e "s/'//g" -e 's/ dagen//')
	    TYPE="-periodes_${TYPE}-"
	fi
	if [ -z "$CSV1" ]
	then
	    echo "$ATTENT Adding csv records to ${CSV_REPORT}${TYPE}${TBL}.csv" 1>&2
	fi
	CSV1="${CSV_REPORT}${TYPE}${TBL}.csv"  # name reflects table and correction factors
	if [ ! -f "$CSV1" ]
	then
	    echo '"PM";"norm";"duur periode";"max dagen";"max gem.";"periode nr";"calibratie";"aantal";"toename";"comment";"dag/jaar gem.";"extrapolated"' >$CSV1
	fi
    fi
    if [ -z "$COR" ]
    then
	COR=ruw
    fi
    QRY2=`echo "$QRY2" | sed "s/'//g"`
    MAX=`echo "$MAX" | sed 's/[^0-9\.]//g'`
    if ! $MYSQL -e 'SHOW TABLES' \
        | grep -q "${TBL}_dayly$"
    then
	YR_COR=""
    fi
    if [ -z "$MAX" ]; then MAX=365; fi
    if [ -z "$LMT" ]; then LMT=0; fi
    if [ ! -f "$CH_EXTRA" ] ; then Get_days_measured $YR_COR ; fi
    perl \
	-e "my \$CSV = '$CSV1'; my \$COR = '$COR'; my \$DAYS = '$CH_EXTRA';" \
	-e "my \$PM10 = '${CAL[pm_10]}'; my \$PM25 = '${CAL[pm_25]}'; my \$SENS='$SENS';" \
	-e "my \$MAX = $MAX; my \$NORM = '$NORM'; my \$STRG = '$STRG';" \
	-e "my \$LMT = $LMT; my \$QRY2 = '$QRY2'; my \$prev=0; my \$first = 0;" \
	-e '
	open DAYS, "<$DAYS" or die "Cannot open $DAYS";
	%ExPol = ();
	while(<DAYS>) {
    	    chomp;
    	    if( /Ex_Pol_(.*)\[([0-9]+).=([0-9\.]+)/ ) {
                $ExPol{$1}{$2} = $3;
    	    }
	}
	close DAYS;
	# put data as well in a CSV1 file, so recalculations can be done
	open CSV, ">>$CSV" if $CSV;
	my $extra="";
	while(<STDIN>) {
	    if( /([0-9]+)\s+([0-9\.]+)/ ) {
   	        my $yr = $1; my $var = $2; my $cor = $var;
		if ( not $ExPol{$SENS} || not $ExPol{$SENS}{$yr} ) {
   	            print "$yr\t$var\n"; next;
   	        }
		if ( /\./ ) {
		    $prev = $cor if not $first; $first = 1;
		    # period average no need to extra polate
   	            printf("%d\t\t%s%14.10s%5.0f%%",$yr,$var,"",
			$first == 0 ? 0 : ($cor-$prev)*100.0/$prev+0.05);
		    printf("\toverschrijding $LMT limiet") if $cor >= $LMT;
		    print "\tOK" if $cor < $LMT;
		    printf CSV ("\"%s\";\"%s\";\"%s\";%.0f;%.0f;%.0f;\"%s\";%.1f;%.0f%%;", 
		        $STRG, $NORM, $QRY2, $MAX, $LMT, $yr, $COR, $cor,
		        $first == 0 ? 0 : ($cor-$prev)*100.0/$prev+0.5) if $CSV;
		    printf CSV ("\"%s\";\"jaar gem.\";;%s\n",
			($cor >= $LMT ? "NOK" : "OK"),($STRG =~ /PM 10/ ? $PM10 : $PM25)) if $CSV;
		    $prev = $cor;
		} else {
   	            $cor=$var*365/$ExPol{$SENS}{$yr} if $ExPol{$SENS}{$yr};
		    my $icor = int($cor+0.5);  # a days period is integer
		    $prev = $icor if not $first; $first = 1;
		    # number of days per period, correct it
   	            printf("%d\t\t(%2.d)%4.d%s\t%5.0f%%",
			$yr,$var,$icor,($var != $icor ? "*" : ""),
			$first == 0 ? 0 : ($icor-$prev)*100.0/$prev+0.05);
		    printf CSV ("\"%s\";\"%s\";\"%s\";%.0f;%.0f;%.0f;\"%s\";%.1f;%.0f%%;", 
		        $STRG, $NORM, $QRY2, $MAX, $LMT, $yr, $COR, $icor,
		        $first == 0 ? 0 : ($icor-$prev)*100.0/$prev+0.5) if $CSV;
		    my $over = $icor - $MAX;
		    printf("\t\toverschrijding max $MAX met %d",$over) if $icor >= $MAX;
		    print "\t\tOK" if $icor < $MAX;
		    printf CSV ("\"%s\";\"dag gem.\";%s;%s\n",
			$icor >= $MAX ? "NOK" : "OK", ($var != $icor ? $var : ""),
			($STRG =~ /PM 10/ ? $PM10 : $PM25)) if $CSV;
		    $prev = $icor;
		}
		print "\n";
	    }
	    else {
		chomp;
		print STDOUT "$_\ttoename/periode\topmerkingen\n";
		$extra = "\t\t*) ge-extrapoleerd aantal dagen\n" if ! /gem\./;
	    }
	}
	print STDOUT $extra;
	close CSV if $CSV;
   '
}

# get the sensors info present on the station (arg: db tabel with measurmenets)
function Sensors_info {
    local TB=$1
    local I FR LS MAX
    DB_pass
    local SNS=$(Get_sensors $TB)
    if [ -z "$SNS" ] ; then return; fi
    Set_Sensor_valids $TB
    MAX=`$MYSQL -e " SELECT count(datum) FROM $TB"`
    echo "Overzicht gebruikte sensors en statistiek over $MAX metingen ($TB)."
    if perl -e "exit not ($N_StdDev < 5.9 && $N_StdDev > 0.68);" ||
       perl -e "exit not ($N_DayStdDev < 5.9 && $N_DayStdDev > 0.68);"
    then
        echo -e "Metingen validatie: gebruikte std dev factoren\n\tvoor volle periode ${N_StdDev}X ($(Propability $N_StdDev)),"
	echo -e "\tvoor dag periode ${N_DayStdDev}X ($(Propability $N_DayStdDev))."
    fi

    printf "\n%12.12s  %10.10s - %10.10s\t%9.9s\n"  \
	"sensor" "eerste" "laatste" "validatie"
    printf "%12.12s  %10.10s - %10.10s\t%4.4s - %4.4s\tgem.\taantal\tongeldig\n" \
	" " " " " " "min" "max"
    for I in $SNS
    do
        FR=`$MYSQL -e "
            SELECT date_format(datum,'%Y/%m/%d')
		FROM $TB WHERE NOT isnull($I) LIMIT 1"`
        LS=`$MYSQL -e "
            SELECT date_format(datum,'%Y/%m/%d')
		FROM $TB WHERE NOT isnull($I) ORDER BY datum DESC LIMIT 1"`
        local NME=$I ; NME=${NME/_/}
	if [ -z "${NME/*[0-9]*/}" ] ; then NME=${NME^^} ; fi
        printf "%-12.12s  %10.10s - %10.10s\t%-4.4s -  %-4.4s\n%37.37s\t" \
	    "$NME" "$FR" "$LS" "${MIN_sens[$I]}" "${MAX_sens[$I]}" " "
        $MYSQL -e "
            SET @max = (SELECT count(datum) FROM $TB);
	    SET @cnt = (SELECT count($I) FROM $TB);
	    SELECT min($I), max($I), round(avg($I),1), count(*),
		concat(round((@max - @cnt) / @max * 100,1),'%')
		FROM $TB WHERE not isnull($I);
	    "
        if perl -e "exit not ($N_StdDev < 5.9 && $N_StdDev > 0.68);" ||
           perl -e "exit not ($N_DayStdDev < 5.9 && $N_DayStdDev > 0.68);"
	then
	    local ND=$N_DayStdDev
	    if perl -e " exit not ($N_DayStdDev < $N_DayStdDev);"
	    then ND=$N_StdDev; fi
            printf "%37.37s\t" "validatie met std dev op $(Propability $ND):"
            $MYSQL -e "
                SET @max = (SELECT count(datum) FROM $TB);
	        SET @cnt = (SELECT count($I) FROM $TB WHERE ${I}_valid);
	        SELECT min($I), max($I), round(avg($I),1), count(*),
		    concat(round((@max - @cnt) / @max * 100,1),'%')
		    FROM $TB WHERE ${I}_valid;
	        "
	fi
    done
    echo
}

# show some overall statistics from the database
function report-days() {
    local NORM=${DFLT_NORM:-EU2010} LOOP=1 SHORT PERIOD=jaar
    if [ -z "$1" ]; then LOOP=0; fi
    while [ $LOOP = 1 ]
    do
        case "${1^^}" in
        ALL*|EU2005|EU2010|SITE|SITE2|WHO)
            NORM=$1; shift
        ;;
	LONG)
	    SHORT=long; shift	# report days exceeding max avg
        ;;
	SHORT)
	    SHORT=short; shift
        ;;
	PERIOD=[0-9][0-9]*)
	    PERIOD=$(echo "$1" | sed 's/.*=//') ; shift
	;;
	PERIOD=JAAR)
	    PERIOD=jaar ; shift
	;;
        *)
	    LOOP=0
        ;;
        esac
    done
    echo    "RAPPORTAGE $SHORT voor STATION nr ${NR:-06},"
    echo -e "PLAATS\t$LOCATION,"
    echo -e "DB\t$TBL tabel,"
    echo -e "PERIODE $PERIOD,"
    echo -e "NORM\t$NORM."
    echo
    local S
    declare -A valid invalid uren eerste laatste
    DB_pass
    Set_Sensor_valids ${TBL} no_report
    check_dflt_sensors ${TBL}

    # statistics for full operational period and measurements
    # overview of operational sensors, values, dates
    Sensors_info "$TBL"

    echo
    # statistics per period of time
    # allow 4 failing hours and generate on norm ALL csv/htmp/png overview
    report-day-stats ${TBL} $NORM $SHORT ALLOW=4 "PERIOD=$PERIOD"

    echo
    # statistics per sensor of less values per day
    # overview of amount of measurements with less as 20
    report-hour-stats ${TBL} 20

    echo
    # statistics report 95% value range per sensor
    # report color scheme used on this website
    report-sensor-stats $TBL
}

# next function seems not to work properly: segmentation violation of rrdtool
function Build_dayly_RRD() {
    local TTBL=$1
    local RRD_dayly=${RRD_DIR}/${TTBL}_dayly.rrd
    rm -f ${RRD_DIR}${RRD_dayly}
    set_std_limits
    #echo "Clear and rebuild the RRD average database from ${TTBL}_dayly" 1>&2
    local HB=$((3600*24*2)) # heartbeat
    local STEPS=30	# amount of steps for average/min/max
    local first=`$MYSQL -e "SELECT UNIX_TIMESTAMP(datum) FROM ${TTBL}_dayly LIMIT 1;"`
    if [ -z "$first" ]
    then
	echo "$ERROR: Cannot find first date in ${TTBL}_dayly from $DB" 1>&2
	return 1
    fi
    first=$(($first-3600))
    depends rrdtool "" 1
    declare -i I=-1
    local S N M
    declare -a A
    local TYPE=
    for S in $(Get_sensors ${TTBL}_dayly no_climate)
    do
	if [ -n "${MAX_sens[$S]}" ]
	then
	    # rebuid your RRD database if one changes something here
	    for N in _avg _min _max
	    do
		I+=1
		A[$I]="DS:${S}${N}:GAUGE:$HB:${MIN_sens[${S}]}:${MAX_sens[${S}]}"
	        TYPE="$TYPE${S}$N:"
	    done
	    for N in AVERAGE MIN MAX
	    do
		for M in 7 30 365 $((365*3)) $((365*6))
		do
		    I+=1
		    A[$I]="RRA:${N}:0.8:$((($M+$WIDTH)/$WIDTH)):$WIDTH"
		done
	    done
	fi
    done
    TYPE=${TYPE/%:/}
    if [ $I -lt 0 ]
    then
	echo "$ERROR: cannot create dayly RRD for table $TTBL, define max/min values." 1>&2
	return 1
    fi
    rrdtool create ${RRD_DIR}/$RRD_dayly --start $first --step $((3600*24)) "${A[@]}"
    DB_pass
    if ! $MYSQL -e "SHOW TABLES;" \
        | grep -q "${TTBL}_dayly"
    then
	Build_dayly ${TTBL}
    fi
    if [ $? != 0 ]
    then
	ECHO "$ERROR: RRD creation of ${RRD_DIR}/$RRD_dayly." 1>&2
	return 1
    fi
    local STYPE=`echo "$TYPE" | sed "s/:/, ':', /g"` 
    $MYSQL -e "
        SELECT CONCAT(UNIX_TIMESTAMP(datum), ':', $STYPE )
	    FROM ${TTBL}_dayly ORDER BY datum ;" \
    | awk "
	    /NULL/ { next; }
	    {
		system(sprintf(\"rrdtool update ${RRD_DIR}/$RRD_dayly -t $TYPE %s\",\$1));
	    }
	"
}
    
# cleanup a table from f_$SENSOR columns, mostly due to interrupted process
# of Build_dayly
function Cleanup_table() {
    local TTBL TTBLs=$1
    DB_pass
    if [ -z "$TTBLs" ] ; then TTBLs=$(Get_info nr ${NR:-06} table); fi
    if [ "$1" = all ]
    then
	TTBLs=$($MYSQL -e "SHOW TABLES" | grep -v -e _da -e _Day -e _Max | grep "^[A-Z]")
    fi
    for TTBL in $TTBLs
    do
        if ! $MYSQL -e "SHOW TABLES;" \
	    | grep -q ${TTBL}
        then
	    echo "$ERROR ${TTBL} does not exists in database $DB" 1>&2
	    continue
        fi
        local COL COLs
        COLs=$($MYSQL -e "DESCRIBE ${TTBL}" | grep f_[a-z] | awk '{print $1;}')
        for COL in $COLs
        do
	    echo "Drop column ${COL} from table ${TTBL}" 1>&2
	    echo "        ALTER TABLE ${TTBL} DROP COLUMN ${COL};"
        done
    done  | $MYSQL
    return 0
}

# create a new table ${TBL}_dayly or ${TBL}_dayly_norm (arg normalized)
# with avg/min/max per day out of hourly values
# argument 3 will not initiate a validation of values
function Build_dayly() {
    set_std_limits	# we need max value of accepted PM values

    local TTBL=$1
    if [ -z "$TTBL" ] ; then DB_pass ; TTBL=$(Get_info nr ${NR:-06} table); fi
    local NRM=$2
    if [ -n "$NRM" ]
    then
	if echo "$TTBL" | grep -q "^NL1"
        then	# we have RIVM values, do not normalize
	    return 0
        fi
	shift ; NRM="_norm"
	if ! $MYSQL -e "SHOW TABLES;" \
            | grep -q ${TTBL}${NRM}
	then
	    if [ -z "$3" ] ; then Set_Sensor_valids ${TTBL} no_report ; fi
	    check_dflt_sensors ${TTBL}
	    echo "$WARNING No ${TTBL}${NRM} correction factors table. Will try to build one." 1>&2
            if [ -z $DBUSER ] ; then DBUSER=$(GET_passwd USER) ; fi
            if [ -z "${DBPASS}" ] ; then DBPASS=$(GET_passwd) ; fi
	    $GET_DATA --user $DBUSER --pass $DBPASS --location "${NR:-06}" --newCor	# create one
	    if ! $MYSQL -e "SHOW TABLES;" \
                | grep -q ${TTBL}${NRM}
	    then
		# echo "$WARNING Cannot create new calibrated table ${TTBL}${NRM}" 1>&2
	        return 1;
	    fi
	fi
    fi
    #echo "Clear and rebuild the ${TTBL}_dayly${NRM} $DB database table" 1>&2
    # create the table
    DB_pass
    start_progressing "Create new dayly table for $TTBL $NRM"
    local SENS
    $MYSQL -e "
        DROP TABLE IF EXISTS ${TTBL}_dayly${NRM};
	CREATE TABLE ${TTBL}_dayly${NRM} (
  	\`id\` TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP COMMENT 'datum/tijd UNIX time stamp',
  	\`datum\` DATETIME DEFAULT now() COMMENT 'datum/tijd UNIX time stamp',
	UNIQUE KEY \`datum\` (\`datum\`)
	) ENGINE=InnoDB CHARSET=latin1 COMMENT='dayly averages, for PM factors:(${CAL[pm_10]}:${CAL[pm_25]})';
	"
    # insert the new dayly values
    # filter out impossible PM10 / PM2.5 values!
    local TIME=12	# make sure we handle summer/winter time correctly
    if date "+%Z" | grep -q S ; then TIME=13; fi
    if [ -z "$3" ] ; then Set_Sensor_valids ${TTBL} no_report ; fi
    local TB_EXTRA=$(mktemp --suffix=.Check.TTBL)
    # take care mysql requires same order on colums and REPLACE
    local SENSORS=$(Get_sensors $TTBL no_climate)
    for SENS in $SENSORS ${HAVE_added_sensors[$TTBL]}
    do
      # we may have added some sensors on failing script earlier on
      # the script will prepend "f_" to the sensor name as factor arithmatic
      if echo "$SENS" | grep -q "^f_"
      then
	 echo "        ALTER TABLE ${TTBL} DROP COLUMN $SENS; "
	 continue
      fi
      if [ -z "${CAL_f[$SENS]}" ] ; then CAL_f[$SENS]=1; fi
      if [ -z "${CAL_n[$SENS]}" ] ; then CAL_n[$SENS]=0; fi
      # if ! $MYSQL -e "DESCRIBE ${TTBL}_dayly${NRM};" | grep -q "${SENS}_avg"
      # then
        echo "        ALTER TABLE ${TTBL}_dayly${NRM}
	  ADD COLUMN \`${SENS}_avg\` decimal(6,2) DEFAULT NULL COMMENT 'day average ${SENS}',
	  ADD COLUMN \`${SENS}_stddev\` decimal(5,2) DEFAULT NULL COMMENT 'day std dev ${SENS}',
	  ADD COLUMN \`${SENS}_min\` decimal(5,2) DEFAULT NULL COMMENT 'day minimum ${SENS}',
	  ADD COLUMN \`${SENS}_max\` decimal(6,2) DEFAULT NULL COMMENT 'day maximum ${SENS}',
	  ADD COLUMN \`${SENS}_cnt\` int(4) DEFAULT NULL COMMENT 'day count of ${SENS} measurements';
	   "
      # fi
      echo "	ALTER TABLE ${TTBL} ADD COLUMN f_${SENS} DECIMAL(6,2) DEFAULT NULL;
	UPDATE ${TTBL} SET f_${SENS} = ${CAL_f[${SENS}]} WHERE ${SENS}_valid;
	"
      if [ -n "$NRM" ]
      then
	  $MYSQL -e "SELECT jaar, ${SENS} FROM ${TTBL}${NRM}" \
          | awk " {
		printf(\"\tUPDATE ${TTBL} SET f_${SENS} = %s WHERE f_${SENS} = ${CAL_f[${SENS}]} AND year(datum) = %s;\\n\", \$2, \$1);  
	          } "
      fi
      echo "	UPDATE ${TTBL}
	    SET f_${SENS} = ${SENS} * f_${SENS} + ${CAL_n[${SENS}]} WHERE ${SENS}_valid;"
    done >>$TB_EXTRA
    if $MYSQL -e 'SELECT @@sql_mode' | grep -q 'ONLY_FULL_GROUP_BY'
    then
        echo "ERROR: Need mysql configured with 'ONLY_FULL_GROUP_BY' disabled. No daylay average can be compiled." 1>&2
        return 1
    fi
    # echo "SET GLOBAL sql_mode=(SELECT REPLACE(@@sql_mode,'ONLY_FULL_GROUP_BY',''));" >>$TB_EXTRA
    # TO DO: use day measurements count and std dev to detect valid day avg
    echo -n "
	REPLACE INTO ${TTBL}_dayly${NRM}
	    SELECT CURRENT_TIMESTAMP AS id, date_format(datum, '%Y-%m-%d ${TIME}:00:00') AS datum" >>${TB_EXTRA}
    for SENS in $SENSORS ${HAVE_added_sensors[$TTBL]}
    do
	if echo "$SENS" | grep -q "^f_" ; then continue; fi
	echo -n ",
		avg(f_${SENS}) AS ${SENS}_avg, stddev(f_${SENS}) AS ${SENS}_stddev,
	        min(f_${SENS}) AS ${SENS}_min, max(f_${SENS}) AS ${SENS}_max,
	        count(${SENS}) AS ${SENS}_cnt"
    done >>${TB_EXTRA}
    echo "
		FROM ${TTBL} GROUP BY year(datum), month(datum), day(datum);" >>${TB_EXTRA}
    for SENS in $SENSORS ${HAVE_added_sensors[$TTBL]}
    do
	if echo "$SENS" | grep -q "^f_" ; then continue; fi
	echo "        ALTER TABLE ${TTBL} DROP COLUMN f_${SENS};"
    done  >>${TB_EXTRA}
    if cat ${TB_EXTRA} \
        | $MYSQL
    then
	stop_progressing
        rm -f ${TB_EXTRA}
    else
	stop_progressing
	cp "${TB_EXTRA}" "/tmp/${TTBL}_dayly${NRM}.sql"
	echo "$ERROR in rebuilding ${TTBL}_dayly${NRM}. See MYsql script /tmp/${TTBL}_dayly${NRM}.sql" 1>&2
        rm -f ${TB_EXTRA}
        Cleanup_table $TTBL
	exit 1
    fi
    Cleanup_table $TTBL
    return $?
}

# show yearly average of args: pm_10|pm_25 
# uses ${TBL}_dayly table of the database $DB
function avg_per_period() {
	local TYPE=$1	# TYPE can be year/jaar or period of N days (default 365)
	local NME=$2			# PM type: pm_10, pm_25
	local LMT=$3			# norm limit
	local NORM=${4:-$DFLT_NORM}	# norm e.g. EU2005, EU2010, WHO or EU2005?

	if [ -z "$NME" ]
	then
	   echo "Wrong $0 call" 1>&2
	   return 0
	fi
	local STRG=`echo "$NME" | sed -e 's/_10/10/' -e 's/_25/2.5/' -e s/pm/PM/`
	if [ -z "$TYPE" ]; then TYPE=365; fi
	if echo "$TYPE" | grep -q "[0-9]"
	then
	   local QRY1="datediff(now(),datum) DIV $TYPE"
	   local QRY2="$TYPE dagen"
	   local QRY3="order by $QRY1 DESC"	# earliest first
	else
	   local QRY1="year(datum)"
	   local QRY2="kalender $TYPE"
	   local QRY3=""
	fi
	echo -e "$STRG: $NORM norm, ruwe data (PM10(${CAL[pm_10]}),PM2.5(${CAL[pm_25]})),"
	echo -e "\t\toverschrijding gemiddelde over periode van $QRY2,"
	echo -e "\t\tavg per periode limiet max $LMT µg/m³"
	if [ -z "$VALID_CNT" ]
	then
	    echo -e "\t\tGeldigheid van daggemiddelde: er is 00 uursmeting."
	else
	    echo -e "\t\tGeldigheid van daggemiddelde: # uursmetingen > ${VALID_CNT}."
	fi
        DB_pass
	check_dflt_sensors ${TBL}
	Set_Sensor_valids ${TBL} no_report
	# AddComment will collect nr of days with missing hour 0 per period
	# missing days is a guess as still there is an day average.
	# TO DO: get days average based on say minimal 20 hour measurements
	# and extrapolate on  (days per period and amount of days with
	# days with valid average.
	if [ -n "$VALID_CNT" ]
	then
          if ! $MYSQL -e "SHOW TABLES;" \
            | grep -q "${TBL}_dayly\$"
	  then
            Build_dayly ${TBL}
          fi
	fi
	if [ -z "$VALID_CNT" ]
	then
	  $MYSQL -e "
            SELECT $QRY1 AS '$QRY2', round(avg(${NME}),1) AS 'periode gem.'
	      FROM ${TBL} WHERE ${NME}_valid GROUP BY $QRY1 $QRY3;
	  "
	else
	  $MYSQL -e "
            SELECT $QRY1 AS '$QRY2', round(avg(${NME}_avg),1) AS 'periode gem.'
	        FROM ${TBL}_dayly WHERE ${NME}_cnt > $VALID_CNT
	        GROUP BY $QRY1 $QRY3;
	  "
	fi | AddComment "$NME" 365 "$NORM" "$STRG" $LMT "$QRY2" "" $VALID_CNT
	echo
	if [ "$NO_CAL" = 0 ]
	then
	    if $MYSQL -e "SHOW TABLES;" \
            | grep -q "${TBL}_norm\$"
            then
              # show table with yearly corrected values
	      if [ -n "$VALID_CNT" ]
	      then
                if ! $MYSQL -e "SHOW TABLES;" \
                    | grep -q "${TBL}_dayly_norm\$"
	        then
                  Build_dayly ${TBL} normalized
                fi 
	      fi
	      echo
	      echo -e "$STRG: $NORM norm, NEN gecorrigeerde waarden,\n\t\toverschrijding gemiddelde van"
	      echo -e "\t\tgemiddelden over periode van $QRY2,"
	      echo -e "\t\tavg per periode limiet max $LMT µg/m³"
	      if [ -z "$VALID_CNT" ]
	      then
	          $MYSQL -e "
                    SELECT $QRY1 AS '$QRY2', round(avg(${NME}),1) AS 'periode gem.'
		        FROM ${TBL}_norm WHERE ${NME}_valid
			    AND minute(datum) != 19
		        GROUP BY $QRY1 $QRY3;
	          "
	      else
	          $MYSQL -e "
                    SELECT $QRY1 AS '$QRY2', round(avg(${NME}_avg),1) AS 'periode gem.'
		         FROM ${TBL}_dayly_norm WHERE ${NME}_cnt > $VALID_CNT
		         GROUP BY $QRY1 $QRY3;
	          "
	      fi | AddComment "$NME" 365 "$NORM" "$STRG" $LMT "$QRY2" NEN $VALID_CNT
	      echo
	    fi
	fi
	return 1
}

# show per year days exceeding a limit
# args: pm_10|pm_25 avg|max|min norm_level SITE|EU2005|EU2008|EU2010|WHO max_days
# uses ${TBL}_daYly table of the database $DB
function period_day_limits() {
	local COR=""
	local STRG="(gebruik ruwe waarden)"
	case "$1" in
	cor*|COR*|norm*)
	    COR="_norm"; shift
	    STRG="(gebruik NEN gecalibreerde waarden)"
	    if [ "$NO_CAL" != 0 ] ; then return; fi
	;;
	esac
	local TYPE=$1		# can be year/jaar or period of N days (default 365)
	local NME=$2			# PM type: pm_10, pm_25
	local LMT=$3			# norm limit
	local LEVEL=$4			# can be min, avg, max
	local NORM=${5:-$DFLT_NORM}	# norm e.g. EU2005, EU2010, WHO or EU2005?
	local MAX_DAYS=${6:-1000}	# maximum of days allowed above limit
	if [ -n "$COR" -a "${NR/#0/}" -ge 100 ]
	then	# do not report RIVM normalized overview
	    return 0
	fi
	if [ -z "$NME" -o -z "$LMT" ]
	then
	   echo "Wrong $0 call" 1>&2
	   return 0
	fi
	if [ -z "$MAX_DAYS" ]
	then
	    MAX_DAYS="?"
	fi
	if [ -z "$TYPE" ]; then TYPE=365; fi
	if ! $MYSQL -e "SHOW TABLES;" \
            | grep -q "${TBL}_dayly${COR}\$"
	then
            Build_dayly ${TBL} ${COR}
        fi
	if ! $MYSQL -e "SHOW TABLES;" \
            | grep -q "${TBL}_dayly${COR}\$"
	then
	    if [ -n "$COR" ]
	    then
	        echo "$ATTENT no yearly calibrations factors defined for this station." 1>&2
	    else
	        echo "$ATTENT Cannot find table ${TBL}_dayly${COR}." 1>&2
	    fi
	    return 1
	fi
	if echo "$TYPE" | grep -q "[0-9]"
	then
	   local QRY1="datediff(now(),datum) DIV $TYPE"
	   local QRY2="$TYPE dagen"
	   local QRY3="ORDER BY $QRY1 DESC"     # earliest first
	else
	   local QRY1="year(datum)"
	   local QRY2="kalender $TYPE"
	   local QRY3=""
	fi
	local STRG=`echo "$NME" | sed -e 's/_10/10/' -e 's/_25/2.5/' -e s/pm/PM/`
	local RUW="ruwe data (PM10(${CAL[pm_10]}),PM2.5(${CAL[pm_25]})),"
	if [ -n "$COR" ] ; then RUW="NEN data," ; fi
	echo -e "$STRG: $NORM norm, $RUW"
	echo -e "\t\toverschrijding #dagen/periode, dag $LEVEL > $LMT µg/m³,"
	echo -e "\t\tperiode van $QRY2 voor max $MAX_DAYS dagen"
	local QRY4=""
	if [ -n "$VALID_CNT" ]
        then
            echo -e "\t\tGeldigheid van daggemiddelde: # uursmetingen > ${VALID_CNT}."
	    QRY4="AND ${NME}_cnt > $VALID_CNT"
        fi
	DB_pass
	# dayly average needs to be rounded at moment of comparation
	# Regeling beoordeling luchtkwaliteit van 2007 en afronding getallen
	$MYSQL -e "
                SELECT $QRY1 AS '$QRY2', count(*) AS '#dagen/periode'
	           FROM ${TBL}_dayly${COR}
	           WHERE ${NME}_$LEVEL + 0.5 > $LMT ${QRY4}
		     AND minute(datum) != 19
	           GROUP BY $QRY1 $QRY3;
	     " \
        | AddComment "$NME" "$MAX_DAYS" "$NORM" "$STRG" "$LMT" "$QRY2" "${COR:+NEN}" $VALID_CNT
	echo
	return 1
}

############################################ graph routines #####################

# RRA database as is created by $GET_DATA(data collection download)
#%rrd_tables = (
##           'so2'=> "DS:so2:GAUGE:7200:0:60 RRA:AVERAGE:0.2:24:8760 RRA:MIN:0.1:24:8760 RRA:MAX:0.1:24:8760",
##           'no'=> "DS:no:GAUGE:7200:0:60 RRA:AVERAGE:0.2:24:8760 RRA:MIN:0.1:24:8760 RRA:MAX:0.1:24:8760",
##           'no2'=> "DS:no2:GAUGE:7200:0:60 RRA:AVERAGE:0.2:24:8760 RRA:MIN:0.1:24:8760 RRA:MAX:0.1:24:8760",
##           'o3'=> "DS:o3:GAUGE:7200:0:60 RRA:AVERAGE:0.2:24:8760 RRA:MIN:0.1:24:8760 RRA:MAX:0.1:24:8760",
#            'pm_10'=> "DS:pm_10:GAUGE:7200:0:350 RRA:AVERAGE:0.1:24:8760 RRA:MIN:0.1:24:8760 RRA:MAX:0.1:24:8760",
#            'pm_25'=> "DS:pm_25:GAUGE:7200:0:250 RRA:AVERAGE:0.1:24:8760 RRA:MIN:0.1:24:8760 RRA:MAX:0.1:24:8760",
##           'zwarte_rook'=> "DS:zwarte_rook:GAUGE:7200:0:60 RRA:AVERAGE:0.1:24:24 RRA:MIN:0.1:24:24 RRA:MAX:0.1:24:8760",
##           'roet'=> "DS:roet:GAUGE:7200:0:60 RRA:AVERAGE:0.1:24:24 RRA:MIN:0.1:24:24 RRA:MAX:0.1:24:8760",
#            'wr'=> "DS:wr:GAUGE:7200:0:360 RRA:AVERAGE:0:24:8760 RRA:MIN:0:24:8760 RRA:MAX:0:24:8760",
#            'ws'=> "DS:ws:GAUGE:7200:0:20 RRA:AVERAGE:0.1:24:8760 RRA:MIN:0.1:24:24 RRA:MAX:0.1:24:8760",
#        # temperature.rrd step 3600 secs, skip 2 hours, min 30, max 60
#        #       store values for N times step size: 24 * one hour
#            'temp'=> "DS:temp:GAUGE:7200:-30:60 RRA:AVERAGE:0.3:24:8760 RRA:MIN:0.2:24:8760 RRA:MAX:0.2:24:8760",
##           'rv'=> "DS:rv:GAUGE:7200:0:100 RRA:AVERAGE:0.2:24:8760 RRA:MIN:0.1:24:8760 RRA:MAX:0.1:24:8760",
#            'luchtdruk'=> "DS:luchtdruk:GAUGE:7200:900:1200 RRA:AVERAGE:0.5:24:8760 RRA:MIN:0.5:24:8760 RRA:MAX:0.5:24:8760",
#
#);

function ARGS() {
    local I
    for I in "$@"
    do
        # need some tricks to avoid new line consumptions by the shell
        perl -e "\$a = '$I';" \
             -e 'print STDERR "'"'\$a'"' \\\n";'
    done
    echo 1>&2
}

# create a PNG picture diagram for climate
# arg1: period in days before NOW, arg2 (optional, default now): end date
# see unix date manual for date string to be used
function klimaat() {
    local DAYS="$1"
    local NOW="$2"
    if [ -z "$NOW" ] ; then NOW=now; fi
    if [ -z "$DAYS" ]; then DAYS=365; fi
    local CF=MAX	# reduction: can be MIN, AVERAGE or MAX
 
    # time span yesterday
    # seconds from zero day today morning seconds after 12
    NOW=`date  --date="$NOW" | sed 's/[0-9]*:/0:/g'`
    if [ $? != 0 ]; then return 1; fi
    NOW=`date --date="$NOW" +%s`
    local GRID
    if [ $DAYS -ge 90 ]
    then
        GRID=WEEK:1:MONTH:1:MONTH:1:172800:%b/%y
	if [ "$WIDTH" -le 500 -o $DAYS -ge 500 ] ; then GRID=WEEK:1:MONTH:1:MONTH:1:172800:%b ; fi
    else
        GRID=HOUR:12:DAY:2:DAY:2:86400:%d/%m
	if [ "$WIDTH" -le 500 ] ; then GRID=HOUR:6:DAY:1:DAY:1:86400:%m ; fi
    fi
    local START=$(($NOW - $DAYS*24*60*60))
    local END=$NOW
    local STRT_DATE=`date --date=@$START "+%F"`
    local END_DATE=`date --date=@$END "+%F"`
    #local FIN=INF 	# colors wind dir to top graph
    local FIN=-20	# colors 10 X 10 quare below zero line
    local SS=$(Get_sensors ${TBL})
    declare -A S
    local P
    depends rrdtool "" 1
    for P in wr ws luchtdruk temp
    do
        if echo "$SS" | grep -q $P \
	   && rrdtool info ${RRD_DIR}/${RRD}.rrd | grep -q "${P}..index"
	then
	    local Mcnt=$($MYSQL -e "
                    SELECT count(*)
                      FROM ${TBL}
                      WHERE NOT isnull($P) AND ${P}_valid AND
		      UNIX_TIMESTAMP(datum) <= $END AND
                      UNIX_TIMESTAMP(datum) >= $START
                ")
	    if [ "$Mcnt" -ge 10 ]
	    then	# only a graph for more as 10 measurements in this period
	        S[$P]=1
	    fi
	fi
    done
    if [ "${#S[@]}" -le 0 ]
    then
	echo "$WARNING Cannot make climate graph missing all climate sensor values for this period $STRT_DATE - $END_DATE." 1>&2
	return
    fi
    mkdir -p "${DIAGRAMS}"
    local OUTPUT="${DIAGRAMS}/klimaat_graph-station_${NR:-06}-$DAYS-$END_DATE.png"
    OUTPUT=${OUTPUT//\/.\//\/}

    ## Graph for $DAYS
    declare -i I=-1
    declare -a A
    A=(COMMENT:'<b><i>legenda</i></b>\:\n')
    if [ -n "${S[luchtdruk]}" ] ; then
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'<b>luchtdruk</b>             ')
        A+=(DEF:druk=${RRD_DIR}/$RRD.rrd:luchtdruk:$CF)
        A+=(CDEF:sdruk=druk,850,-,5,/)
        A+=(CDEF:tdruk=sdruk,42.0,GT,UNKN,sdruk,IF)
        A+=(LINE2:tdruk#1B92E1:"  luchtdruk       ")
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'  metingen             ')
        A+=(GPRINT:druk:AVERAGE:"Gem.\: %5.0lf mBar")
        A+=(GPRINT:druk:MAX:"Max\: %5.0lf mBar")
        A+=(GPRINT:druk:MIN:"Min\: %5.0lf mBar\\n")
    fi
    if [ -n "${S[temp]}" ] ; then
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'<b>temperatuur</b>             ')
        A+=(DEF:temperature=${RRD_DIR}/${RRD}.rrd:temp:$CF)
        A+=(CDEF:freeze=temperature,-20.0,GE,temperature,0.0,GE,UNKN,temperature,IF,UNKN,IF)
        A+=(CDEF:tlow=temperature,0.0,GT,temperature,15.0,GE,UNKN,temperature,IF,UNKN,IF)
        A+=(CDEF:tmatig=temperature,15.0,GT,temperature,21.0,GE,UNKN,temperature,IF,UNKN,IF)
        A+=(CDEF:twarm=temperature,21.0,GT,temperature,27.0,GE,UNKN,temperature,IF,UNKN,IF)
        A+=(CDEF:thigh=temperature,27.0,GT,temperature,$MAX_temp.0,GE,UNKN,temperature,IF,UNKN,IF)
        A+=(CDEF:ttemp=temperature,$MAX_temp.0,GE,UNKN,temperature,IF)
        A+=(LINE2:ttemp#C0C0C0:"")
        A+=(LINE2:freeze#0000FF:"vriezend")
        A+=(LINE2:tlow#1DE0D2:"koud")
        A+=(LINE2:tmatig#6F9810:"aangenaam")
        A+=(LINE2:twarm#C2B712:"warm")
        A+=(LINE2:thigh#EE9920:"heet")
        A+=(COMMENT:" \\n")
        A+=(COMMENT:"  metingen             ")
        A+=(GPRINT:temperature:AVERAGE:"Gem.\: %5.1lf <sup>o</sup>C")
        A+=(GPRINT:temperature:MAX:"Max\: %5.1lf <sup>o</sup>C")
        A+=(GPRINT:temperature:MIN:"Min\: %5.1lf <sup>o</sup>C\\n")
    fi
    if [ -n "${S[wr]}" ] ; then
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'<b>windrichting</b> in kleuren\:')
        A+=(DEF:a=${RRD_DIR}/${RRD}.rrd:wr:AVERAGE)
        A+=(DEF:b=${RRD_DIR}/${RRD}.rrd:wr:MAX)
        A+=(DEF:c=${RRD_DIR}/${RRD}.rrd:wr:MIN)
        A+=(DEF:i=${RRD_DIR}/${RRD}.rrd:wr:AVERAGE)
        A+=(CDEF:cdefa=i,0.0,GE,i,22.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdefb=i,22.6,GE,i,67.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdefc=i,67.6,GE,i,112.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdefd=i,112.6,GE,i,157.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdefe=i,157.6,GE,i,202.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdeff=i,202.6,GE,i,247.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdefg=i,247.6,GE,i,292.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdefh=i,292.6,GE,i,337.5,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(CDEF:cdefi=i,337.6,GE,i,360.99,GE,UNKN,$FIN,IF,UNKN,IF)
        A+=(AREA:cdefa#D293D5:\"N\")
        A+=(AREA:cdefb#9DD0FF:\"NO\")
        A+=(AREA:cdefc#35C47C:\"O\")
        A+=(AREA:cdefd#5DDB27:\"ZO\")
        A+=(AREA:cdefe#85BA41:\"Z\")
        A+=(AREA:cdeff#D9E624:\"ZW\")
        A+=(AREA:cdefg#CD9E4A:\"W\")
        A+=(AREA:cdefh#D6492D:\"NW\")
        A+=(AREA:cdefi#D293D5:\"N\")
        A+=(COMMENT:" \\n")
        A+=(COMMENT:"  metingen             ")
        A+=(GPRINT:a:AVERAGE:"Gem.\: %5.1lf graden")
        A+=(GPRINT:b:MAX:"Max\: %5.1lf graden")
        A+=(GPRINT:c:MIN:"Min\: %5.1lf graden\\n")
    fi
    if [ -n "${S[ws]}" ] ; then
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'<b>windsnelheid</b>\:           ')
        A+=(DEF:winds=${RRD_DIR}/${RRD}.rrd:ws:$CF)
        A+=(CDEF:swinds=winds,2,*,20,-)
        A+=(CDEF:low=swinds,-20.0,GE,swinds,-10.0,GE,UNKN,swinds,IF,UNKN,IF)
        A+=(CDEF:matig=swinds,-10.1,GE,swinds,8.0,GE,UNKN,swinds,IF,UNKN,IF)
        A+=(CDEF:hard=swinds,8.1,GE,swinds,20.0,GE,UNKN,swinds,IF,UNKN,IF)
        A+=(CDEF:storm=swinds,20.1,GE,swinds,60.0,GE,UNKN,swinds,IF,UNKN,IF)
        A+=(LINE1.5:-10#6C5353aa:'""')
        A+=(LINE1.5:swinds#000000:"windsnelheid")
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'  metingen             ')
        A+=(GPRINT:winds:AVERAGE:'Gem.\: %5.0lf m/sec')
        A+=(GPRINT:winds:MAX:'Max\: %5.0lf m/sec')
        A+=(GPRINT:winds:MIN:'Min\: %5.0lf m/sec\n')
    fi

    declare -a B
    B=("$OUTPUT")
    B+=(-w $WIDTH -h $HEIGHT -a PNG)
    B+=(--pango-markup)
    B+=(--font DEFAULT:7:Helvetica)
    B+=(--slope-mode)
    B+=(--start $START --end $END)
    B+=(--title "klimaat $LOCATION van $STRT_DATE t/m $END_DATE")
    B+=(--watermark "grafiek d.d. $(LC_TIME='nl_NL.UTF-8' date '+%e %b %Y %H:%M'), bron: http://www.$SITE2 $ORGANISATION meetstation $ID")
    B+=(--vertical-label "temperatuur <sup>o</sup>C")
    B+=(--right-axis-label "windsnelheid m/sec")
    B+=(--lower-limit -20)
    B+=(--upper-limit 40)
    B+=(--right-axis 1:20)
    B+=(--right-axis-format %2.0lf)
    B+=(--x-grid $GRID)
    B+=(--alt-y-grid --rigid)

    # create the graph
    local INFO
    ARGS \
        /usr/bin/rrdtool graph \
        "${B[@]}" \
        "${A[@]}" 2>/var/tmp/rrdtool_cmd${DEBUG}
    if [ -n "$DEBUG" ] ; then
echo DEBUG=====\"$DEBUG\"
        echo "rrdtool klimaat graph call:" 1>&2
        cat /var/tmp/rrdtool_cmd${DEBUG} 1>&2
    fi
    if [ -f /var/tmp/rrdtool_cmd${DEBUG} ]
    then
        INFO=$(/bin/sh /var/tmp/rrdtool_cmd${DEBUG})
        rm -f /var/tmp/rrdtool_cmd${DEBUG}
    else
        INFO=$(/usr/bin/rrdtool graph "${B[@]}" "${A[@]}")
    fi

    if [ ! -f $OUTPUT ]
    then
        echo "$ERROR: error on creation of $OUTPUT by rrdtool graph" 1>&2
        return 1
    fi
    if depends exiftool "perl Image Magic"
    then
        local COMMENT="klimaat grafiek"
	if [ -n "$LOCATION" ] ; then COMMENT+=", $LOCATION, station ${NR:-06}" ; fi
        if perl -e "exit not ($N_StdDev < 5.9 && $N_StdDev > 0.68);" ||
           perl -e "exit not ($N_DayStdDev < 5.9 && $N_DayStdDev > 0.68);"
	then
	    COMMENT+=", gebruikte validatie periode: ${N_StdDev}X sigma, dag ${N_DayStdDev}X sigma."
	fi
	exiftool -q -m -overwrite_original_in_place \
    		-Title="klimaat $LOCATION van $STRT_DATE t/m $END_DATE" \
		-Comment="$COMMENT" \
		-Author="$AUTHOR" \
		-Copyright="Creative Commons" \
		-Keywords=climate \
		-Description="regionale Klimaat metingen" \
		-CreationTime="$(date)" \
		$OUTPUT
    fi

    echo "OUTPUT png klimaat graph: $OUTPUT (${WIDTH}X${HEIGHT})" 1>&2
}

declare -A Level
# create a PNG picture diagram for particul polution
# arg1: optional (raw,both,norm) define which RRD database to be used:
#	raw or normalized data (default: raw).
# arg1: period in days before NOW,
# arg2: (optional, default now): end date
# arg3: standard norm to be used.
# see unix date manual for date string to be used
# create pollutants diagram for limit standard (arg 1) default EU2005
# call: pollutant_graph [raw|both|twee|norm] days norm endDate
function pollutant_graph() {
    local EXT=""
    # defaults grading: originates from PLIM webside usage
    case "$1" in
    raw)
	shift
    ;;
    twee|both)
	shift
	pollutant_graph raw $*
	pollutant_graph norm $*
	return $?
    ;;
    norm)
	shift
	EXT=$RRDEXT
    ;;
    esac
    if [ ! -f ${RRD_DIR}/${RRD}${EXT}.rrd ]
    then
	echo "$ERROR RRD data dir ${RRD_DIR} of bestand ${RRD}${EXT}.rrd bestaat niet"
	echo "Maak het eerst aan, zie command help voor details"
	return 1
    fi
    depends rrdtool "" 1
    set_std_limits
    local DAYS=$1 ; shift
    # the limit used in this graph
    local NORM=$1 ; shift
    local NOW="$1" ; shift ; shift
    if [ -z "$NOW" ] ; then NOW=now; fi
    if [ -z "$DAYS" ]; then DAYS=365; fi
    local CF="MAX"	# take MAX, MIN or AVERAGE
    local POLs
    if [ $# -gt 0 ] ; then POLs=($*)
    else POLs=(pm_25 pm_10 roet) ; fi
    local levels=(groen geel oranje rood black)   # type of grading used
    local Snorm=$NORM
    if [ -n "$AQIon" ] ; then
	levels=(level0 level51 level101 level151 level201 level301)
	Snorm=AQI
    fi
    local P
    for P in ${POLs[@]}
    do
        if [ -z "${CAL_f[$P]}" ] ; then CAL_f[$P]=1; fi
        if [ -z "${CAL_n[$P]}" ] ; then CAL_n[$P]=0; fi
    done
 
    # time span yesterday
    # seconds from zero day today morning seconds after 12
    NOW=`date  --date "$NOW" | sed 's/[0-9]*:/0:/g'`
    if [ $? != 0 ]; then return 1; fi
    NOW=`date --date="$NOW" +%s`
    local GRID
    if [ "$DAYS" -ge 90 ]
    then
        GRID=WEEK:1:MONTH:1:MONTH:1:172800:%b/%y
	if [ "$WIDTH" -le 500  -o $DAYS -ge 500 ] ; then GRID=WEEK:1:MONTH:1:MONTH:1:172800:%b ; fi
    else
        GRID=HOUR:12:DAY:1:DAY:2:86400:%d/%m
	if [ "$WIDTH" -le 500 ] ; then GRID=DAY:1:MONTH:1:MONTH:1:86400:%m ; fi
    fi
    local START=$(($NOW - $DAYS*24*60*60))
    local END=$NOW
    local STRT_DATE=`date --date=@$START "+%F"`
    local END_DATE=`date --date=@$END "+%F"`
    if [ -z "$NORM" ]; then NORM=EU2005; Snorm=$NORM ; fi
    case $NORM in
    site|site2|EU2005|EU2010|WHO)
	echo "Using limits of norm $Snorm index for pollutant graph" 1>&2
    ;;
    *)
	echo "$ERROR: Unkown limit standard" 1>&2
	return
    ;;
    esac

    local OUTPUT
    mkdir -p "${DIAGRAMS}"
    if [ -z "$EXT" ]
    then
        OUTPUT="${DIAGRAMS}/fijnstof_graph-station_${NR:-06}-raw-${NORM}-$DAYS-$END_DATE.png"
    else
        OUTPUT="${DIAGRAMS}/fijnstof_graph-station_${NR:-06}-norm-${NORM}-$DAYS-$END_DATE.png"
    fi
    OUTPUT=${OUTPUT//\/.\//\/}
    local SN=$NORM
    if [ "$NORM" = site ] ; then SN=$SITE ; fi
    if [ "$NORM" = site2 ] ; then SN=$SITE2 ; fi
    if [ -n "$AQIon" ] ; then SN="AQI index" ; fi
    local TYPE="(ruwe data)"
    if [ -n "$EXT" ]
    then
	TYPE="(genormaliseerde data)"
    fi

    local SS=$(Get_sensors ${TBL})
    declare -A S
    for P in ${POLs[@]}		# check if pollutants are in the rrd database
    do
        if echo "$SS" | grep -q $P && rrdtool info ${RRD_DIR}/${RRD}${EXT}.rrd \
		| grep -q "${P}..index"
	then
	    local Mcnt=$($MYSQL -e "
                    SELECT count(*)
                      FROM ${TBL}
                      WHERE NOT isnull($P) AND ${P}_valid AND
		      UNIX_TIMESTAMP(datum) <= $END AND
                      UNIX_TIMESTAMP(datum) >= $START
                ")
	    if [ "$Mcnt" -ge 10 ]
	    then	# only a graph for more as 10 valid measurements in this period
	        S[$P]=1		# pollutant is in the database
	    fi
	fi
    done
    if [ "${#S[@]}" -le 0 ]
    then
	echo "$WARNING Cannot make pollutant graph missing e.g. PM10 and PM2.5 sensor values for period $STRT_DATE - $END_DATE." 1>&2
	return
    fi

    declare -A qual
    qual[groen]=goed
    qual[geel]=matig
    qual[oranje]=oppassen
    qual[rood]=hoog
    qual[black]=schadelijk

    qual[level0]=goed
    qual[level51]=matig
    qual[level101]=opgepast
    qual[level151]=ongezond
    qual[level201]=riskant
    qual[level301]=inferieur
    qual[level401]=hachelijk

    declare -A names

    declare -A color
    # color graph backgrounds for levels groen, geel, oranje, rood
    declare -A Band	# max are, take care pollution area is longest first
    local NR=0
    for P in ${POLs[@]}
    do
        if [ -z "${S[$P]}" ] ; then continue ; fi
        for SS in ${levels[@]} limit line   # make sure all levels have a color
	do color[$P,$SS]=B4FF00 ; done

        case "${P}X" in
        pm_25X)
            names[pm_25]="ultrafijnstof"
            # this band reaches to highest value boundary MAX pm_10
            # HTML coloring PM 2.5
            color[pm_25,groen]=00EE00AA
            color[pm_25,geel]=FFFF00AA
            color[pm_25,oranje]=FFA200AA
            color[pm_25,rood]=FF4141AA
            if echo "$NORM" | grep -q site2
            then
	        # website uses: ffffc3, ffe6c8, f7c2bc, 7d0c82
                color[pm_25,groen]=DBD953AA
                color[pm_25,geel]=CB9A66AA
                color[pm_25,oranje]=C47F78AA
                color[pm_25,rood]=7D0C82AA
            fi
            color[pm_25,black]=D0D0D0AA	# not used
# Polution pm_25 = 0 has color 0x00E400 "goed", on AQI level 0.
# Polution pm_25 = 12 has color 0xFFFF00 "matig", on AQI level 51.
# Polution pm_25 = 35 has color 0xFF7E00 "opgepast", on AQI level 101.
# Polution pm_25 = 55 has color 0xFF0000 "ongezond", on AQI level 151.
# Polution pm_25 = 150 has color 0x99004C "riskant", on AQI level 201.
# Polution pm_25 = 250 has color 0x7E0024 "inferieur", on AQI level 301.
# Polution pm_25 = 350 has color 0x24000A "hachelijk", on AQI level 401.
	    norm[AQI,level0,pm_25]=12
	    norm[AQI,level51,pm_25]=35
	    norm[AQI,level101,pm_25]=55
	    norm[AQI,level151,pm_25]=150
	    norm[AQI,level201,pm_25]=250
	    norm[AQI,level301,pm_25]=350
	    norm[AQI,level401,pm_25]=500

            color[pm_25,limit]=7C0404
            color[pm_25,line]=E36E05
	;;
	pm_10X)
            names[pm_10]=fijnstof
            # HTML coloring PM 10
            color[pm_10,groen]=A5F1A5
            color[pm_10,geel]=F8F79F
            color[pm_10,oranje]=FBD79F
            color[pm_10,rood]=FF8080
            if echo "$NORM" | grep -q site2
            then
	        # website uses: ffffc3, ffe6c8, f7c2bc, 7d0c82
                color[pm_10,groen]=FFFFC3
                color[pm_10,geel]=FFE6C8
                color[pm_10,oranje]=F7C2BC
                color[pm_10,rood]=DC16E3
            fi
            color[pm_10,black]=C0C0C0
            color[pm_10,groen]=00EE00AA
            color[pm_10,geel]=FFFF00AA
            color[pm_10,oranje]=FFA200AA
            color[pm_10,rood]=FF4141AA
            if echo "$NORM" | grep -q site2
            then
	        # website uses: ffffc3, ffe6c8, f7c2bc, 7d0c82
                color[pm_10,groen]=DBD953AA
                color[pm_10,geel]=CB9A66AA
                color[pm_10,oranje]=C47F78AA
                color[pm_10,rood]=7D0C82AA
            fi
            color[pm_10,black]=D0D0D0AA	# not used
# Polution pm_10 = 0 has color 0x00E400 "goed", on AQI level 0.
# Polution pm_10 = 55 has color 0xFFFF00 "matig", on AQI level 51.
# Polution pm_10 = 155 has color 0xFF7E00 "opgepast", on AQI level 101.
# Polution pm_10 = 255 has color 0xFF0000 "ongezond", on AQI level 151.
# Polution pm_10 = 355 has color 0x99004C "riskant", on AQI level 201.
# Polution pm_10 = 425 has color 0x7E0024 "inferieur", on AQI level 301.
# Polution pm_10 = 505 has color 0x24000A "hachelijk", on AQI level 401.
	    norm[AQI,level0,$P]=55
	    norm[AQI,level51,$P]=155
	    norm[AQI,level101,$P]=255
	    norm[AQI,level151,$P]=355
	    norm[AQI,level201,$P]=425
	    norm[AQI,level301,$P]=505
	    norm[AQI,level401,$P]=600
        
            color[pm_10,limit]=7B1464
            color[pm_10,line]=BF0404
	;;
	coX)
            names[co]=koolstofmonoxide
# Polution co = 0 has color 0x00E400 "goed", on AQI level 0.
# Polution co = 0.006 has color 0xFFFF00 "matig", on AQI level 54.
# Polution co = 0.012 has color 0xFF7E00 "opgepast", on AQI level 104.
# Polution co = 0.016 has color 0xFF0000 "ongezond", on AQI level 158.
# Polution co = 0.020 has color 0x99004C "riskant", on AQI level 206.
# Polution co = 0.038 has color 0x7E0024 "inferieur", on AQI level 304.
# Polution co = 0.050 has color 0x24000A "hachelijk", on AQI level 401.
	    norm[AQI,level0,$P]=0.006
	    norm[AQI,level51,$P]=0.012
	    norm[AQI,level101,$P]=0.016
	    norm[AQI,level151,$P]=0.020
	    norm[AQI,level201,$P]=0.038
	    norm[AQI,level301,$P]=0.050
	    norm[AQI,level401,$P]=0.1
            color[co,line]=B4FF00		# groen

            echo "Please define colors for different levels for $P" 1>&2
	;;
	co2X)
            names[co2]=koostofdioxide
            color[co2,line]=C4FF00		# groen
            echo "Please define colors for different levels for $P" 1>&2
	;;
	noX)
            names[no]=stikstofmonoxide
            color[no,line]=B6C103		# geel
            echo "Please define colors for different levels for $P" 1>&2
	;;
	no2X)
# Polution no2 = 0 has color 0x00E400 "goed", on AQI level 0.
# Polution no2 = 109 has color 0xFFFF00 "matig", on AQI level 51.
# Polution no2 = 204 has color 0xFF7E00 "opgepast", on AQI level 101.
# Polution no2 = 731 has color 0xFF0000 "ongezond", on AQI level 151.
# Polution no2 = 1316 has color 0x99004C "riskant", on AQI level 201.
# Polution no2 = 2532 has color 0x7E0024 "inferieur", on AQI level 301.
# Polution no2 = 3342 has color 0x24000A "hachelijk", on AQI level 401.
	    norm[AQI,level0,$P]=109
	    norm[AQI,level51,$P]=204
	    norm[AQI,level101,$P]=731
	    norm[AQI,level151,$P]=1316
	    norm[AQI,level201,$P]=2532
	    norm[AQI,level301,$P]=3342
	    norm[AQI,level401,$P]=5000

            color[no2,line]=C6C103		# geel
            echo "Please define colors for different levels for $P" 1>&2
	;;
	o3X)
            names[o3]=ozon
# Polution o3 = 0 has color 0x00E400 "goed", on AQI level 0.
# Polution o3 = 150 has color 0xFFFF00 "matig", on AQI level 51.(extrapolated)
# Polution o3 = 264 has color 0xFF7E00 "opgepast", on AQI level 101.
# Polution o3 = 348 has color 0xFF0000 "ongezond", on AQI level 151.
# Polution o3 = 433 has color 0x99004C "riskant", on AQI level 201.
# Polution o3 = 856 has color 0x7E0024 "inferieur", on AQI level 301.
# Polution o3 = 1067 has color 0x24000A "hachelijk", on AQI level 401.
	    norm[AQI,level0,$P]=150
	    norm[AQI,level51,$P]=264
	    norm[AQI,level101,$P]=348
	    norm[AQI,level151,$P]=433
	    norm[AQI,level201,$P]=857
	    norm[AQI,level301,$P]=1067
	    norm[AQI,level401,$P]=2000

            color[o3,line]=7C0404		#
            echo "Please define colors for different levels for $P" 1>&2
	;;
	so2X)
            names[so2]=zwaveldioxide
# Polution so2 = 0 has color 0x00E400 "goed", on AQI level 0.
# Polution so2 = 101 has color 0xFFFF00 "matig", on AQI level 51.
# Polution so2 = 214 has color 0xFF7E00 "opgepast", on AQI level 101.
# Polution so2 = 524 has color 0xFF0000 "ongezond", on AQI level 151.
# Polution so2 = 860 has color 0x99004C "riskant", on AQI level 201.
# Polution so2 = 1706 has color 0x7E0024 "inferieur", on AQI level 301.
# Polution so2 = 2270 has color 0x24000A "hachelijk", on AQI level 401.
	    norm[AQI,level0,$P]=101
	    norm[AQI,level51,$P]=214
	    norm[AQI,level101,$P]=524
	    norm[AQI,level151,$P]=860
	    norm[AQI,level201,$P]=1706
	    norm[AQI,level301,$P]=2270
	    norm[AQI,level401,$P]=4000

            color[so2,line]=E36E05		#
            echo "Please define colors for different levels for $P" 1>&2
	;;
	so)
            names[so]=zwaveloxide
            color[so,line]=C34E05		#
            echo "Please define colors for different levels for $P" 1>&2
	;;
	roetX)
            names[roet]=roet
            # HTML coloring roet
            color[roet,groen]=B4FF00
            color[roet,geel]=B6C103
            color[roet,oranje]=E1C103
            color[roet,rood]=C62804
            color[roet,black]=D0D0D0   # not used

            # color[roet,limit]=7C0404   # no limit defined
            color[roet,line]=000000		# graph line in black
        
	;;
	*)
            echo "Do not have name, colors for $P" 1>&2
            return
	;;
        esac
	color[$P,level0]=00E400AA
	color[$P,level51]=FFFF00AA
	color[$P,level101]=FF7E00AA
	color[$P,level151]=FF0000AA
	color[$P,level201]=99004CAA
	color[$P,level301]=7E0024AA
	color[$P,level401]=24000AAA
        # Band[$P]=$((${norm[$NORM,rood,pm_25]} * 2 * (${#S[@]} - $NR) / ${#S[@]} )) # twice as pm_10 band
        Band[$P]=$((${MAX_sens[pm_10]} * (${#S[@]} - $NR) / ${#S[@]} )) # twice as pm_10 band
        NR=$(($NR + 1))
    done

    declare -A Cor 		# correction details for different pollutants
    # next should be an ordered list
    local level type=$Snorm
    echo "Using pollutant color grading levels: ${levels[@]}, norm: ${type/AQI/AQI index}" 1>&2
    echo "Using bands for pollutants: ${!Band[@]}, values: ${Band[*]}" 1>&2
    # std limits used in the graph
    for indx in ${POLs[@]}
    do
	for level in ${levels[@]}
	do
            Level[$level,$indx]=${norm[$type,$level,${indx}]}
	done
        # here we introduce the artificial correction factors for the graph
        Cor[$indx,p]=${CAL_n[$indx]/-/} ; Cor[$indx,a]="+"
        if echo "${CAL_n[$indx]}" | grep -q -e '-' ; then Cor[$indx,a]="-" ; fi
    done

    ## Graph for $DAYS
    declare -i I ; I=-1
    declare -a A
    local EXTRA
    A+=(COMMENT:"<b><i>legenda</i></b>\:\\n")

    local Pname RAXIS		# simple name, right axis label names
    local -i STACK=0
    for P in ${POLs[@]}
    do
        if [ -z "${S[$P]}" ] ; then continue ; fi
	Pname=${P/_/} ; Pname=${Pname/25/2.5} ; Pname=${Pname^^}
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'                            ')
        A+=(COMMENT:" \\n")
        A+=(COMMENT:"<b>${names[$P]} ${Pname//[0-9.]/}<sub><small>${Pname//[A-Z]/}</small></sub> µg/m<sup><small>3</small></sup></b> kleur band ${STACK}-$((${MAX_sens[pm_10]} / ${#S[@]} + ${STACK}))")
        A+=(DEF:${P}i=${RRD_DIR}/${RRD}${EXT}.rrd:${P}:$CF )
        A+=(CDEF:${P}=${P}i,${CAL_f[$P]},*,${Cor[$P,p]},${Cor[$P,a]} )

        if [ -n "${Band[$P]}" ]
        then
	    for (( level=0; level < ${#levels[@]}; level++))
            do
		if [ -z "${Level[${levels[$(($level+1))]},$P]}" ]
		then continue ; fi
                
	        # A+=(CDEF:${levels[$level]}${P}=${P},${Level[${levels[$level]},$P]},GE,$P,${Level[${levels[$(($level+1))]},$P]},GE,UNKN,${Band[$P]},IF,UNKN,IF )
	        A+=(CDEF:${levels[$level]}${P}=${P},${Level[${levels[$level]},$P]},GE,$P,${Level[${levels[$(($level+1))]},$P]},GE,UNKN,$((${MAX_sens[pm_10]} / ${#S[@]})),IF,UNKN,IF )
                
		A+=(LINE0.1:$STACK)
	        # A+=(AREA:${levels[$level]}${P}#${color[${P},${levels[$level]}]}:"${qual[${levels[$level]}]}" )
	        A+=(AREA:${levels[$level]}${P}#${color[${P},${levels[$level]}]}:"${qual[${levels[$level]}]}:STACK" )
            done
        fi
	STACK+=$((${MAX_sens[pm_10]} / ${#S[@]}))
	RAXIS+="  ${Pname//[0-9.]/}<sub><small>${Pname//[A-Z]/}</small></sub> "

        A+=(COMMENT:" \\n" )
        A+=(COMMENT:"  metingen             " )
        A+=(PRINT:${P}:AVERAGE:"AVG(${P})=%.1lf")
        A+=(GPRINT:${P}:AVERAGE:"Gem.\: %5.1lf (max/jaar ${norm[$NORM,year,${P}]}) µg/m<sup><small>3</small></sup>")
        A+=(PRINT:${P}:MAX:"MAX(${P})=%.1lf")
        A+=(GPRINT:${P}:MAX:"Max\: %5.1lf µg/m<sup><small>3</small></sup>")
        A+=(PRINT:${P}:MIN:"MIN(${P})=%.1lf")
        A+=(GPRINT:${P}:MIN:"Min\: %5.1lf µg/m<sup><small>3</small></sup>\\n")
        A+=(COMMENT:" \\n")
    done

    # the graph lines
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'<b>stoffen in µg/m<sup><small>3</small></sup></b>      ')

    for P in ${POLs[@]}
    do
        if [ -z "${S[$P]}" ] ; then continue ; fi
	Pname=${P/_/} ; Pname=${Pname/25/2.5} ; Pname=${Pname^^}
	if [ -n "${norm[$NORM,rood,$P]}" ]
	then
            A+=(LINE1.5:${norm[$NORM,rood,$P]}#${color[$P,limit]}:"${Pname//[0-9.]/}<sub><small>${Pname//[A-Z]/}</small></sub> ${norm[$NORM,rood,$P]} µg/m<sup><small>3</small></sup> avg dag limiet":dashes=1,2)
	fi
	if [ -n "${norm[$NORM,year,$P]}" ]
	then
            A+=(LINE1.5:${norm[$NORM,year,$P]}#${color[$P,limit]}:"${Pname//[0-9.]/}<sub><small>${Pname//[A-Z]/}</small></sub> ${norm[$NORM,year,$P]} µg/m<sup><small>3</small></sup> avg 365 dagen limiet":dashes=1,1)
	fi
	# draw the line graph for the pollutant
        A+=(LINE2:${P}#E0E0E0:"")
        A+=(LINE1.5:${P}#${color[${P},line]}:"${Pname//[0-9.]/}<sub><small>${Pname//[A-Z]/}</small></sub> waarden":dashes=1,1)
        A+=(COMMENT:" \\n")
        A+=(COMMENT:'                               ')
	#EXTRA+=" ${names[$P]}(${CAL[${P}]})"
	EXTRA+=" ${names[$P]}"
    done
    if [ -n "$EXTRA" ] ; then EXTRA=":$EXTRA" ; fi

    A+=(COMMENT:" \\n")

    declare -a B
    B=("$OUTPUT")
    B+=(-w $WIDTH -h $HEIGHT -a PNG)
    B+=(--pango-markup)
    B+=(--font DEFAULT:7:Helvetica)
    B+=(--slope-mode)
    B+=(--start $START --end $END)
    B+=(--title "luchtvervuilende stoffen (${names[*]}) $LOCATION van $STRT_DATE t/m $END_DATE, $SN norm")
    B+=(--watermark "bron: $ORGANISATION meetstation $ID, $TYPE grafiek${EXTRA}, d.d. $(LC_TIME='nl_NL.UTF-8' date '+%e %b %Y %H:%M')")
    B+=(--vertical-label "stof in µg/m<sup><small>3</small></sup>")
    B+=(--lower-limit 0)
    B+=(--upper-limit ${MAX_sens[pm_10]})
    B+=(--right-axis-format %2.0lf)
    B+=(--right-axis-label "${RAXIS}")
    B+=(--x-grid $GRID)
    B+=(--alt-y-grid --rigid)

    # create the graph
    local INFO
    ARGS \
        /usr/bin/rrdtool graphv \
        "${B[@]}" \
        "${A[@]}" 2>/var/tmp/rrdtool_cmd${DEBUG}
    if [ -n "$DEBUG" ] ; then
        echo "rrdtool pollutant graph call:" 1>&2
        cat /var/tmp/rrdtool_cmd${DEBUG} 1>&2
    fi
    if [ -s /var/tmp/rrdtool_cmd${DEBUG} ]
    then
        INFO=$(/bin/sh /var/tmp/rrdtool_cmd${DEBUG})
        rm -f /var/tmp/rrdtool_cmd${DEBUG}
    else
        INFO=$(/usr/bin/rrdtool graphv "${B[@]}" "${A[@]}")
    fi

    if [ ! -f $OUTPUT ]
    then
        echo "$ERROR: error on creation of $OUTPUT by rrdtool pollutant graphv" 1>&2
        return 1
    fi
    if depends exiftool "perl Image Magic"
    then
        local COMMENT="luchtvervuiling (ruwe gegevens) grafiek norm volgens $NORM"
        if [ -n "$EXT" ]
        then
            COMMENT="luchtvervuiling (gecalibreerde gegevens) grafiek volgens norm $NORM"
	fi
	COMMENT+=", luchtvervuilende stoffen: ${names[@]}"
	COMMENT+=", periode van $DAYS voor datum $STRT_DATE t/m $END_DATE"
	COMMENT+=", norm used in color grading: ${NORM:-$DFLT_NORM}"
	if [ -n "$LOCATION" ] ; then COMMENT+=", $LOCATION, station ${NR:-06}" ; fi
        if perl -e "exit not ($N_StdDev < 5.9 && $N_StdDev > 0.68);" ||
           perl -e "exit not ($N_DayStdDev < 5.9 && $N_DayStdDev > 0.68);"
	then
	    COMMENT+=", gebruikte validatie periode: ${N_StdDev}X sigma, dag ${N_DayStdDev}X sigma."
	fi
	exiftool -q -m -overwrite_original_in_place \
    		-Title="luchtverontreiniging $LOCATION van $STRT_DATE t/m $END_DATE, $SN norm" \
		-Comment="$COMMENT" \
		-Author="$AUTHOR" \
		-Copyright="Creative Commons" \
		-Keywords=luchtverontreiniging \
		-Description="Regionale luchtvervuilende stoffen metingen" \
		-CreationTime="$(date)" \
		$OUTPUT
    fi

    echo "Info over grafiek:" 1>&2
    INFO=$(echo "$INFO" | sed -e 's/print... = //' -e 's/"//g'  -e 's/ = /=/')
    local DATUM
    DATUM=$(echo "$INFO" | grep _start)
    echo Start datum grafiek: $(date --date=@${DATUM/*=/} +%Y/%m/%d) 1>&2
    DATUM=$(echo "$INFO" | grep _end)
    echo Eind datum grafiek: $(date --date=@${DATUM/*=/} +%Y/%m/%d) 1>&2
    echo "$INFO" | grep pm_ 1>&2
    echo "OUTPUT png pollutant graph: $OUTPUT (${WIDTH}X${HEIGHT})" 1>&2

}

# create a PNG picture diagram for particul polution
# optional arg1: type of rrd file (normalized or raw)
# arg1: sensor name(s)
# arg2: (optional) period in days before NOW, default 365,
# arg3: (optional, default now): end date
# see unix date manual for date string to be used
# create diagram with MIN and MAX bandwidth for the sensor
# this routine uses norm EU2010
function sensor_graph() {
    local EXT=""
    case "$1" in
    raw)
        shift
    ;;
    twee|both)
        sensor_graph raw $2 $3 $4 $5
        sensor_graph norm $2 $3 $4 $5
        return $?
    ;;
    norm)
        shift
        EXT=$RRDEXT
    ;;
    esac
    if [ ! -f ${RRD_DIR}/${RRD}${EXT}.rrd ]
    then
	echo "$ERROR Bestand ${RRD_DIR}/${RRD}${EXT}.rrd bestaat niet"
	echo "Maak het eerst aan, zie command help voor details"
	return 1
    fi

    set_std_limits	# defines the norm[EU2010,rood|year,sensor] values
    # graph shows a "red" band based on max year and max day avg
    # EU2010 is second phase norms, 7th Environment Pollution EU Programme
    norm[EU2010,max1,pm_10]=${norm[EU2010,year,pm_10]}
    norm[EU2010,max2,pm_10]=${norm[EU2010,rood,pm_10]}
    norm[EU2010,max1,pm_25]=$((${norm[EU2010,year,pm_25]} - 5))
    norm[EU2010,max2,pm_25]=${norm[EU2010,year,pm_25]}
    norm[EU2010,max1,roet]=$((${norm[EU2010,year,pm_25]} - 5))
    norm[EU2010,max2,roet]=${norm[EU2010,year,pm_25]}
    norm[EU2010,max1,no2]=40	# bron ruimtelijkplannen.nl
    norm[EU2010,max2,no2]=60	# bron ruimtelijkplannen.nl
    norm[EU2010,max1,nox]=30	# bron ruimtelijkplannen.nl
    norm[EU2010,max2,nox]=30	# bron ruimtelijkplannen.nl

    local SENSORS="$1" ; shift
    local DAYS=$1 ; shift
    local NOW="$1"
    if [ -z "$NOW" ] ; then NOW=now; fi
    if [ -z "$DAYS" ]; then DAYS=365; fi
 
    # time span yesterday
    # seconds from zero day today morning seconds after 12
    NOW=`date  --date="$NOW" | sed 's/[0-9]*:/0:/g'`
    if [ $? != 0 ]; then return 1; fi
    NOW=`date --date="$NOW" +%s`
    local GRID
    if [ "$DAYS" -ge 90 ]
    then
        GRID=WEEK:1:MONTH:1:MONTH:1:86400:%b/%y
	if [ "$WIDTH" -le 500  -o $DAYS -ge 200 ] ; then GRID=WEEK:1:MONTH:1:MONTH:3:172800:%b/%y ; fi
    else
        GRID=DAY:1:WEEK:1:WEEK:1:86400:%d/%m
	if [ "$WIDTH" -le 500 ] ; then GRID=DAY:1:WEEK:2:MONTH:1:86400:%m ; fi
    fi
    local START=$(($NOW - $DAYS*24*60*60))
    local END=$NOW
    local STRT_DATE=$(date --date=@$START "+%F")
    local END_DATE=$(date --date=@$END "+%F")

    # colors of the graphs
    declare -A C
    C[roet]=b50650
    C[pm_10]=ffa200
    C[pm_25]=8a00ff
    C[no2]=78ff00
    C[no]=002aff
    local S
    declare -a MYSENS
    declare -i LIMIT=0
    for S in $SENSORS
    do
        if echo "$(Get_sensors ${TBL})" | grep -q "$S" &&
           rrdtool info ${RRD_DIR}/${RRD}${EXT}.rrd | grep -q "${S}..index"
        then
            local Mcnt=$($MYSQL \
                 -e "
                    SELECT count(*) FROM ${TBL}
                      WHERE NOT isnull($S) AND
		      ${S}_valid AND UNIX_TIMESTAMP(datum) <= $END AND
		      UNIX_TIMESTAMP(datum) >= $START
                ")
	        if [ -z "$Mcnt" -o "$Mcnt" -lt 10  ]
	        then	# only a graph for more as 10 valid measurements in this period
	            echo "$ERROR: sensor \"$S\" in table ${TBL} has not enough values." 1>&2
		else
		    MYSENS+=("${S}")
		    declare -i L
		    L=$($MYSQL -e "
                          SELECT round(max(${S}))
                            FROM ${TBL}
                            WHERE NOT isnull($S) AND
                            ${S}_valid AND
                            UNIX_TIMESTAMP(datum) <= $END AND
                            UNIX_TIMESTAMP(datum) >= $START
                        ")
		    if [ $L -gt $LIMIT ] ; then LIMIT=$L ; fi
	        fi
        else
	    echo "$ERROR: sensor \"$S\" does not exists in table ${TBL}." 1>&2
        fi
    done
    if [ ${#MYSENS[@]} = 0 ]
    then
	echo "$ERROR: found no sensor values for the  graphs for this period." 1>&2
	return 1
    fi
    mkdir -p "${DIAGRAMS}"
    OUTPUT="${DIAGRAMS}/sensors_graph-station_${NR:-06}-$DAYS-$END_DATE.png"
    OUTPUT=${OUTPUT//\/.\//\/}
    ## Graph for $DAYS
    depends rrdtool "" 1
    declare -a A
    local SS SSS TSS TS
    A+=(COMMENT:"<b><i>legenda</i></b>\:\\n" COMMENT:" \\n")
    for S in ${MYSENS[@]}
    do
	if [ -z "${C[$S]}" ]
	then
	    echo "$WARNING: color for sensor $S not defined, add it!" 1>&2
	    C[$S]=${C[roet]}
	fi
	SS="$S" ; SSS="$S"
	case $SS in
	pm_10)
	    SS="PM<sub><small>10</small></sub> µg/m<sup><small>3</small></sup>"
	    SSS="PM<sub><small>10</small></sub>"
	;;
	pm_25)
	    SS="PM<sub><small>2.5</small></sub> µg/m<sup><small>3</small></sup>"
	    SSS="PM<sub><small>2.5</small></sub>"
	;;
	roet)
	    SS="roet µg/m<sup><small>3</small></sup>"
	;;
	no2)
	    SS="stikstof dioxide µg/m<sup><small>3</small></sup>"
	;;
	nox)
	    SS="stikstof oxides µg/m<sup><small>3</small></sup>"
	;;
	esac
	if [ -n "$TSS" ] ; then TSS+=", $SSS" ; else TSS="$SSS" ; fi
	if [ -n "$TS" ] ; then TS+=", $SS" ; else TS="$SS" ; fi
	SS=$(printf "%10s" "${SS}")
        A+=(COMMENT:"<b>sensor ${SS}</b>")
        A+=(DEF:${S}Avg=${RRD_DIR}/${RRD}${EXT}.rrd:${S}:AVERAGE)
        A+=(DEF:${S}Min=${RRD_DIR}/${RRD}${EXT}.rrd:${S}:MIN)
        A+=(DEF:${S}Max=${RRD_DIR}/${RRD}${EXT}.rrd:${S}:MAX)

	if [ -n "${norm[EU2010,max1,${S}]}" ]
	then
	    A+=(CDEF:${S}max1=${norm[EU2010,max1,${S}]},${S}Min,POP)
	    A+=(LINE0.5:${S}max1#${C[$S]}99:"Year max AVG ${norm[EU2010,max1,${S}]} µg/m<sup><small>3</small></sup>")
	    if [ -n "${norm[EU2010,max2,${S}]}" ]
	    then
		A+=(CDEF:${S}max2=${norm[EU2010,max2,${S}]},${S}Min,POP)
		A+=(CDEF:${S}stack=${S}max2,${S}max1,-)
        	A+=(AREA:${S}stack#${C[$S]}0F::STACK)
	        A+=(LINE0.5:${S}max2#${C[$S]}BB:"and DAY max AVG ${norm[EU2010,max2,${S}]} µg/m<sup><small>3</small></sup>")
	    fi
	fi

        A+=(CDEF:${S}Range=${S}Max,${S}Min,-)
        A+=(LINE1:${S}Min:)
        A+=(AREA:${S}Range#${C[$S]}33:"Range":STACK)
        A+=(LINE1:${S}Min#${C[$S]}66:"Min")
        A+=(LINE1:${S}Max#${C[$S]}66:"Max")
        A+=(LINE1.5:${S}Avg#${C[$S]}:"Average\\n")
        A+=(LINE0.5:${S}Avg#46202099:)
        A+=(COMMENT:" \\n")
    done

    declare -a B
    B=("$OUTPUT")
    B+=(-w $WIDTH -h $HEIGHT -a PNG)
    B+=(--pango-markup)
    B+=(--font DEFAULT:7:Helvetica)
    B+=(--slope-mode)
    B+=(--start $START --end $END)
    B+=(--title "sensor ${TSS} gemiddelden/min/max grafiek voor $LOCATION van $STRT_DATE t/m $END_DATE")
    B+=(--watermark "bron: $ORGANISATION meetstation $ID, grafiek d.d. $(LC_TIME='nl_NL.UTF-8' date '+%e %b %Y %H:%M')")
    B+=(--vertical-label "sensor(s): ${TS}")
    B+=(-M --lower-limit 0)
    B+=(--upper-limit $((($LIMIT+5)*105/100)))
    B+=(--right-axis-format %2.0lf)
    B+=(--x-grid $GRID)
    B+=(-c 'MGRID#A20000AA')
    B+=(--alt-y-grid --rigid)

    # create the graph
    local INFO
    ARGS \
        /usr/bin/rrdtool graph \
        "${B[@]}" \
        "${A[@]}" 2>/var/tmp/rrdtool_cmd${DEBUG}
    if [ -n "$DEBUG" ] ; then
        echo "rrdtool sensor graph call:" 1>&2
        cat /var/tmp/rrdtool_cmd${DEBUG} 1>&2
    fi
    if [ -f /var/tmp/rrdtool_cmd${DEBUG} ]
    then
        INFO=$(/bin/sh /var/tmp/rrdtool_cmd${DEBUG})
        rm -f /var/tmp/rrdtool_cmd${DEBUG}
    else
        INFO=$(/usr/bin/rrdtool graph "${B[@]}" "${A[@]}")
    fi

    if [ ! -f $OUTPUT ]
    then
        echo "$ERROR: error on creation of $OUTPUT by rrdtool sensor graph" 1>&2
        return 1
    fi
    if depends exiftool "perl Image Magic"
    then
        local COMMENT="average/min/max sensor ${SENSORS} values graph"
	COMMENT+=" period of $DAYS from date $STRT_DATE till $END_DATE"
	COMMENT+=", Norm used in color grading: ${NORM:-EU2010}"
	if [ -n "$LOCATION" ] ; then COMMENT+=", $LOCATION, $ORGANISATION station $ID" ; fi
	exiftool -q -m -overwrite_original_in_place \
    		-Title="average/min/max sensor ${SENSORS} of $LOCATION period: $STRT_DATE till $END_DATE" \
		-Comment="$COMMENT" \
		-Author="$AUTHOR, $ASSOCIATION" \
		-Copyright="Creative Commons" \
		-Keywords=luchtverontreiniging \
		-Description="Regional pollution ${SENSORS} measurements" \
		-CreationTime="$(date)" \
		$OUTPUT
    fi

    echo "OUTPUT png sensor graph: $OUTPUT (${WIDTH}X${HEIGHT})" 1>&2

}

# sensor_graph "pm_10 pm_25 roet" 180

# create luchtverontreiniging PM average diagram for limit standard (arg 3) default EU2005
# defaults: 365 days, both graph pollutions, EU2005 limits, "now" end date
# eg fijnstof_avg 365 PM25 EU2005
# eg fijnstof_avg 365 PM10 EU2010
# eg fijnstof_avg 365 PM10 EU2010 2013/8/24
# eg fijnstof_avg 365 both EU2005

# TAKE care! with luchtkwaliteit_dayly database rrdtool will give core dump
# maybe wrong RRD initialisation?
# examples of the call
#fijnstof_avg 365 PM25 EU2005 2013/8/24
#fijnstof_avg luchtkwaliteit 365 PM10 EU2005
#fijnstof_avg luchtkwaliteit_dayly 365 both EU2005
# called as: fijnstof_avg [RRD file] [ days [PM [norm]]]
function fijnstof_avg() {
    local EXT=""
    case "$1" in
    raw)
	shift
    ;;
    twee|beide*|both)
	fijnstof_avg raw $2 $3 $4 $5
	fijnstof_avg norm $2 $3 $4 $5
	return $?
    ;;
    norm)
	shift
	EXT=$RRDEXT
    ;;
    esac
    set_std_limits
    local RRD_file
    if echo "$1" | grep -q `basename -s .rrd "${RRD}"` 
    then
	RRD_file=$1; shift
    else
	RRD_file=${RRD}${EXT}
    fi
    RRD_file=`echo "${RRD_file}" | sed 's/\.rrd$//'`
    if [ ! -f ${RRD_DIR}/"${RRD_file}.rrd" ]
    then
	echo "$ERROR: ${RRD_DIR}/${RRD_file}.rrd not existent." 1>&2
	return 1
    fi
    local DAYS=$1 ; shift
    if [ -z "$DAYS" ]; then DAYS=365; fi
    local PM=$1 ; shift
    PM=`echo "$PM" | sed 's/[^0-9]//g'`
    if [ -z "$PM" ]; then PM=both; fi
    if ! echo "$(Get_sensors $TBL)" | grep -q pm_10
    then
	if ! echo "$(Get_sensors $TBL)" | grep -q pm_25
	then
	    echo "$WARNING Cannot make graph as no PM10 and/or PM2.5 is present." 1>&2
	    return
	else
	    PM=25
	fi
    elif ! echo "$(Get_sensors $TBL)" | grep -q pm_25
    then
	PM=10
    fi
    if [ "$PM" != 10 -a "$PM" != 25 -a "$PM" != both ]
    then
	echo "Cannot handle PM $PM value." 1>&2
	return 1
    fi
    local NORM=$1; shift
    local NOW="$1"
    if [ -z "$NOW" ] ; then NOW=now; fi
    if [ -z "$NORM" ]; then NORM=EU2005; fi
    local NOTE1=""
    local NOTE2=""
    local NOTE4=""
    local NOTE5=""
    local DCMNT=""
    local -A PMtype PMres  # extentions in DEF name and DEF value resolutions
    if echo "${RRD_file}" | grep -q dayly
    then	# we have the dayly RRD and use different names
        PMtype[avg]="_avg" ; PMtype[min]="_min" ; PMtype[max]="_max"
        PMres[avg]=MAX ; PMres[min]=MAX ; PMres[max]=MAX;
    else	# we have the standard RRD file
        PMtype[avg]="" ; PMtype[min]="" ; PMtype[max]=""
        PMres[avg]=AVERAGE ; PMres[min]=MIN ; PMres[max]=MAX;
    fi
 
    # time span yesterday
    # seconds from zero day today morning seconds after 12
    local NOW=`date  --date="$NOW" | sed 's/[0-9]*:/0:/g'`
    if [ $? != 0 ]; then return 1; fi
    NOW=`date --date="$NOW" +%s`
    local GRID
    if [ $DAYS -ge 90 ]
    then
        #GRID=WEEK:1:MONTH:1:MONTH:1:172800:%b/%y
        GRID=WEEK:1:MONTH:1:MONTH:1:0:%b/%y
	if [ "$WIDTH" -le 500  -o $DAYS -ge 500 ] ; then GRID=WEEK:1:MONTH:1:MONTH:1:172800:%b ; fi
    else
        GRID=HOUR:12:DAY:1:DAY:2:86400:%d/%m
	if [ "$WIDTH" -le 500 ] ; then GRID=HOUR:6:DAY:1:DAY:1:86400:%m ; fi
    fi
    local START=$(($NOW - $DAYS*24*60*60))
    local END=$NOW
    local STRT_DATE=`date --date=@$START "+%F"`
    local END_DATE=`date --date=@$END "+%F"`
    # the limit used in this graph
    case $NORM in
    site2|site|EU2005|EU2010|WHO)
	echo "Using limits from $NORM for PM graph" 1>&2
    ;;
    *)
	echo "$ERROR: Unkown limit standard" 1>&2
	return
    ;;
    esac

    local -A color
    local -A TR		# transparency
    TR[pm_10]=DD
    TR[pm_25]=DD
    TR[both]=00
    if [ "$PM" = both ]
    then
	TR[both]=${TR[pm_10]}
    fi

    local -A groen geel oranje rood black
    # std limits used in the graph
    for indx in pm_10 pm_25  both
    do
        # HTML coloring PM 10
        case "$indx" in
        both)
            #color[both,white]=E0E0E0
            color[both,white]=FFFFFF${TR[both]}
            color[both,black]=C0C0C0${TR[both]}
            color[both,groen]=A5F1A5${TR[both]}
            color[both,geel]=F8F79F${TR[both]}
            color[both,oranje]=FBD79F${TR[both]}
            color[both,rood]=6A1100${TR[both]}		# 999999
            color[both,limit]=7B1464${TR[both]}
            color[both,line]=0A0A0A${TR[both]}
            color[both,area]=FFFF00${TR[both]}
        ;;
        pm_10)
            #color[pm_10,white]=E0E0E0
            color[pm_10,white]=FFFFFF
            color[pm_10,black]=C0C0C0${TR[pm_10]}
            color[pm_10,groen]=A5F1A5${TR[pm_10]}
            color[pm_10,geel]=F8F700${TR[pm_10]}     #  F8F79F${TR[pm_10]}
            color[pm_10,oranje]=FBD79F${TR[pm_10]}
            color[pm_10,rood]=999999
            color[pm_10,limit]=7B1464
            color[pm_10,line]=0A0A0A
            color[pm_10,area]=FFFF00${TR[pm_10]}
        ;;
        # HTML coloring PM 2.5
        pm_25|*)
            color[pm_25,white]=FFFFFF
            color[pm_25,groen]=00EE00${TR[pm_25]}
            color[pm_25,geel]=FFFF00${TR[pm_25]}
            color[pm_25,oranje]=FFA200${TR[pm_25]}
            color[pm_25,rood]=FF0000${TR[pm_25]}
            color[pm_25,black]=D0D0D0${TR[pm_25]}	# not used
            color[pm_25,limit]=7C0404
            color[pm_25,line]=FF0000
            color[pm_25,area]=FFA200${TR[pm_25]}
        ;;
        esac
        Level[groen,$indx]=${norm[$NORM,groen,$indx]}
        Level[geel,$indx]=${norm[$NORM,geel,$indx]}
        Level[oranje,$indx]=${norm[$NORM,oranje,$indx]}
        Level[rood,$indx]=${norm[$NORM,rood,$indx]}
        Level[black,$indx]=${norm[$NORM,black,$indx]} # PM max value
    done

    # here we introduce the artificial correction factors for the graph
    local PM25_p=`echo "${CAL_n[pm_25]}" | sed s/-//`
    local PM10_p=`echo "${CAL_n[pm_10]}" | sed s/-//`
    local PM25_a="+" PM10_a="+"
    if echo "${CAL_n[pm_25]}" | grep -q -e '-' ; then PM25_a="-" ; fi
    if echo "${CAL_n[pm_25]}" | grep -q -e '-' ; then PM10_a="-" ; fi

    local OUTPUT
    if [ -z "$EXT" ]
    then
       OUTPUT=${DIAGRAMS}/${RRD_file}-avg_fijnstof-station_${NR:-06}-raw-${NORM}-PM${PM}_graph-${DAYS}-$END_DATE.png
    else
       OUTPUT=${DIAGRAMS}/${RRD_file}-avg_fijnstof-station_${NR:-06}-norm-${NORM}-PM${PM}_graph-${DAYS}-$END_DATE.png
    fi
    OUTPUT=${OUTPUT//\/.\//\/}
    local PMcnt=`echo "$PM" | sed 's/25/2.5/'`
    local TITLE="PM<sub><small>$PMcnt</small></sub>"
    if [ "$PM" = both ]
    then
	PM=25 ; TR[both]=${TR[pm_10]} ; PMcnt=2.5
	TITLE="PM<sub><small>2.5</small></sub> en PM<sub><small>10</small></sub>"
        NOTE1="PM<sub><small>10</small></sub> dag bandbreedte"
	NOTE2="PM<sub><small>10</small></sub> ${Level[rood,pm_10]} µg/m<sup><small>3</small></sup> $NORM daggem. norm"
        NOTE4="PM<sub><small>10</small></sub> µg/m<sup><small>3</small></sup> daggemiddelde"
	NOTE5="PM<sub><small>10</small></sub> ${norm[$NORM,year,pm_10]} µg/m<sup><small>3</small></sup> $NORM jaargem. norm"
    fi
    local TYPE="(ruwe data)"
    if [ -n "$EXT" ]
    then
	TYPE="(genormaliseerde data)"
    fi
    ## Graph for $DAYS
    local SN=$NORM
    if [ "$NORM" = site ] ; then SN=$SITE ; fi
    if [ "$NORM" = site2 ] ; then SN=$SITE2 ; fi
    if [ -n "$AQIon" ] ; then SN="AQI index" ; fi
    depends rrdtool "" 1
    declare -a A
    A+=(DEF:both_avg_pm_10i=${RRD_DIR}/${RRD_file}.rrd:pm_10${PMtype[avg]}:${PMres[avg]})
    A+=(CDEF:both_avg_pm_10=both_avg_pm_10i,${CAL_f[pm_10]},*,${PM10_p},${PM10_a})
    A+=(DEF:both_min_pm_10i=${RRD_DIR}/${RRD_file}.rrd:pm_10${PMtype[min]}:${PMres[min]})
    A+=(CDEF:both_min_pm_10=both_min_pm_10i,${CAL_f[pm_10]},*,${PM10_p},${PM10_a})
    A+=(DEF:both_max_pm_10i=${RRD_DIR}/${RRD_file}.rrd:pm_10${PMtype[max]}:${PMres[max]})
    A+=(CDEF:both_max_pm_10=both_max_pm_10i,${CAL_f[pm_10]},*,${PM10_p},${PM10_a})
    A+=(COMMENT:"<b><i>legenda</i></b>\:\\n")
    A+=(COMMENT:' \n')
    A+=(COMMENT:'dagwaarden                 ')
    A+=(AREA:both_max_pm_10#${color[both,area]}:"$NOTE1")
    A+=(AREA:both_min_pm_10#${color[both,white]}:)
    A+=(LINE1.5:both_max_pm_10#${color[both,oranje]}:)
    A+=(LINE1.5:both_min_pm_10#${color[both,oranje]}:)
    A+=(DEF:avg_pm_${PM}i=${RRD_DIR}/${RRD_file}.rrd:pm_$PM${PMtype[avg]}:${PMres[avg]})
    A+=(CDEF:avg_pm_${PM}=avg_pm_${PM}i,${CAL_f[pm_10]},*,${PM10_p},${PM10_a})
    A+=(DEF:min_pm_${PM}i=${RRD_DIR}/${RRD_file}.rrd:pm_$PM${PMtype[min]}:${PMres[min]})
    A+=(CDEF:min_pm_${PM}=min_pm_${PM}i,${CAL_f[pm_10]},*,${PM10_p},${PM10_a})
    A+=(DEF:max_pm_${PM}i=${RRD_DIR}/${RRD_file}.rrd:pm_$PM${PMtype[max]}:${PMres[max]})
    A+=(CDEF:max_pm_${PM}=max_pm_${PM}i,${CAL_f[pm_10]},*,${PM10_p},${PM10_a})
    A+=(AREA:max_pm_$PM#${color[pm_$PM,area]}:"PM<sub><small>$PMcnt</small></sub> dag bandbreedte")
    A+=(AREA:min_pm_$PM#${color[pm_$PM,white]}:)
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'                                     ')
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'daggemiddelde limieten' )
    A+=(LINE1.5:${Level[rood,pm_10]}#${color[both,limit]}:"$NOTE2":dashes=1,2)
    A+=(LINE1.5:${Level[rood,pm_$PM]}#${color[pm_$PM,limit]}:"PM<sub><small>$PMcnt</small></sub> ${Level[rood,pm_10]} µg/m<sup><small>3</small></sup> $NORM daggem. norm":dashes=1,2)
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'                                     ')
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'                                     ')
    A+=(LINE1.5:max_pm_$PM#${color[pm_$PM,oranje]}:)
    A+=(LINE1.5:both_avg_pm_10#${color[both,rood]}:"$NOTE4")
    A+=(LINE1.5:avg_pm_$PM#${color[pm_$PM,rood]}:"PM<sub><small>$PMcnt</small></sub> µg/m<sup><small>3</small></sup> daggemiddelde")
    A+=(LINE1.5:min_pm_$PM#${color[pm_$PM,oranje]}:"PM<sub><small>$PMcnt</small></sub> waarden")
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'                                     ')
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'jaargemiddelde limieten (stippellijnen)' )
    A+=(LINE1.5:${norm[$NORM,year,pm_10]}#${color[both,limit]}:"$NOTE5":dashes=1,4)
    A+=(LINE1.5:${norm[$NORM,year,pm_$PM]}#${color[pm_$PM,limit]}:"PM<sub><small>$PMcnt</small></sub> ${norm[$NORM,year,pm_10]} µg/m<sup><small>3</small></sup> $NORM jaargem. norm":dashes=1,4)
    A+=(COMMENT:" \\n")
    A+=(COMMENT:'                                                           ')
    A+=(LINE1.5:${norm[WHO,year,pm_10]}#${color[pm_10,limit]}:"PM<sub><small>10</small></sub> ${norm[WHO,year,pm_10]} µg/m<sup><small>3</small></sup> WHO jaargem. norm     ":dashes=1,6)
    A+=(LINE1.5:${norm[WHO,year,pm_25]}#${color[pm_25,limit]}:"PM<sub><small>2.5</small></sub> ${norm[WHO,year,pm_25]} µg/m<sup><small>3</small></sup> WHO jaargem. norm":dashes=1,6)
    A+=(COMMENT:" \\n")

    B=($OUTPUT)
    B+=(-w $WIDTH -h $HEIGHT -a PNG)
    B+=(--pango-markup)
    B+=(--font DEFAULT:8:Helvetica)
    B+=(--slope-mode)
    B+=(--start $START --end $END)
    B+=(--title "gemiddelde $TITLE luchtverontreiniging $LOCATION van $STRT_DATE t/m $END_DATE, $SN norm")
    B+=(--watermark "$TYPE grafiek PM2.5(${CAL[pm_25]}) PM10(${CAL[pm_10]}), d.d. $(LC_TIME='nl_NL.UTF-8' date '+%e %b %Y %H:%M'), bron: http://www.$SITE2 $ORGANISATION meetstation $ID")
    B+=(--vertical-label "luchtverontreiniging $TITLE µg/m<sup><small>3</small></sup>")
    B+=(--lower-limit 0)
    B+=(--upper-limit ${Level[black,pm_$PM]})
    B+=(--right-axis-format %2.0lf)
    B+=(--x-grid $GRID)
    B+=(--alt-y-grid --rigid)

    # create the graph
    local INFO
    ARGS \
        /usr/bin/rrdtool graph \
        "${B[@]}" \
        "${A[@]}" 2>/var/tmp/rrdtool_cmd${DEBUG}
    if [ -n "$DEBUG" ] ; then
        echo "rrdtool fijnstof graph call:" 1>&2
        cat /var/tmp/rrdtool_cmd${DEBUG} 1>&2
    fi
    if [ -f /var/tmp/rrdtool_cmd${DEBUG} ]
    then
        INFO=$(/bin/sh /var/tmp/rrdtool_cmd${DEBUG})
        rm -f /var/tmp/rrdtool_cmd${DEBUG}
    else
        INFO=$(/usr/bin/rrdtool graph "${B[@]}" "${A[@]}")
    fi

    if [ ! -f $OUTPUT ]
    then
        echo "$ERROR: error on creation of $OUTPUT by rrdtool fijnstof graph" 1>&2
        return 1
    fi
    if depends exiftool "perl Image Magic"
    then
        local COMMENT="luchtverontreiniging (ruwe gegevens) daggemiddelden grafiekvolgens norm $NORM"
        if [ -n "$EXT" ]
        then
            COMMENT="luchtverontreiniging (gecalibreerde gegevens) daggemiddelden grafiek volgens norm $NORM"
	fi
	COMMENT+=" periode van $DAYS voor datum $STRT_DATE t/m $END_DATE"
	if [ -n "$LOCATION" ] ; then COMMENT+=", $LOCATION, station ${NR:-06}" ; fi
        if perl -e "exit not ($N_StdDev < 5.9 && $N_StdDev > 0.68);" ||
           perl -e "exit not ($N_DayStdDev < 5.9 && $N_DayStdDev > 0.68);"
	then
	    COMMENT+=", gebruikte validatie periode: ${N_StdDev}X sigma, dag ${N_DayStdDev}X sigma."
	fi
	COMMENT+=", Norm used in color grading: ${NORM:-$DFLT_NORM}"
	exiftool -q -m -overwrite_original_in_place \
    		-Title="gemiddelde $TITLE luchtverontreiniging $LOCATION van $STRT_DATE t/m $END_DATE, $SN norm" \
		-Comment="$COMMENT" \
		-Author="$AUTHOR" \
		-Copyright="Creative Commons" \
		-Keywords=luchtverontreiniging \
		-Description="Regionale luchtverontreiniging metingen" \
		-CreationTime="$(date)" \
		$OUTPUT
    fi

    echo "OUTPUT png fijnstof graph: $OUTPUT (${WIDTH}X${HEIGHT})" 1>&2

}

# some prework for histogram, extract some max values
function histo_spread() {
    local SENS  QRY1 QRY2
    local TTBL=$1 ; if [ -z "$1" ]; then return 1; fi
    local PERIOD=$2	# period 0 everything (dflt), N days, jaar
    local SS=$3		# sensors
    local VALID=$4	# use only valid measurements
    local QSTART=${5/START=/} QEND=${6/END=/} # only from start date - end date
    if [ -n  "Q$START" ]
    then QSTART=" AND datum >= '${QSTART}' "
    fi
    if [ -z "$QEND" ]
    then  QEND="now()"
    else
	QEND="'${QEND}'"
	QSTART+=" AND datum <= ${QEND}"
    fi
    case "$PERIOD" in
    0)
		QRY1="'1'" ; QRY2=""
        ;;
    [1-9]*)
		QRY1="datediff(${QEND},datum) DIV $PERIOD + 1"
                QRY2="datediff(${QEND},datum) DIV $PERIOD,"
        ;;
    JAAR|jaar|kalender|calendar)
                QRY1="year(${QEND}) - year(datum) + 1"
                QRY2="year(datum),"
        ;;
    *)
		QRY1="'1'" ; QRY2=""
        ;;
    esac
    for SENS in $SS
    do
	echo "SELECT '$SENS' as sensor, round($SENS) AS value,
	      count(*) AS number, $QRY1 AS periode
              FROM $TTBL WHERE not isnull(${SENS}) ${QSTART}"
	if [ -n "$VALID" ] ; then echo -e "\tAND ${SENS}_valid" ; fi
	echo -e "\tGROUP BY $QRY2 round($SENS) order by periode, value;"
    done \
    | $MYSQL \
    | perl -e '
	my %xmax = (); %ymax = (); my %pmax = ();
	my $sens = "x"; my @snrs = (); 
	while ( <STDIN> ) {
	    next if not $_;
	    my @ln = split /\s+/;
	    next if $#ln != 3;
	    if ( $sens ne $ln[0] ) {
		$sens = $ln[0];
		$xmax{$sens} = 0; $ymax{$sens} = 0; $pmax{$sens} = 0;
		push @snrs, $sens;
	    }
	    $xmax{$sens} = $ln[1] if $ln[1] > $xmax{$sens} && $ln[2] > 2;
	    $ymax{$sens} = $ln[2] if $ln[2] > $ymax{$sens};
	    $pmax{$sens} = $ln[3] if $ln[3] > $pmax{$sens};
	}
	$sens = "";
	for ( my $i = 0; $i <= $#snrs; $i++) {
	   $sens .= sprintf("%s:xmax=%d:ymax=%d:period=%d,",
		$snrs[$i],
		$xmax{$snrs[$i]}, $ymax{$snrs[$i]}, $pmax{$snrs[$i]});
	}
	$sens =~ s/,$//; print "$sens\n";
    '
    return $?
}

# create a PNG image with graph (GNUPlot) freq of values, in periods
# takes argument 0 as table name, arg 1 as setting: PERIOD=, ...
function histogram() {
    local SENS TTBL=$1 ; if [ -z "$1" ]; then return 1; fi
    local NRM EU
    
    local MAX DONE=0
    local PERIOD=0	# 0 denotes no periods, use all values in one group
    local VALID		# if set only use valid measurements (dflt: no)
    local LS S SS=$(Get_sensors $TTBL no_climate)
    local QSTART QEND STRT_DATE END_DATE
    if ! depends gnuplot "GNU graph plotting tooling" ; then return 1; fi
    while [ -n "$2" ]
    do
	case "${2^^}" in
	PERIOD*)
	    PERIOD=$(echo "${2^^}" | sed 's/.*=//')
	;;
	SENS*)
	    S=$(echo "$2" | sed 's/.*=//')
	    for SENS in $S
	    do
		if echo "$SS" | grep -q "$SENS"
		then
		    LS+=" $SENS"
		fi
	    done
	    SS=$LS
	;;
	VALID)
	   VALID=VALID
	   Set_Sensor_valids $TTBL no_report
	;;
	START*)
	    QSTART=${2/START=/} ; STRT_DATE=${QSTART}
        ;;
	END*)
	    QEND=${2/END=} ; END_DATE=${QEND}
	;;
	esac
	shift
    done
    if [ -z "$STRT_DATE" ]
    then
	DB_pass
	STRT_DATE=$($MYSQL -e "
                SELECT date_format(datum, '%Y/%m/%d') FROM $TTBL LIMIT 1
                              ")
    fi
    if [ -z "$END_DATE" ]
    then
	DB_pass
	END_DATE=$($MYSQL -e "
                SELECT date_format(datum, '%Y/%m/%d')
                  FROM $TTBL ORDER BY datum DESC LIMIT 1
                            ")
    fi
    set_std_limits
    for SENS in $SS
    do
	if echo "$SENS" | grep -q "pm_"
	then 
	    for NM in EU2010 EU2005 WHO
	    do
	        if [ -n  "${norm[$NM,year,${SENS}]}" ]
	        then
		    if ! echo "$NRM" | grep -q "$NM=${norm[$NM,year,${SENS}]}.$SENS"
		    then
		        NRM+=",'$NM=${norm[$NM,year,${SENS}]}_$SENS'"
		    fi
	        fi
	    done
	fi
    done
    if [ -n  "$QSTART" ]
    then QSTART=" AND datum >= '${QSTART}' "
    fi
    if [ -z "$QEND" ]
    then  QEND="now()"
    else
	QEND="'${QEND}'"
	QSTART+=" AND datum <= '${QEND}' "
    fi
    local QRY1 QRY2
    case "$PERIOD" in
    0)
		QRY1="'1'" ; QRY2=""
	;;
    [1-9]*)
		QRY1="datediff(${QEND},datum) DIV $PERIOD + 1"
		QRY2="datediff(${QEND},datum) DIV $PERIOD,"
	;;
    JAAR|jaar|kalender|calendar)
		QRY1="year(${QEND}) - year(datum) + 1"
		QRY2="year(datum),"
	;;
    *)
		QRY1="'1'" ; QRY2="" ; PERIOD=0
	;;
    esac
    local OUTPUT=${DIAGRAMS}/histogram-station_${NR:-06}-periods_$PERIOD
    OUTPUT=${OUTPUT//\/.\//\/}
    if [ -n "$VALID" ] ; then OUTPUT+="-validated" ; fi
    # get max values for later processing
    NRM=$(echo "$NRM" | sed -e 's/,,/,/g' -e 's/^,//')
    # histo_spread is sensible to argument order
    MAX=$(histo_spread $TTBL "$PERIOD"  "$SS" "$VALID" "${STRT_DATE}" "${END_DATE}")
    if [ -z "$MAX" -o -z "$SS" ]
    then
	echo "$WARNING No measurements values found." 1>&2
        return 1
    fi
    if [ -n "$VALID" ] ; then VALID="gevalideerde " ; fi
    for SENS in $SS
    do
	if [ "$DONE" = 0 ]
	then
	    echo "SELECT '$MAX';"	# make sure to keep the list sorted
	    DONE=1
	fi
	echo "
	    SELECT '$SENS' as sensor, round($SENS) AS value,
	      count(*) AS number, $QRY1 AS periode
	      FROM $TTBL WHERE not isnull(${SENS}) ${QSTART}"
	if [ -n "$VALID" ] ; then echo -e "\tAND ${SENS}_valid" ; fi
	echo -e "\tGROUP BY $QRY2 round($SENS) order by periode, value;"
    done \
    | $MYSQL \
    | perl \
	-e "my \$FILE='$OUTPUT'; my \$period = '$PERIOD'; my \$valid='$VALID';" \
	-e "my @norm = ($NRM); my \$place = '$LOCATION';" \
	-e "my \$strt_date = '$STRT_DATE'; my \$end_date = '$END_DATE';" \
	-e '
	use POSIX qw(strftime);
        my $sens = "" ; my $indx = -1; %snrs = (); my $defs = 0;
	my $xmax = 0; my $ymax = 0; my %yscale = (); my @snrs = ();
	my $per = 0; my $pmax = 1; my %pscale = ();
        sub hsv2rgb {	# need this for similar colors for each period
            # H 0 .. 360, S 0 .. 1, V 0 ..1
	    my $H = shift; my $S = shift; my $V = shift;
            my $R; my $G; my $B; $H %= 360; $H /= 360;
            if ( $S == 0 ) {
               $R = int($V*255+0.5); $G = int($V*255+0.5); $B = int($V*255 +0.5);
            }
            else {
               $H = $H * 6; if ( $H == 6 ) { $H = 0 ; }      #H must be < 1
               my $v1 = $V * ( 1 - $S );
               my $v2 = $V * ( 1 - $S * (       $H - int($H) ) );
               my $v3 = $V * ( 1 - $S * ( 1 - ( $H - int($H) ) ) );

               if    ( int($H) == 0 ) { $R = $V  ; $G = $v3 ; $B = $v1 ;}
               elsif ( int($H) == 1 ) { $R = $v2 ; $G = $V  ; $B = $v1 ;}
               elsif ( int($H) == 2 ) { $R = $v1 ; $G = $V  ; $B = $v3 ;}
               elsif ( int($H) == 3 ) { $R = $v1 ; $G = $v2 ; $B = $V  ;}
               elsif ( int($H) == 4 ) { $R = $v3 ; $G = $v1 ; $B = $V  ;}
               else                   { $R = $V  ; $G = $v1 ; $B = $v2 ;}
	                 #RGB integer results from 0 to 255
               $R = int($R*255+0.5); $G = int($G*255+0.5); $B = int($B*255+0.5);
            }
	    return sprintf("%0.2X%0.2X%0.2X",$R,$G,$B);
        }
	sub palt { my $indx = shift; return $indx*41+205; }

        while( <STDIN> ) { chomp;
            next if not $_;
	    if( /([xy]max|period)=/ ) {	# got sensor max definition
		$defs++;
		my @def = split /,/;	# order of array is important
		for ( my $i=0; $i <= $#def; $i++) {
		    if( $def[$i] =~ /^(.*):xmax=([0-9]+):ymax=([0-9]+):period=([0-9]+)$/ ) {
			my ($s,$x,$y,$p)=($1,$2,$3,$4);
			push @snrs, $s;
			$xmax = $x if $x > $xmax;
			$yscale{$s} = $y;
			$ymax = $y if $y > $ymax;
			$pscale{$s} = $p;
			$pmax = $p if $p > $pmax;
		    }
		}
		$ymax += int($ymax/20);  # get +5% emptyness
		$xmax += int($xmax/20);  # get +5% emptyness
		# set frequency count to max of a sensor
		for my $s ( keys %yscale ) {
		    $yscale{$s} *= ($ymax / $yscale{$s});
		}
		my $day = strftime("%Y/%m/%d", localtime(time));
		if ( $period =~ /^[1-9][0-9]+/ ) {
		    $period = "   grafieken: $pmax perioden van $period dagen";
		} elsif ( $period =~ /(jaar|kalender|calendar)/i ) {
		    $period = " grafieken: $pmax kalender jaren"
		} else { $period = ""; }
        	print "
            	    set output \"$FILE.eps\"
            	    set title \"frequentie verdeling sensors, locatie $place, periode $strt_date t/m $end_date\" font \"Helvetica, 14\"
		    set key font \"Helvetica, 12\"
            	    set term postscript eps rounded font \"Helvetica\" lw 3 solid
		    set style line 99 lc rgb \"#606060\" lt 1
		    set border 3 front ls 99
		    unset ytics
		    set tics nomirror
		    set termoption enhanced
		    xmax = $xmax
		    ymax = $ymax
		    scl(a) = a/ymax*100
            	    set xtics 0, 100/10, 100 font \"Helvetica, 10\"
            	    set xrange [ 0:xmax ]
            	    set xlabel \"{/Symbol m}g / m^3  $period\" font \"Helvetica, 12\"
            	    set label \"(GNUplot d.d. $day)\"  at xmax,-9 font \"Helvetica, 9\" tc rgb \"#606060\" right
	    	    set ytics format \"\" font \"Helvetica, 10\"
            	    set yrange [0:105]
            	    set ylabel \"frequentie van ${valid}waarden\" font \"Helvetica, 12\"
		    set multiplot;
        	";
		for( my $i=0; $i <= $#norm; $i++ ){
	    	    # set norm limits, and use color sensor
		    if ( $norm[$i] =~ /^(.*)=([0-9]+)_?(.*)$/ ) {
			my ($n,$v,$s) = ($1,$2,$3); $n =~ s/EU2/EU 2/;
			my $si = 0;	# si has color via index in palette
			for( $si = 0; $si <= $#snrs; $si++) {
			    last if $snrs[$si] eq $s;
			}

	    	        printf("set arrow from $v,0 to $v,%3.1f nohead lc rgb \"#%s\" lt 3\n",	
	    	          105-105*0.035*($#norm+2), hsv2rgb(palt($si),1, 1));
			# set top tics in color
	        	printf("set arrow from $v,104.5 to $v,105 nohead lc rgb \"#%s\" lt 3\n", hsv2rgb(palt($si),1, 1));
			$s =~ s/pm_/PM_/; $s =~ s/PM_25/PM_2.5/;
                        $s =~ s/_/ /g; $s =~ s/PM ([0-9\.]+)/PM\@_\{$1\}/;
			printf("set label \"$n $s\" at $v,%3.1f font \"Helvetica, 12\" textcolor rgb \"#%s\" center\n",
			  (105-105*0.035*($#norm+2))+105*0.035*($i+1), hsv2rgb(palt($si),0.7, 1));
		    }
		}
		my $frst = 1;
        	for ( my $i = 0; $i <= $#snrs; $i++ ) {
		    for ( my $p = 1; $p <= $pscale{$snrs[$i]}; $p++ ) {
	                if( $frst ) { print "plot "; $frst = 0; }
			else { print ",\\\n     "; }
			$_ = $snrs[$i]; s/pm_/PM_/; s/PM_25/PM_2.5/;
		        s/_/ /g; s/PM ([0-9\.]+)/PM\@_\{$1\}/;

                        printf("\"-\" using 2:(scl(\$3)) title \"%s%s    \" smooth csplines lc rgb \"#%s\" lw %2.1f",
			    $period ? "periode $p  " : "", $_,
			    hsv2rgb(palt($i),1.0-(($p-1)/($pmax*1.7)), 1.0),
			    2.55 - 0.75*($p-1)/$pmax);
		    }
        	}
		print "\n";
		next;
	    }
	    next if not $defs;
	    my @ln = split /\s+/;
	    next if $#ln != 3;
	    $sens = $ln[0] if not $sens;
	    $per = $ln[3] if not $per;
	    if ( $sens ne $ln[0]  || $per != $ln[3] ) {
		$sens = $ln[0]; $per = $ln[3];
	        print "e\n";
	    }
            printf ("%s\n", $_);
        }
        print "e\n";
    ' | \
    gnuplot
	
    if [ $? != 0 ]
    then
	echo "$ERROR on creation of frequency files on $OUTPUT" 1>&2
	return 1
    else
        echo "Sensors frequency graph in $OUTPUT.{eps|png}" 1>&2
    fi
    if depends exiftool "perl Image Magic"
    then
	local TMP=$(mktemp --suffix=.Check.png)
	convert -density 576 -resize 25% ${OUTPUT}.eps ${TMP}
	convert $TMP -shave 4x4 -trim +repage -bordercolor white -border 4x4 ${OUTPUT}.png
	rm -f $TMP
        local COMMENT="Grafiek met $VALID sensor waarde frequenties"
	COMMENT+=" periode van ${PERIOD} dagen/jaar, GNUplot dd $(date +%Y/%m/%d)."
	COMMENT+=" Operationele periode ${STRT_DATE} t/m ${END_DATE}."
	if [ -n "$LOCATION" ] ; then COMMENT+=" Lokatie $LOCATION, station ${NR:-06}." ; fi
	exiftool -q -m -overwrite_original_in_place \
    		-Title="frequenties ${VALID}sensor waarden, lokatie $LOCATION" \
		-Comment="$COMMENT" \
		-Author="$AUTHOR" \
		-Copyright="Creative Commons" \
		-Keywords="fijnstof, luchtkwaliteit" \
		-Description="Regionale luchtkwaliteit en luchtverontreiniging metingen. Frequentie van meetwaarden over een periode." \
		-CreationTime="$(date)" \
		${OUTPUT}.png
    fi
}

################################## end of graph routines ###############

function Rebuild_Dayly() {
    local TTBL=$1
    if [ -z "$TTBL" ] ; then DB_pass ; TTBL=$(Get_info nr ${NR:-06} table) ; fi
    Build_dayly ${TTBL}
    if [ -n "$NR" ] && [ "$NO_CAL" = 0 -a "${NR/#0/}" -lt 100 ]
    then	# only for non RIVM values
        Build_dayly ${TTBL} normalized
    fi
}

function try_update_db() {
    local DIFF TTBL=$1
    if [ -z "$TTBL" ] ; then TTBL=$TBL ; fi
    TTBL=${TTBL/_dayly*/}
    DB_pass
    if ! $MYSQL -e "SHOW TABLES;" \
        | grep -q "${TTBL}_dayly\$"
    then
	echo "$ATTENT Average databases do not exists yet. Build them.." 1>&2
	Rebuild_Dayly $TTBL
	return 0
    fi
    # table should have the correct PM correction factors if not rebuild them
    DIFF=`$MYSQL -e "SHOW CREATE TABLE ${TTBL}_dayly" \
        | sed -e 's/.* PM factors:.//' -e 's/[)(]//g' -e "s/'//g"`
    if [ "$DIFF" != "${CAL[pm_10]}:${CAL[pm_25]}" ]
    then
	echo "$ATTENT Rebuild average databases due to previous build with other correction factors." 1>&2
	Rebuild_Dayly $TTBL
	return 0
    fi
    # see if the tables are not too old: max 7 days
    DIFF=`$MYSQL -e "
              SELECT (
                (SELECT to_days(datum) FROM $TTBL ORDER BY datum DESC limit 1) -
	        (SELECT to_days(datum) FROM ${TTBL}_dayly ORDER BY datum DESC limit 1)
              ) AS diff
	"`
    if [ -z "$DIFF" ] ; then return 1; fi
    if [ $DIFF -ge 7 ]
    then	# rebuild te ${TTBL}_dayly database after 7 days last one build
	echo "$ATTENT Rebuild average databases due to older as 7 days" 1>&2
	Rebuild_Dayly $TTBL
    fi
    return 0
}

# report limit crossings for PM2.5 and PM10 limits
# argument is jaar or number of days, usually 365
function report() {
    set_std_limits	# make sure table of std limits is defined
    local PERIOD=$1
    local NRM=$2
    if [ -z "$TBL" ]
    then
	DB_pass ; TBL=$(Get_info nr ${NR:-06} table)
    fi
    check_dflt_sensors ${TBL}
    try_update_db	# check database for needed updates
    if [ "$NRM" = all ] ; then NRM="EU2005 EU2010 WHO"; fi
    echo "Luchtkwaliteitsmetingen voor ${TBL} voor periode $PERIOD"

    local IND EXT
    for IND in $NRM
    do
        if echo "$IND" | grep -q "EU20[01][05]" ; then EXT="?";  else EXT="" ; fi
	echo
	printf "*%.0s" {1..66}
	echo -en "\n*** normering volgens $IND\t"
	printf "*%.0s" {1..34}
        # question mark for those which limit is not well defined
	if ! echo "${HAVE_added_sensors[$TBL]}" | grep -q pm_25
	then
	  echo
	  printf "=%.0s" {1..66}
	  echo -en "\n======\tPM2.5\tvolgens norm $IND\t"
	  printf "=%.0s" {1..26}
	  echo
          avg_per_period $PERIOD pm_25 ${norm[$IND,year,pm_25]} ${IND}${EXT}
          period_day_limits $PERIOD pm_25 ${norm[$IND,rood,pm_25]} avg ${IND}${EXT} ${norm[$IND,days,pm_25]}
          period_day_limits CORRECTED $PERIOD pm_25 ${norm[$IND,rood,pm_25]} avg ${IND}${EXT} ${norm[$IND,days,pm_25]}
	fi
	if ! echo "${HAVE_added_sensors[$TBL]}" | grep -q pm_10
	then
	  echo
	  printf "=%.0s" {1..66}
	  echo -en "\n======\tPM10\tvolgens norm $IND\t"
	  printf "=%.0s" {1..26}
	  echo
          avg_per_period $PERIOD pm_10 ${norm[$IND,year,pm_10]} $IND
          period_day_limits $PERIOD pm_10 ${norm[$IND,rood,pm_10]} avg $IND ${norm[$IND,days,pm_10]}
          period_day_limits CORRECTED $PERIOD pm_10 ${norm[$IND,rood,pm_10]} avg $IND ${norm[$IND,days,pm_10]}
	fi
    done

}

# report smog warnings, days per period (year)
# arguments: [HOURS] [LEVEL1 ... LEVELn] [PERIOD=N] [NR1 ... NRn]
# HOURS minimal # hours of > limit per day (dflt 0)
# different levels per country, dflt BE UK NL
# PERIOD of days, dflt calendar years
# NR station number(s) or all
function report_smog() {
    local TBL SENS LEVEL NAME
    local HOURS=0		# must be more as N hours per day
    local LL="70 75 200"	# BE level, UK level, NL level
    local PERIOD=jaar TYPE=jaar
    if echo "$1" | grep -q '^[0-9][0-9]*$'
    then
	HOURS=$1 ; shift
    fi
    if echo "$1" | grep -q '^[0-9][0-9]*'
    then
	LL="$1" ; shift
    fi
    if echo "${1^^}" | grep -q 'PER.*='
    then
	PERIOD=$(echo "$1" | sed 's/.*=//') ; shift
	TYPE=periode
    fi
    if echo "$PERIOD" | grep -q "[0-9]"
    then
       TYPE=periode
       local QRY1="datediff(now(),datum) DIV $PERIOD"
       local QRY2="periode $PERIOD dagen"
    else
       TYPE=jaar
       local QRY1="year(datum)"
       local QRY2="$TYPE"
    fi
    DB_pass
    local TBLs=""
    if [ "${1^^}" = ALL ]
    then
	TBLs=$($MYSQL -e "SHOW TABLES" \
            | grep -v -e "_" -e stations)
    elif [ -z "$1" ]
    then
	TBLs=$(Get_info nr ${NR:-06} table)
    elif echo "$1" | grep -q NR
    then
	shift
	local T="$*" ; if [ -z "$T" ] ; then T=${NR:-06} ; fi
	local T ; for T in $T ; do TBLs+=" $(Get_info nr $T table)" ; done
    else
	return
    fi
    for TBL in $TBLs
    do
	NAME=$(Get_info table $TBL name)
	echo "Smog rapportage voor $NAME, "
	echo "aantal dagen per $QRY2 met meer dan $HOURS uren/dag van overschrijding:"
	for SENS in pm_10
	do
	    for LEVEL in $LL
	    do
		echo "
		    select '${QRY2}', '${SENS} >${LEVEL}';
	            select $QRY1 as '$QRY2', month(datum) as maand, day(datum) as dag, count(*) as '${SENS} >${LEVEL}'
	                from ${TBL} where ${SENS} >= ${LEVEL}
			group by ${QRY1}, maand, dag
		    ;
		"
	    done
	done \
	| $MYSQL \
	| perl -e "my \$hours=$HOURS;" -e '
	    # filter needed to count days from # hours > limit
	    my $yr = "x"; my $cnt = 0;
	    while( <STDIN> ) { chomp;
		next if /^$/;
		my @line = split /\s+/;
		if( /[a-z]/i ) {	# header
		    if( $yr ne "x" ) { print "$yr\t$cnt\n" if $cnt; }
		    $yr = "x"; $cnt = 0; s/_//; s/pm25/pm2.5/i; $_ = uc($_) if /[0-9]/;
		    print "$_\n"; next;
		}
		if ( $yr ne $line[0] ) {
		    if( $yr ne "x" ) { print "$yr\t$cnt\n" if $cnt; }
		    $cnt = 0; $yr = $line[0];
		}
		$cnt++ if $line[3] > $hours; # hours above level
	    }
	    if( $yr ne "x" ) { print "$yr\t$cnt\n" if $cnt; }
	' \
| tee @out2 \
	| perl -e '
	    my %lines = (); my $max = -1;
	    while(<STDIN>) { chomp;
	        my @line = split /\t/;
	        for( my $i = 0; $i <= $#line; $i++ ){
		    my $j;
		    $line[0]++ if length($line[0]) < 2;
		    if( not $lines{$line[0]} ) {
	                $lines{$line[0]}[0] = $line[0];
			for( $j = 1; $j < $max; $j++ ) {
			    $lines{$line[0]}[$j] = " ";
			}
		    } else { $j = $#{$lines{$line[0]}} + 1; }
		    next if $i == 0;
	            $lines{$line[0]}[$j] =  $line[$i];
		    if( $max < $#{$lines{$line[0]}} ) {
		#	foreach my $k (keys %lines) {
		#	    for( $j = $#{$lines{$k}} +1; $j <= $max; $j++){
		#		$lines{$k}[$j] = " " if not defined $lines{$k}[$j];
		#	    }
		#	}
			$max = $#{$lines{$line[0]}};
		    }
	        }
	    }
	    my @size = ();
	    for my $row ( sort { $a <=> $b } keys %lines ) {
		if ( $#size < 0 ) {
		    for( my $i = 0; $i <= $#{$lines{$row}}; $i++ ) {
			$size[$i] = length($lines{$row}[$i])+1;
		    }
		}
		for( my $i = 0; $i <= $#{$lines{$row}}; $i++ ) {
		    printf("%$size[$i].$size[$i]s ", $lines{$row}[$i]);
		}
		print "\n";
	    }
	    print "\n";
	'
    done
} 

# get the ordered list of sensors in the RRD database file
function RRD_sensors() {
    local TBL=$1
    local FILE=${RRD_DIR}/${RRD}.rrd
    if [ ! -f $FILE ]
    then
	echo "ERROR: $FILE does not exists!" 1>&2
	return 1
    fi
    rrdtool info $FILE | perl -e '
        my @index = ();
        while( <STDIN> ) {
           chop; s/\s+$//;
           if( not /^ds/ ) { next; }
           if( /^ds\[([^\]]+)\]\.index\s+=\s+([0-9]+)/ ) {
	        $index[$2] = $1; next;
           }
        }
        for ( my $i = 0; $i <= $#index; $i++ ) {
           printf("%s%s", ($i > 0? ",": ""),$index[$i]);
        }
        printf("\n");
    '
    return $?
}

# recreate the RRD database file for a MySQL database table
function RRD_recreate() {
    local TBL=$1
    if [ ! -d ${RRD_DIR} ]
    then
        echo "ERROR: cannot find RRD DB directory ${RRD_DIR}." 1>&2
        exit 1
    fi
    local FILE=${RRD_DIR}/${RRD}.rrd
    declare -i FIRST=$($MYSQL -e "SELECT UNIX_TIMESTAMP(datum) FROM $TBL LIMIT 1")
    FIRST=$(($FIRST - 1))
    if [ ! -f $FILE -o "$FIRST" -le 0 ]
    then
        if [ -z "$DBUSER"] ; then DBUSER=$(GET_passwd USER) ; fi
        if [ -z "$DBPASS"] ; then DBPASS=$(GET_passwd) ; fi
	$GET_DATA --user $DBUSER --pass $DBPASS --location "${NR:-06}" -R
	if [ ! -f $FILE ]
	then
	    echo "ERROR: $FILE does not exists or no start date in MySQL $DB database!" 1>&2
	   return 1
	fi
	return 0
    fi
    local CMD
    CMD=$(rrdtool info $FILE | \
	perl -e "my \$first=$FIRST ; my \$file='$FILE'; my \$W=$WIDTH; " \
	     -e '
    my @index = ();
    my %DS;
    my @RRA = ();
    my @RRAs = ();
    my $CUR = "none";
    my $filename = "none";
    my $steps = 0;
    while( <STDIN> ) {
       chop; s/\s+$//;
       if( /^filename\s+=\s+"([^"]+)"/ ) { $filename = $1; next; }
       if( /^step\s+=\s+([0-9]+)$/ ) { $steps = $1; next; }
       if( not /^(ds|rra)/ ) { next; }
       if( /^ds\[([^\]]+)\]\.index\s+=\s+([0-9]+)/ ) {
	    $index[$2] = $1; next;
       }
       s/"//g;
       if( /^ds\[([^\]]+)\]\.(type|minimal_heartbeat|min|max)\s+=\s+([A-Z0-9].*)$/ ) {
	    $DS{$1}{$2} = $3; next;
       }
       if( /^ds/ ) { next; }
       if( /^rra\[([0-9]+)\]\.(cf|rows|pdp_per_row|xff)\s+=\s+(.*)$/ ) {
            my $ind = $1; my $type = $2; my $val = $3;
            if ( $type =~ /cf/ ) {
	        my $rcnt; for( $rcnt = 0; $rcnt <= $#RRA; $rcnt++ ) { 
		    last if ($RRA[$rcnt] =~ /$val/ );
	        }
	        $RRA[$rcnt] = $val;
	        $CUR = $val; next;
	    }
	    $RRAs[int($ind/($#RRA+1))]{$CUR}{$type} = $val; next;
       }
    }
    my @CMD = ("--start", "$first", "--step", "$steps");
    for ( my $i = 0; $i <= $#index; $i++ ) {
       # DS:so2:GAUGE:7200:0:60
       # printf("%s%s", ($i > 0? ",": ""),$index[$i]);
       push @CMD, sprintf( "DS:%s:%s:%d:%d:%d",$index[$i],$DS{$index[$i]}{type},$DS{$index[$i]}{minimal_heartbeat},int($DS{$index[$i]}{min}),int($DS{$index[$i]}{max}));
       # RRA:AVERAGE:0.2:24:8760
       #foreach $key (@RRA) {
       #    push @CMD, sprintf("RRA:%s:%.1f:%d:%d", $key, $RRAs[$i]{$key}{xff},$RRAs[$i]{$key}{pdp_per_row},$RRAs[$i]{$key}{rows}) if $RRAs[$i]{$key}{rows};
       #}
    }
    for my $R_type ("AVERAGE", "MIN", "MAX") { # add rra directives
        # one day, one week, one month, one year, 3 years
        for my $steps ( 24, 24*7, 24*30, 24*365, 24*365*3, 24*365*6 ) {
            push @CMD, sprintf(" RRA:%s:0.5:%d:%d",$R_type,int(($steps+$W)/$W),$W);
        }
    }
    # printf("\n");
    #system("rm","-f", "$file");
    #system(@CMD);
    printf("%s\n",join(" ",@CMD));
    ')
    rm -f $FILE
    rrdtool create $FILE $CMD
    return $?
}

# update RRD database file from MySQL table 
# from last in RRD database up to last in database
function RRD_update() {
    local TBL=$1
    if [ ! -d ${RRD_DIR} ]
    then
        echo "Cannot find RRD directory ${RRD_DIR}." 1>&2
        exit 1
    fi
    local FILE=${RRD_DIR}/${RRD}.rrd
    declare -i LAST_RRD=$(rrdtool lastupdate $FILE | tail -1 | sed -e 's/^[^0-9]*//' -e 's/:.*//')
    if [ $? -ne 0 ]
    then
	echo "ERROR: RRD database does it exists for $TBL?" 1>&2
	return 1
    fi
    local SENSORS=$(RRD_sensors $TBL)
    if [ $? -ne 0 ]
    then
	echo "ERROR: No sensors found, cannot rebuild RRD database for $TBL." 1>&2
	return 1
    fi
    local VALIDS=$(perl -e "my \$A='$SENSORS';" -e '$A =~ s/,/_valid OR /g; $A =~ s/$/_valid/; print $A;')
    if ! $MYSQL -e "
            SELECT UNIX_TIMESTAMP(datum),$SENSORS FROM $TBL 
	        WHERE UNIX_TIMESTAMP(datum) > $LAST_RRD AND
		($VALIDS) ORDER BY datum" \
	| perl -e "my \$file='$FILE'; " -e '
		while (<STDIN>) {
		    chop;
		    s/NULL/U/g; s/\s+/:/g ; s/:$//;
		    printf("update %s %s\n",$file,$_);
		}
	' \
        | rrdtool -  \
        | perl -e "my \$file='$FILE';" -e '
        		    $cnt = 0; $errs = 0;
        		    while(<STDIN>) {
			 	if ( /OK u:/ ) { $cnt++; }
				else { $errs++; print {"STDERR"} "$_"; } }
        		    printf STDERR ("INFO: RRDupdate $file: RRD records added: %d\n",$cnt);
        		    printf STDERR ("WARNING: RRDupdate $file: RRD records with error: %d\n",$errs) if $errs;
	'
    then
	echo "ERROR: Failed to update $FILE" 1>&2
        return 1
    fi
    return 0
}

# update DB table stations and print current sensors status.
# sensors with no measuremnts in last month are within parentheses.
function Update_stations() {
    local TBLS=($@)
    if [ -z "$1" ]
    then
        TBLS=($($MYSQL -e "SHOW TABLES" | grep -v -e _ -e stations) )
    fi
    local TBL
    local -i NOW=$(date +%s) ; NOW=$(($NOW - 30*24*60*60))      # one month
    for TBL in ${TBLS[@]}
    do
        local CUR=() LOST=()
        local POLS=($($MYSQL -e "DESCRIBE $TBL" | awk '{ print $1; }' | \
            grep -E -v -e id -e datum -e '_ppb' -e '_color' -e '_valid)' | \
            sed -e 's/luchtdruk/z_luchtdruk/' -e 's/temp/z_temp/' \
                -e 's/rv/z_rv/' -e 's/wr/z_wr/' -e 's/ws/z_ws/' | \
            sort | sed 's/z_//g') )
        local POL
        for POL in ${POLS[@]}
        do
            if (( $($MYSQL -e "SELECT count(datum) FROM $TBL WHERE UNIX_TIMESTAMP(datum) >= $NOW and not ISNULL($POL)") > 0 ))
            then
                CUR+=($POL)
            else
                LOST+=($POL)
            fi
        done
        local STRG=$($MYSQL -e "SELECT CONCAT(name,', ',municipality,', ',province) FROM stations WHERE stations.table = '$TBL' LIMIT 1")
        echo -e "\nLocation $STRG, DB table $TBL:"
        if (( ${#CUR[@]} > 0 ))
        then
            echo -e "\tcurrent pollutants: ${CUR[@]}"
        fi
        if (( ${#LOST[@]} > 0 ))
        then
            echo -e "\tnot active pollutants: ${LOST[@]}"
        fi
        if (( (${#CUR[@]} + ${#LOST[@]}) < 1 ))
        then
            echo -e "\tno measurements found in database"
        else
            STRG=''
            for POL in ${CUR[@]}
            do
                if [ -n "$STRG" ] ; then STRG+=',' ; fi
                STRG+=$POL
            done
            for POL in ${LOST[@]}
            do
                if [ -n "$STRG" ] ; then STRG+=',' ; fi
                STRG+="($POL)"
            done
            $MYSQL -e "UPDATE stations SET sense = '$STRG' WHERE stations.table = '$TBL'"
        fi
    done
}

# RRD_recreate NL10131
# RRD_update NL10131

# get number of days operational
# argument: end date (default now)
function Days_operational() {
    local END=$1
    if [ -z "$END" ]; then END=now; fi
    if echo "$END" | grep -q '^now$'
    then END='now()'
    elif echo "$END" | grep -q '20[0-9][0-9].[01]*[0-9].[0-3]*[0-9]$'
    then END="'$END'"
    elif echo "$END" | grep -q '^[0-9][0-9]*$'
    then echo "$END" ; return 0
    else END='now()'
    fi
    DB_pass
    if ! $MYSQL -e "
            SELECT (to_days($END) - (SELECT to_days(datum) FROM $TBL LIMIT 1) - 1)"
    then
	echo "$WARNING Cannot find operations start date." 1>&2
	echo 365
    fi
    return $?
}

# set all global used variables as nr station, name station, DB table station
function Set_Location_Globals() {
    DB_pass
    if [ -z "$NR" ]			# station/location identifier
    then		# if not defined try otherwise
	if [ -n "$TBL" ] ; then NR=$(Get_info table "$TBL" nr)
	elif [ -n "$LOCATION" ] ; then NR=$(Get_info name "$LOCATION" nr)
	fi
    fi
    NR=${NR:-06}        		# default number of location prov. Limburg
    if [ -z "$LOCATION" ] ; then LOCATION=$(Get_info nr "$NR" name) ; fi
    if [ -z "$ORGANISATION" ] ; then ORGANISATION=$(Get_info nr "$NR" organisation) ; fi
    if [ -z "$ID" ] ; then ID=$(Get_info nr "$NR" id) ; fi
    if [ -z "$TBL" ] ; then TBL=$(Get_info nr "$NR" table) ; fi
    TBL=${TBL:-HadM}			# core database table
    if [ -z "$N_StdDev" ] ; then N_StdDev=$(Get_info nr "$NR" a_stddev) ; fi
    N_StdDev=${N_StdDev:-6}		# take all measurements valid
    if [ -z "$N_DayStdDev" ] ; then N_DayStdDev=$(Get_info nr "$NR" day_stddev) ; fi
    N_DayStdDev=${N_DayStdDev:-6}	# take all measurements valid
    if [ "$WIDTH" -le 500 ]
    then
        LOCATION=`echo "$LOCATION" | sed -e 's/\([A-Za-z]\)[a-z0-9][a-z0-9]*/\1/g' -e 's/ //g'`
    fi
    RRD=./luchtkwaliteit-${TBL}		# set RRD databases for graphs
    if [ -n "$CSV_REPORT" ]		# optional base name for spreadsheet report
    then		# delete old spreadsheet reports
        rm -f ${CSV_REPORT}-*${TBL}.csv
    fi
}

############################### end of globals and functions

trap 'if [ -n "$pid" ] ; then kill  $pid; fi ; exit' INT TERM
trap 'if [ -n "$pid" ] ; then kill  $pid; fi ; rm -f /tmp/tmp.*.Check.* /var/tmp/rrdtool_cmd; exit' EXIT

if [ -z "$1" ]
then
    $0 help
    exit 0
fi
if [ -n "$CSV_REPORT" ]
then
    rm -f ${CSV_REPORT}-periodes_*$TBL.csv
fi
if [ -n "$DBFORCED" ] ; then create_stations rebuild ; fi

while true
do
    if [ -z "$1" ]; then break; fi
    case "$1" in
    help|-h)			# HELP information, user manual
	echo "
De meetgegevens komen van prov. meetstation $LOCATION en zijn verzameld van
de website $SITE (first) en de website $SITE2 (second).

Deze script behandeld een aantal zaken.
De diverse functies zijn aan te roepen via argumenten bij de aanroep van de script.
Per functies zijn een aantal opties mogelijk.
Per aanroep kunnen er meerdere functies (met de opties) tegelijk gegeven worden.
Bijv. $0 check overzicht short

Met de PM correction factor kan met oude gegevens toekomstige overschrijdingen de
limieten voor diverse situaties ge-evalueerd worden.

De script neem enkele definities over (of vult ze aan met default waarden
voor Horst aan de Maas) voor de volgende (shell omgevings) variabelen:
SITE		website tot half aug 2014	default: $SITE
SITE2		website na mei 2014	default: $SITE2
DBFORCED	Indien gedefinieerd, (her)bouw tabel stations in $DB database.
		Stations tabel opbouw uit locations.pl info bestand.
LOCATION	locatie/plaats		default: $LOCATION
TBL		meetwaarden tabelnaam in de database $DB default: $TBL
NR		locatie nummer meetstation default: ${NR:-06}
AQI		Gebruik AQI index als kleur gradatie en niveaux ipv EU norm
Deze definities mogen ook als argument megegeven worden.
Voorbeeld: TBL=$TBL $0 overzicht	Overzicht voor meetwaarden $TBL
	   $0 NR=06 report

	help		Deze boodschap. (default)
	PM{10|25}=F,[-]A
			Zet PM10 of PM25 calibratie factors. Dlt 1,0 (geen)
			Hierbij is F de vermenigvulgingsfactor (>0),
			A is het aantal µg/m³ erbij te tellen (bijv. -2.5).
			PM calibraties maken modelberekeningen mogelijk.
	VALID=[N]	Minimaal aantal metingen om daggemiddelde valide
			te verklaren. N: null, 0-24, dflt: $VALID_CNT
			null: meting om 00 uur maakt gemiddelde valide.
	DEV=N[%]	Definieer Std Deviatie factor voor volledige
			operationele periode voor station nr ${NR:-06}.
			Bij N% wordt waarschijnlijkheid getal omgezet in N.
			Bijv. 90.5% definieert 1.67 X sigma, 1.97 -> 95%.
	DAYDEV=N[%]	Definieer nieuwe Std Dev factor voor dag uur metingen
			validatie proces voor station nr ${NR:-06}.
	check 		Overzicht van de dagen met minder als 24 metingen.

	Enkele DB tabel en RRD database vernieuwings functies:
	rebuild [dayly|RRD [TYPE]|RRD-dayly|stations]
		Default dayly:
			Clear en (re)build de database tabel voor dagelijkse
			gemiddelden (min,max,avg,stddev,aantal per dag).
	        RRD [TYPE]: Clear en Rebuild de RRD database. Dit duurt even.
			TYPE: type RRD database, raw (default), both (beide),
			norm (genormalizeerde waarden volgens NEN).
			norm (re)builds alleen normalized RR database.
			both (re)builds both (raw and norm) RRD databases.
		RRD-dayly: Clear en Rebuild de RRD database van daggemiddelden.
			Deze functie niet gebruiken!
			Bevat nog create/definition fouten (rrdtool core dump).
		stations: Drop stations tabel in $DB database en rebuild
			dit tabel vanuit de $LOCATIONS script.
			Dflt: bouw automatisch de tabel als tabel niet bestaat.
		valids:
			Forceeer herberekening van validiteit van waarden
			in een table (default automatisch).

        update [TABLE]  Update DB table stations with current sensors.
                        Default: all stations with measurements in the database.
	printinfo [NR]	Lokatie details van nr NR (default: alle lokaties).
	overzicht [NORM] [SHORT] [PERIOD={jaar|N}]
			Verkort overzicht van alle vergaarde (ruwe) data.
			Vermelding van aantal incomplete waarden, en
			aangetroffen kleurenschemas (kleur en bandbreedte).
			Geeft datum en waarde van daggem. overschrijding (NORM).
			optie NORM: geen dagelijkse overschrijdings details.
			NORM: EU2005, EU2010, WHO, site, site2 (dft geen melding)
			SHORT: geen details of enkele details (dflt)
			PERIOD=jaar per kalenderjaar (dft) of N periode van dagen
	kleurschema     Een overzicht van pm ranges en gebruikte kleurschemas.
			De website 'first' (tot mei 2014) hanteert een ander
			kleuren schema als 'second' (vanaf april 2014).
			De meetwaarden van beiden zijn hetzelfde.
        verwijder [column_name]
                        Verwijdert permanent de kolum met naam eindigend
                        op column_name van de database tabel.
                        Default: namen eindigend op _color.
	REPORT=[name]	Definieerd basis naam voor csv/html tabel voor
			functie report. dfl: geen naam, geen tabel.
	report [PERIOD|PERIOD={jaar,N}] [NORM] [DETAIL|LONG|NORMAL]
			Overview en volledig rapportage overzicht lokatie
			gegevens, jaar of kalender en
			PERIOD|PERIOD={jaar,N}: jaar, laatste aantal dagen
			(default 365).
			NORM: all, site, site2, EU2005, EU2010, WHO (default: all).
			\"all\" zijn alle formele normeringen.
			DETAIL: normal of long (toon alle dag overschrijdingen),
			default: short.
			report maakt, als CSV_REPORT naam is gedefinieert,
			een csv bestand en een HTML tabel bestand aan met een
			report overzicht in een tabel vorm.
	kalender	Een overzicht per kalender jaar van norm overschrijdingen.
			Komt overeen met commando \"report jaar all\".
	PERIOD		Een overzicht van normoverschrijdingen
			per periode van laatste PERIOD dagen.
			Komt overeen met commando \"report PERIOD all\".
	smog [PERIOD] [COUNT=N] ['NR i ... k' | all] [LEVEL='L1 ... Ln'
			Rapporteer aantal dagen per jaar dat smog limieten
			overschreden worden.
			PERIOD N bijv N=365, dflt per kalenjaar. 
			Bijv. 'NR 04 06 131' voor 04 etc. of
			all voor alle stations in DB
			dflt: alleen station ${NR:-06}
			COUNT=N minimum uren met meting >= nivo (dflt 0)
			LEVEL='L1 L2 ..' smog PM 10 levels.
		        Default nivo's: 70 (BE) 75 (UK) 200 (NL).

Voor het navolgende (diagrammen/grafieken) maakt de volgorde van de parameters
niet uit:

	graph [TYPE] [PERIOD] [NORM]
			Maak een nieuwe RRD diagrammen
			(klimaat en fijnstof) aan: uurwaarden en daggemiddelden.
			PERIOD: getal|start, laatste aantal dagen (default 365)
			        start vanaf begin metingen
			NORM: site, site2, EU2005, EU2010, WHO (default $DFLT_NORM)
			Maakt ook klimaat, luchtverontreiniging en daggemiddelden defaults aan.
	klimaat [PERIOD] [END_DATE] Maak een nieuw diagram voor
			klimaat metingen (default PERIOD 365 dagen).
			END_DATE Eind datum van periode (zie formaat UNIX date)
			Default END_DATE: now. Default NORM: EU2005.
	pollutant [TYPE] [PERIOD] [NORM] [END_DATE] [pollutant namen]
			Maak een nieuw diagram voor
			luchtverontreiniging metingen (default PERIOD 365 dagen).
			END_DATE Eind datum van periode (zie formaat UNIX date)
			Default END_DATE: now. Default NORM: EU2005.
			TYPE: type RRD database, resp raw (default), both (beide),
			norm (genormalizeerde waarden volgens NEN).
			pollutant namen: default: pm_10 pm_25 roet
			fijnstof ipv pollutant werkt ook.
	sensors [TYPE] [PERIOD] [END_DATE] SENSOR ...
			Where SENSOR is {pm_10,pm_25,roet,no2,co2,no}
			Grafiek van gemiddelden, minima, maxima in periode.
	histogram [PERIOD] [SENDSORS] [VALID] [[START] END]
			Maak een grafiek van freqenties van sensor waarden per
			periode. Default alle sensors, periode: hele operationele
			periode. PERIOD=jaar|N (aantal dagen, bijv 365).
			SENSORS=[sensor, ...] Bijv pm_10 pm_25 (dflt alle sensors). 
			VALID: alleen gevalideerde waarden (dflt: geen validatie).
			START/END datums: van (dflt: begin) datum t/m eind datum
			(dflt: now).
	daggemiddelde [TYPE] [PERIOD] [NORM] [END_DATE] [PM]
			Maak een nieuw diagram voor
			luchtverontreiniging daggemiddelden metingen (default PERIOD 365 dagen).
			END_DATE Eind datum van periode (zie formaat UNIX date)
			Default END_DATE: now. Default NORM: $DFLT_NORM.
			PM: PM10|PM2.5|twee default: twee
		Voor datum opgave mogelijkheden zie UNIX \"date\" manual:
		bijv ook: now, yesterday, \"last week\", \"month ago\", etc.
	cleanup [all|TABLE]
		Zal een table restanten van intermediate waarden verwijderen.
		default table HadM, all: alle beschikbare tabellen in database.
		f_{sensor} colummen worden verwijderd uit de tabel.
	" 1>&2
    ;;
					#  change PM calibration factors
    PM25=[0-9]*,*|{pP][mM]10=[0-9]*,*)
	case "$1" in
	[Pp][Mm]10*)
	    CAL[pm_10]=${1/*=/}
	    Set_calibration pm_10
	;;
	[Pp][mM]25*)
	    CAL[pm_25]=${1/*=/}
	    Set_calibration pm_25
	;;
	esac
    ;;
    TBL=*|NR=*)			# define via station number report
    				# define via table name the station report
	if echo "${1/*=/}" | grep -q '[A-Za-z]'
	then
	    TBL=${1/*=/} ; NR=06
	else
	    NR=${1/*=/}
	fi
	Set_Location_Globals
    ;;
    LOCATION=*)				# define via name of station
	LOCATION=${1/*=/}
	Set_Location_Globals
    ;;
    SITE=*)
	SITE=${1/*=/}
    ;;
    VALID=*)				# change dflt allowed cnt of missing days
	VALID_CNT=`echo "$1" | sed -e 's/.*=//' -e 's/[^0-9]//g' -e '/0./s/0//'`
	# max is 24 hourly measurements
	if [ -n "$VALID_CNT" -a "$VALID_CNT" -ge 24 ] ; then VALID_CNT=23; fi
    ;;
    DEV=*)
	N_StdDev=$(echo "$1" | sed -e 's/.*=//' -e 's/[^0-9\.%]//g')
	if echo "$N_StdDev" | grep -q '%'
        then N_StdDev=$(Propability $N_StdDev) ; fi
	Set_Location_Globals
	Set_StdDev ${TBL} N_StdDev=$N_StdDev
    ;;
    DAYDEV=*)
	N_DayStdDev=$(echo "$1" | sed -e 's/.*=//' -e 's/[^0-9\.%]//g')
	if echo "$N_DayStdDev" | grep -q '%'
        then N_DayStdDev=$(Propability $N_DayStdDev) ; fi
	Set_Location_Globals
	Set_StdDev ${TBL} N_DayStdDev=$N_DayStdDev
    ;;
    REPORT=*)				# define general name for spreadsheet report
	CSV_REPORT=`echo "$1" | sed 's/.*=//'`
    ;;
    AQI|AQI=*)
	AQIon=1	# we use AQI index and grading levels in graphs
    ;;
    overview|overzicht)			# short report for a station
	NORM="$DFLT_NORM" LOOP=1 SHORT="" PERIOD=jaar
	if [ -z "$2" ]; then LOOP=0; fi
        while [ $LOOP = 1 ]
        do
	    case "${2^^}" in
	    EU2005|EU2010|SITE|SITE2|WHO|ALL*)
	        NORM=$2; shift		# default: $DFLT_NORM
	    ;;
	    LONG)
		SHORT=long; shift	# default normal
	    ;;
	    SHORT)
		SHORT=short; shift
	    ;;
	    PERIOD=[0-9][0-9]*)
		PERIOD=$(echo "$2" | sed 's/.*=//') ; shift
	    ;;
	    PERIOD=JAAR)
		PERIOD=jaar ; shift	# default
	    ;;
	    *)
	        LOOP=0
	    ;;
	    esac
        done
        Set_Location_Globals
        if [ "$SHORT" = long ]  # make sure to re-evaluate measurements
        then
            set_last_validated ${TBL} "" "2000/1/1"
        fi
	report-days $NORM $SHORT PERIOD="$PERIOD"	# give overview of data collected: short, verbosity level
    ;;
    check)				# check table on missing days and hours
        if [ -z "$TBL" ]
        then
            DB_pass ; TBL=$(Get_info nr ${NR:-06} table)
        fi
	Set_Location_Globals
	has-not-24 1 $TBL	# give overview of days with less as 24 hourly measurements
        Missing-Days $TBL       # give details about days with fully missing sensor values
    ;;
    verwijder|delete)                   # delete a column of columns with ext
       if [ -z "$TBL" ]
        then
            DB_pass ; TBL=$(Get_info nr ${NR:-06} table)
        fi
        Set_Location_Globals
        Del_Column "$TBL" ${2:-_color}
        exit $?
    ;;
    kleurschema|color*)			# report colors used for values of a station
        if [ -z "$TBL" ]
        then
            DB_pass ; TBL=$(Get_info nr ${NR:-06} table)
        fi
	Set_Location_Globals
        report-sensor-stats $TBL _color # give overview of PM value ranges and used color schemes
    ;;
    convert)				# convert a spreadsheet file to HTML/PNG
	if [ -n "$2" ]
	then
	    if [ -f "$2" ]
	    then
		Convert_CSV_HTML "$2"
		shift
	    fi
	fi
	;;
    printinfo)				# show information of a station
	N=""	# default is all available locations in Lmbrg
	if echo "B$2" | grep -q "^B[0-9][0-9]*$"
	then
	    N=$2; shift
	fi
	Set_Location_Globals
	Print_info $N
    ;;
    report)				# detailed report of a station
        CNT=365         # default 365 days
        N=all   # default is for all standards
        LOOP=1
	S=SHORT
        while [ "$LOOP" = 1 -a -n "$2" ]
        do
            LOOP=0
            if echo "$2" | grep -q "^[0-9][0-9]*\$"
            then CNT=$2; shift; LOOP=1
            fi
            if echo "$2" | grep -q "^kalender\$"
            then CNT=jaar; shift; LOOP=1
            fi
            if echo "$2" | grep -q "^jaar\$"
            then CNT=$2; shift; LOOP=1
            fi
            case "${2^^}" in
            SITE2|SITE|EU2005|EU2010|WHO|ALL*)
                N=$2; shift; LOOP=1		# default all
            ;;
	    # next are for overvioew report part: days() -> report-day-stats()
	    SHORT)
		S=short ; shift ; LOOP=1	# default
	    ;;
	    NORMAL)
		S="" ; shift ; LOOP=1
	    ;;
	    DETAIL|LONG)
		S=long; shift; LOOP=1
	    ;;
	    PERIOD=*)				# default period of 365 days
		CNT=$(echo "$2" | sed 's/.*=//') ; shift; LOOP=1
	    ;;
            esac
        done
	Set_Location_Globals
	echo "$SHORT RAPPORTAGE voor STATION nr ${NR:-06}, PLAATS $LOCATION, DB tabel $TBL, norm $N, periode lengte $CNT"
	echo -e "=============================================================\n"
	# give location details
	Print_info ${NR:-06}
        # argument has verbosity
        report-days $N $S PERIOD="$CNT"         # give overview of data collected
	# smog report
	report_smog 0 "70 75 200" PERIOD=$CNT NR ${NR:-06}
        # per calendar year and period of 365 days
        report $CNT $N
	if [ -n "$CSV_REPORT" ]
	then
	    Convert_CSV_HTML "${CSV_REPORT}-periodes_${CNT}-${TBL}.csv"
	fi
    ;;
    kalender|jaar|year)		# report per calendar year (all norms)
	Set_Location_Globals
	Print_info ${NR:-06}
        report jaar all
    ;;
    [0-9][0-9]*)		# report for a station number (all norms)
        P=`echo "ARG" | sed 's/[^0-9]//g'`
	Set_Location_Globals
	Print_info ${NR:-06}
        report "$P" all
    ;;
    rebuild)			# rebuild some tables or values in a table
	N=$2; if [ -z "$N" ] ; then N=dayly ; fi
	if [ -n "$N" ]
	then
	    case "${N^^}" in
	    VALID*)		# force validation of values for a TBL
		Set_Location_Globals
		set_last_validated ${TBL} "" "2000/1/1"
                echo "The table database validation will take about 4-7 minutes." 1>&2
		Set_Sensor_valids $TBL
		Build_dayly $TBL
		shift
	    ;;
	    STATION*)		# force $LOCATIONS script into DB table
		shift
		create_stations rebuild
	    ;;
            RRD-DAYLY|RRDDAYLY)
		shift
	        echo "$1 functie wordt nog niet ondersteund." 1>&2
	        shift ; continue
		Set_Location_Globals
	        start_progressing "Rebuilding the RRD dayly average database "
	        Build_dayly_RRD ${TBL}
	        stop_progressing
            ;;
            RRD)		# force RRD DB files update
		shift
		Set_Location_Globals
	        FILES=${RRD_DIR}/${RRD}.rrd
	        N_R="" ; TYPE=""
	        if [ -n "$2" ]
	        then
	            case "$2" in
	            norm*)
		        N_R="-n" ; FILES=${RRD_DIR}/${RRD}_normalized.rrd
		        TYPE=norm; shift
	            ;;
	            both|beide*)
		        N_R="-n -n"
		        FILES="$FILES ${RRD_DIR}/${RRD}_normalized.rrd"
		        TYPE=both; shift
	            ;;
	            raw)
		        shift
	            ;;
	            esac
	        fi
                DB_pass
	        # if [ -f ${RRD_DIR}/${RRD}.rrd ]
		# then
	        #     echo "RRD database rebuild by $MY_NAME (not via $GET_DATA)"
                #     echo "The RRD database rebuild will take less then 1 minute." 1>&2
                #     start_progressing "Rebuilding the RRD database "
	        #     Set_Sensor_valids ${TBL} no_report
		#     RRD_recreate $TBL
		#     RRD_update $TBL
		#     if [ "$TYPE" = "both" ]
		#     then
	 	#       RRD_recreate ${TBL}_norm
	 	#       RRD_update ${TBL}_norm
	 	#     fi
                #     stop_progressing
		# else
                    FIRST=`$MYSQL -e "
                            SELECT date_format(datum,'%Y/%c/%e')
                              FROM $TBL LIMIT 1;"`
                    LAST=`$MYSQL -e "
                            SELECT date_format(datum,'%Y/%c/%e')
			      FROM $TBL ORDER BY datum DESC LIMIT 1; "`
                    if [ -d ${RRD_DIR} ]
                    then
                        echo "The RRD database rebuild via $GET_DATA will take about several minutes." 1>&2
	                Set_Sensor_valids ${TBL} no_report
                        rm -f $FILES	# remove old files else $GET_DATA will complain
                        if [ -z $DBUSER  ] ; then DBUSER=$(GET_passwd USER) ; fi
                        if [ -z "$DBPASS" ] ; then DBPASS=$(GET_passwd) ; fi
                        start_progressing "Rebuilding the RRD database "
                        $GET_DATA -u $DBUSER -p$DBPASS -l "${NR:-06}" -a $N_R -S "$FIRST" "$LAST"
                        stop_progressing
                    else
                        echo "ERROR: cannot find RRD dir ${RRD_DIR}" 1>&2
                    fi
		# fi
                # and create a new default RRD diagrams
                # klimaat 365 now
                # pollutant_graph $TYPE 365 EU2005 now
                # pollutant_graph $TYPE 365 EU2010 now
                # pollutant_graph $TYPE 365 site now
                # pollutant_graph $TYPE 365 site2 now
                # fijnstof_avg $TYPE 365 twee EU2005 now
                # fijnstof_avg $TYPE 365 twee EU2010 now
            ;;
            *)
		if [ -n "$2" -a "$2" = dayly ] ; then shift ; fi
		Set_Location_Globals
	        # clear and renew (raw) average database table
	        Rebuild_Dayly $(Get_info nr ${NR:-06} table)
            ;;
	    esac
	fi
    ;;
    update)
        # update sensors in DB table stations
        DB_pass
        Update_stations $2
        if [ -n "$2" ] ; then shift ; fi
    ;;
    diagram|grafiek|graph)	# recreate diagrams for climate and pollution
        D=now
        CNT=365
	N=$DFLT_NORM
	TYPE=""
	LOOP=1
	while [ "$LOOP" = 1 -a -n "$2" ]
	do
	    LOOP=0
            # check if date command except it as a date
            date --date="$2" >/dev/null 2>/dev/null
            if [ "$?" = 0 ]
            then
                D="$2" ; shift ; LOOP=1 ; continue	# default today
            fi
	    case "$2" in
	    start|[0-9][0-9]*)
		CNT=$2; shift; LOOP=1 ; continue	# default period of 365 days
	    ;;
	    site2|site|EU2005|EU2010|WHO)
		N=$2; shift; LOOP=1 ; continue		# default $DFLT_NORM
	    ;;
	    both|beide*|raw|norm)
		TYPE=$2; shift; LOOP=1; continue
	    ;;
	    esac
        done
	Set_Location_Globals
	if [ "$CNT" = start ] ; then CNT=`Days_operational "$D"` ; fi
	klimaat $CNT "$D"
	pollutant_graph $TYPE $CNT $N "$D" # e.g. pollutant_graph 365 site2
        #fijnstof_avg $CNT PM25 $N $D
        #fijnstof_avg $CNT PM10 $N $D
        fijnstof_avg $TYPE $CNT twee $N $D
    ;;
    klimaat|climat)		# recreate diagrams for climate
	CNT=365
	D=now
	LOOP=1
	while [ "$LOOP" = 1 -a -n "$2" ]
	do
	    LOOP=0
	    # check if date command except it as a date
	    date --date="$2" >/dev/null 2>/dev/null
	    if [ "$?" = 0 ]
	    then
		D="$2" ; shift; LOOP=1 ; continue	# default up to today
	    fi
	    if echo "$2" | grep -q -e "^[0-9][0-9]*\$" -e '^start$'
	    then CNT=$2; shift; LOOP=1 ; continue	# default 365 days
	    fi
	done
	Set_Location_Globals
	if [ "$CNT" = start ] ; then CNT=`Days_operational "$D"` ; fi
	klimaat $CNT "$D"
    ;;
    fijnstof|luchtvervuiler|pollutant)	# recreate air pollution diagrams for a period
	CNT=365
	D=now
	N=$DFLT_NORM
	LOOP=1
	TYPE=""
	while [ "$LOOP" = 1 -a -n "$2" ]
	do
	    LOOP=0
            # check if date command except it as a date
            date --date="$2" >/dev/null 2>/dev/null
            if [ "$?" = 0 ]
            then
                D="$2" ; shift; LOOP=1 ; continue	# default up to today
            fi
	    case "$2" in
	    start|[0-9][0-9]*)
		CNT=$2; shift; LOOP=1 ; continue	# default 365 days
	    ;;
	    site2|site|EU2005|EU2010|WHO)
		N=$2; shift; LOOP=1 ; continue		# default $DFLT_NORM
	    ;;
	    both|beide*|raw|norm)
		TYPE=$2; shift; LOOP=1; continue
	    ;;
	    esac
        done
	Set_Location_Globals
	if [ "$CNT" = start ] ; then CNT=`Days_operational "$D"` ; fi
	pollutant_graph $TYPE $CNT $N "$D" $* # args may define polltants (DB names)
	exit
    ;;
    senso*)                   # recreate air pollution diagrams for a period
        CNT=365
	N=""
        D=now
        LOOP=1
        TYPE=""
        while [ "$LOOP" = 1 -a -n "$2" ]
        do
            LOOP=0
            # check if date command except it as a date
            date --date="$2" >/dev/null 2>/dev/null
            if [ "$?" = 0 ]
            then
                D="$2" ; shift; LOOP=1 ; continue       # default up to today
            fi
            case "$2" in
	    pm_10|pm_25|roet|no2|o3|no|zwarte_rook)
		N+="$2 "; shift; LOOP=1 ; continue
	    ;;
            start|[0-9][0-9]*)
                CNT=$2; shift; LOOP=1 ; continue        # default 365 days
            ;;
            both|beide*|raw|norm)
                TYPE=$2; shift; LOOP=1; continue
            ;;
            esac
        done
	if [ -z "$N" ] ; then N=pm_25 ; fi		# default PM2.5
        Set_Location_Globals
        if [ "$CNT" = start ] ; then CNT=`Days_operational "$D"` ; fi
        sensor_graph $TYPE "$N" $CNT "$D"
    ;;
    daggem*|gemid*|average)		# recreate diagrams for dayly average
	CNT=365
        D=now
        N=$DFLT_NORM
	PM=both
        LOOP=1
	TYPE=""
        while [ "$LOOP" = 1 -a -n "$2" ]
        do
            LOOP=0
            # check if date command except it as a date
            date --date="$2" >/dev/null 2>/dev/null
            if [ "$?" = 0 ]
            then
                D="$2" ; shift; LOOP=1 ; continue	# default up to today
            fi
            case "$2" in
	    start|[0-9][0-9]*)
		CNT=$2; shift; LOOP=1 ; continue	# default 365 days
	    ;;
            site2|site|EU2005|EU2010|WHO)
                N=$2; shift; LOOP=1 ; continue		# default EU2010
            ;;
	    both|twee)
		PM=both; shift; LOOP=1; continue
	    ;;
	    [Pp][Mm]25|[Pp][Mm]2.5|pm_25)
		PM=pm_25; shift; LOOP=1 ; continue	# default both
	    ;;
	    [Pp][Mm]10|pm_10)
		PM=pm_10; shift; LOOP=1 ; continue
	    ;;
	    beide*|raw|norm)
		TYPE=$2; shift; LOOP=1; continue
	    ;;
	    esac
        done
	Set_Location_Globals
	# eg fijnstof_avg 180 PM25 EU2005
	# eg fijnstof_avg 365 PM25 EU2005 now
	# eg fijnstof_avg 365 PM10 EU2010 2013/8/24
	if [ "$CNT" = start ] ; then CNT=`Days_operational "$D"` ; fi
	fijnstof_avg $TYPE $CNT $PM $N $D
    ;;
    smog)		# report smog details per period of time for a region
	LOOP=1
	P="" ; T="" ; C=0 ; L="70 75 200"
	while [ "$LOOP" = 1 -a -n "$2" ]
        do
            LOOP=0
	    case "${2^^}" in
	    PERIOD=*)		# default period is per calendar year
		P="$2" ; LOOP=1; shift; continue
	    ;;
	    COUNT=*)		# default minimu nr hours exceeding 0
		C=$(echo "$2" | sed 's/COUNT=//') ; LOOP=1; shift; continue
	    ;;
	    ALL|NR*)		# default for default NR location only
		T="$2" ; LOOP=1; shift ; continue
	    ;;
	    LEVEL=*)		# default levels of BE, UK, and NL
		L="$(echo "$2" | sed 's/.*=//')" ; shift ; LOOP=1 ; continue
	    ;;
	    esac
	done
	Set_Location_Globals
	report_smog "$C" "$L" ${P} ${T}
    ;;
    histogram)		# create frequency histogram of sensor values
	LOOP=1
	P="" ; S="" ; V="" ; DS="" ; DE=""
	while [ "$LOOP" = 1 -a -n "$2" ]
        do
	    date --date="$2" >/dev/null 2>/dev/null
	    if [ $? = 0 ]
	    then
		if [ -n "$DE" ] ; then DS="START=${DE/END=/}"; fi
		DE="END=$(date --date="$2" "+%Y/%m/%d")"
	        shift ; continue
	    fi
            case "${2^^}" in
            PERIOD*=*)          # default period is full operational period
                P="$2" ; shift; continue
            ;;
	    SENSO*=*)		# sensor names, dflt all available sensors
		S="$2" ; shift; continue
	    ;;
	    VALID)
		V=VALID ; shift ; continue
	    ;;
	    *)
		LOOP=0 ; break
	    ;;
	    esac
	done
	Set_Location_Globals
	histogram "${TBL}" $P $S $V $DS $DE
    ;;
    cleanup)
        LOOP=
	if [ -n "$2" ]; then LOOP=$2 ; shift ; fi
	Cleanup_table $LOOP
    ;;
    *)
	echo "$ERROR Unknown function or file: $1" 1>&2
	exit 1
    esac
    shift
done
